// Copyright(C) 1999-2023 National Technology & Engineering Solutions
// of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
// NTESS, the U.S. Government retains certain rights in this software.
//
// See packages/seacas/LICENSE for details

#include "Utst_structured_decomp.h"
#define DOCTEST_CONFIG_NO_SHORT_MACRO_NAMES
#define DOCTEST_CONFIG_SUPER_FAST_ASSERTS
#define DOCTEST_CONFIG_NO_MULTITHREADING
#include <doctest.h>

// Disable these tests on NVCC. It tries to optimize and takes forever to build...
#if !defined(__NVCC__) && !defined(__IOSS_WINDOWS__)
#if defined(__clang__) || (defined(__GNUC__) && !defined(__INTEL_COMPILER))
#define ATTRIBUTE_NO_SANITIZE_ADDRESS __attribute__((no_sanitize_address))
#else
#define ATTRIBUTE_NO_SANITIZE_ADDRESS
#endif

ATTRIBUTE_NO_SANITIZE_ADDRESS
DOCTEST_TEST_CASE("Rocket-1496")
{
  int                                       zone = 1;
  std::vector<Iocgns::StructuredZoneData *> zones;

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1", zones.back()->m_zone, "zone720", 720, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2", zones.back()->m_zone, "zone842", 842, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4", zones.back()->m_zone, "zone241", 241, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A5", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A6", zones.back()->m_zone, "zone845", 845, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B39", zones.back()->m_zone, "zone1275", 1275,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B44", zones.back()->m_zone, "zone1386", 1386, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A46", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A47", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A48", zones.back()->m_zone, "zone985", 985, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A49", zones.back()->m_zone, "zone640", 640, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B213", zones.back()->m_zone, "zone920", 920,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B392", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B401", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A402", zones.back()->m_zone, "zone733", 733, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A403", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A404", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2133", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2957", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2962", zones.back()->m_zone, "zone221", 221,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2963", zones.back()->m_zone, "zone184", 184,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2964", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2965", zones.back()->m_zone, "zone5", 5, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2127", zones.back()->m_zone, "zone1171", 1171,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2961", zones.back()->m_zone, "zone1496", 1496,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2965", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2966", zones.back()->m_zone, "zone220", 220,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2967", zones.back()->m_zone, "zone7", 7, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2130", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2922", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2927", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2964", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2968", zones.back()->m_zone, "zone185", 185,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{51, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2969", zones.back()->m_zone, "zone7", 7, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2126", zones.back()->m_zone, "zone1171", 1171,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2926", zones.back()->m_zone, "zone1477", 1477,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2931", zones.back()->m_zone, "zone1479", 1479,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2967", zones.back()->m_zone, "zone5", 5, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2969", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2915", zones.back()->m_zone, "zone1472", 1472,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2917", zones.back()->m_zone, "zone1473", 1473,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2970", zones.back()->m_zone, "zone9", 9, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2971", zones.back()->m_zone, "zone553", 553, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2972", zones.back()->m_zone, "zone75", 75,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2119", zones.back()->m_zone, "zone1169", 1169,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2180", zones.back()->m_zone, "zone1185", 1185,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2970", zones.back()->m_zone, "zone8", 8, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2973", zones.back()->m_zone, "zone199", 199, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2974", zones.back()->m_zone, "zone10", 10,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2122", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2177", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2974", zones.back()->m_zone, "zone9", 9, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2975", zones.back()->m_zone, "zone78", 78, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2976", zones.back()->m_zone, "zone190", 190,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2977", zones.back()->m_zone, "zone203", 203, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2434", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2978", zones.back()->m_zone, "zone183", 183, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2979", zones.back()->m_zone, "zone15", 15, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2980", zones.back()->m_zone, "zone12", 12, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2981", zones.back()->m_zone, "zone21", 21, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2368", zones.back()->m_zone, "zone1247", 1247, Ioss::IJK_t{{-2, 3, -1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{12, 101, 1}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2980", zones.back()->m_zone, "zone11", 11, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2982", zones.back()->m_zone, "zone13", 13, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2983", zones.back()->m_zone, "zone20", 20,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2984", zones.back()->m_zone, "zone30", 30, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2410", zones.back()->m_zone, "zone1263", 1263,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2982", zones.back()->m_zone, "zone12", 12, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2985", zones.back()->m_zone, "zone15", 15, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2986", zones.back()->m_zone, "zone30", 30, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2987", zones.back()->m_zone, "zone16", 16, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B47", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B397", zones.back()->m_zone, "zone1475", 1475,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B399", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A405", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A406", zones.back()->m_zone, "zone940", 940, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A407", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2262", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2979", zones.back()->m_zone, "zone11", 11, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2985", zones.back()->m_zone, "zone13", 13, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2988", zones.back()->m_zone, "zone182", 182,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2989", zones.back()->m_zone, "zone17", 17, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2409", zones.back()->m_zone, "zone1262", 1262,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2987", zones.back()->m_zone, "zone13", 13, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2990", zones.back()->m_zone, "zone17", 17, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2991", zones.back()->m_zone, "zone19", 19, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2992", zones.back()->m_zone, "zone29", 29, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2431", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2989", zones.back()->m_zone, "zone15", 15, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2990", zones.back()->m_zone, "zone16", 16, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2993", zones.back()->m_zone, "zone181", 181,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2994", zones.back()->m_zone, "zone18", 18, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2421", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2994", zones.back()->m_zone, "zone17", 17, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2995", zones.back()->m_zone, "zone167", 167, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2996", zones.back()->m_zone, "zone23", 23, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2997", zones.back()->m_zone, "zone19", 19, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2412", zones.back()->m_zone, "zone1265", 1265,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2991", zones.back()->m_zone, "zone16", 16, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2997", zones.back()->m_zone, "zone18", 18, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2998", zones.back()->m_zone, "zone22", 22, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2999", zones.back()->m_zone, "zone28", 28,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2370", zones.back()->m_zone, "zone1248", 1248, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2983", zones.back()->m_zone, "zone12", 12,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3000", zones.back()->m_zone, "zone57", 57, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3001", zones.back()->m_zone, "zone21", 21, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3002", zones.back()->m_zone, "zone31", 31, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2355", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2981", zones.back()->m_zone, "zone11", 11, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3001", zones.back()->m_zone, "zone20", 20, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3003", zones.back()->m_zone, "zone66", 66,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3004", zones.back()->m_zone, "zone65", 65, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2400", zones.back()->m_zone, "zone1259", 1259,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2998", zones.back()->m_zone, "zone19", 19, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3005", zones.back()->m_zone, "zone23", 23, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3006", zones.back()->m_zone, "zone26", 26, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3007", zones.back()->m_zone, "zone27", 27, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2426", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2996", zones.back()->m_zone, "zone18", 18, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3005", zones.back()->m_zone, "zone22", 22, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3008", zones.back()->m_zone, "zone166", 166, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3009", zones.back()->m_zone, "zone24", 24, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2545", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3009", zones.back()->m_zone, "zone23", 23, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3010", zones.back()->m_zone, "zone165", 165,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3011", zones.back()->m_zone, "zone26", 26, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3012", zones.back()->m_zone, "zone37", 37, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B306", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B394", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B404", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B407", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A408", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A409", zones.back()->m_zone, "zone941", 941, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2392", zones.back()->m_zone, "zone1256", 1256,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3006", zones.back()->m_zone, "zone22", 22, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3011", zones.back()->m_zone, "zone24", 24, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3013", zones.back()->m_zone, "zone35", 35,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3014", zones.back()->m_zone, "zone27", 27, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2397", zones.back()->m_zone, "zone1258", 1258,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3007", zones.back()->m_zone, "zone22", 22, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3014", zones.back()->m_zone, "zone26", 26, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3015", zones.back()->m_zone, "zone34", 34,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3016", zones.back()->m_zone, "zone28", 28, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2403", zones.back()->m_zone, "zone1260", 1260,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2999", zones.back()->m_zone, "zone19", 19, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3016", zones.back()->m_zone, "zone27", 27, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3017", zones.back()->m_zone, "zone29", 29, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3018", zones.back()->m_zone, "zone33", 33,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2405", zones.back()->m_zone, "zone1261", 1261,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2992", zones.back()->m_zone, "zone16", 16, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3017", zones.back()->m_zone, "zone28", 28, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3019", zones.back()->m_zone, "zone32", 32,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3020", zones.back()->m_zone, "zone30", 30, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2374", zones.back()->m_zone, "zone1249", 1249,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2984", zones.back()->m_zone, "zone12", 12, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2986", zones.back()->m_zone, "zone13", 13, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3020", zones.back()->m_zone, "zone29", 29, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3021", zones.back()->m_zone, "zone31", 31,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2376", zones.back()->m_zone, "zone1250", 1250,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3002", zones.back()->m_zone, "zone20", 20, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3021", zones.back()->m_zone, "zone30", 30,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3022", zones.back()->m_zone, "zone57", 57, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3023", zones.back()->m_zone, "zone32", 32, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2379", zones.back()->m_zone, "zone1251", 1251,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3019", zones.back()->m_zone, "zone29", 29,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3023", zones.back()->m_zone, "zone31", 31, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3024", zones.back()->m_zone, "zone56", 56, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3025", zones.back()->m_zone, "zone33", 33, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2383", zones.back()->m_zone, "zone1252", 1252,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3018", zones.back()->m_zone, "zone28", 28,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3025", zones.back()->m_zone, "zone32", 32, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3026", zones.back()->m_zone, "zone55", 55, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3027", zones.back()->m_zone, "zone34", 34, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2387", zones.back()->m_zone, "zone1255", 1255,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3015", zones.back()->m_zone, "zone27", 27,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3027", zones.back()->m_zone, "zone33", 33, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3028", zones.back()->m_zone, "zone38", 38, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3029", zones.back()->m_zone, "zone35", 35, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2394", zones.back()->m_zone, "zone1257", 1257, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3013", zones.back()->m_zone, "zone26", 26,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3029", zones.back()->m_zone, "zone34", 34, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3030", zones.back()->m_zone, "zone38", 38, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3031", zones.back()->m_zone, "zone37", 37, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B51", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B398", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A410", zones.back()->m_zone, "zone447", 447, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A411", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A412", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A413", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2559", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3012", zones.back()->m_zone, "zone24", 24, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3031", zones.back()->m_zone, "zone35", 35, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3032", zones.back()->m_zone, "zone40", 40,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3033", zones.back()->m_zone, "zone39", 39,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2320", zones.back()->m_zone, "zone1228", 1228,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3028", zones.back()->m_zone, "zone34", 34, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3030", zones.back()->m_zone, "zone35", 35, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3034", zones.back()->m_zone, "zone39", 39, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3035", zones.back()->m_zone, "zone55", 55, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2324", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3033", zones.back()->m_zone, "zone37", 37, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3034", zones.back()->m_zone, "zone38", 38, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3036", zones.back()->m_zone, "zone41", 41,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3037", zones.back()->m_zone, "zone63", 63, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1832", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3032", zones.back()->m_zone, "zone37", 37,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3038", zones.back()->m_zone, "zone41", 41, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3039", zones.back()->m_zone, "zone89", 89, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3040", zones.back()->m_zone, "zone43", 43, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1834", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3036", zones.back()->m_zone, "zone39", 39,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3038", zones.back()->m_zone, "zone40", 40, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3041", zones.back()->m_zone, "zone59", 59, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3042", zones.back()->m_zone, "zone42", 42, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1829", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3042", zones.back()->m_zone, "zone41", 41, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3043", zones.back()->m_zone, "zone43", 43, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3044", zones.back()->m_zone, "zone44", 44, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3045", zones.back()->m_zone, "zone474", 474, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1825", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3040", zones.back()->m_zone, "zone40", 40, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3043", zones.back()->m_zone, "zone42", 42, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3046", zones.back()->m_zone, "zone46", 46, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3047", zones.back()->m_zone, "zone90", 90,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2765", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3044", zones.back()->m_zone, "zone42", 42, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3048", zones.back()->m_zone, "zone46", 46, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3049", zones.back()->m_zone, "zone45", 45, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3050", zones.back()->m_zone, "zone467", 467, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2753", zones.back()->m_zone, "zone1411", 1411,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3049", zones.back()->m_zone, "zone44", 44, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3051", zones.back()->m_zone, "zone466", 466,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3052", zones.back()->m_zone, "zone54", 54,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3053", zones.back()->m_zone, "zone48", 48, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2754", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3046", zones.back()->m_zone, "zone43", 43, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3048", zones.back()->m_zone, "zone44", 44, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3054", zones.back()->m_zone, "zone48", 48, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3055", zones.back()->m_zone, "zone49", 49,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B198", zones.back()->m_zone, "zone887", 887,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B403", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B413", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A414", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A415", zones.back()->m_zone, "zone469", 469, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A416", zones.back()->m_zone, "zone102", 102, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2749", zones.back()->m_zone, "zone1410", 1410, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3053", zones.back()->m_zone, "zone45", 45, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3054", zones.back()->m_zone, "zone46", 46, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3056", zones.back()->m_zone, "zone50", 50,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3057", zones.back()->m_zone, "zone54", 54, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2818", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3055", zones.back()->m_zone, "zone46", 46,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3058", zones.back()->m_zone, "zone144", 144, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3059", zones.back()->m_zone, "zone51", 51, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3060", zones.back()->m_zone, "zone50", 50, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2745", zones.back()->m_zone, "zone1408", 1408, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3056", zones.back()->m_zone, "zone48", 48,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3060", zones.back()->m_zone, "zone49", 49, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3061", zones.back()->m_zone, "zone52", 52, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3062", zones.back()->m_zone, "zone53", 53, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2809", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3059", zones.back()->m_zone, "zone49", 49, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3063", zones.back()->m_zone, "zone145", 145, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3064", zones.back()->m_zone, "zone445", 445,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3065", zones.back()->m_zone, "zone52", 52, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2776", zones.back()->m_zone, "zone1418", 1418,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3061", zones.back()->m_zone, "zone50", 50, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3065", zones.back()->m_zone, "zone51", 51,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3066", zones.back()->m_zone, "zone53", 53, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3067", zones.back()->m_zone, "zone456", 456,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2773", zones.back()->m_zone, "zone1417", 1417,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3062", zones.back()->m_zone, "zone50", 50, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3066", zones.back()->m_zone, "zone52", 52, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3068", zones.back()->m_zone, "zone54", 54,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3069", zones.back()->m_zone, "zone460", 460, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2770", zones.back()->m_zone, "zone1416", 1416,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3052", zones.back()->m_zone, "zone45", 45, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3057", zones.back()->m_zone, "zone48", 48, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3068", zones.back()->m_zone, "zone53", 53,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3070", zones.back()->m_zone, "zone465", 465, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2384", zones.back()->m_zone, "zone1254", 1254,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3026", zones.back()->m_zone, "zone33", 33, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3035", zones.back()->m_zone, "zone38", 38, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3071", zones.back()->m_zone, "zone56", 56, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3072", zones.back()->m_zone, "zone63", 63,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2336", zones.back()->m_zone, "zone1234", 1234,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3024", zones.back()->m_zone, "zone32", 32, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3071", zones.back()->m_zone, "zone55", 55, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3073", zones.back()->m_zone, "zone64", 64,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3074", zones.back()->m_zone, "zone57", 57, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2340", zones.back()->m_zone, "zone1235", 1235,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3000", zones.back()->m_zone, "zone20", 20, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3022", zones.back()->m_zone, "zone31", 31,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3074", zones.back()->m_zone, "zone56", 56, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3075", zones.back()->m_zone, "zone65", 65, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B293", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B408", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B414", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A417", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A418", zones.back()->m_zone, "zone983", 983, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A419", zones.back()->m_zone, "zone125", 125,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1839", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3041", zones.back()->m_zone, "zone41", 41, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3076", zones.back()->m_zone, "zone63", 63, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3077", zones.back()->m_zone, "zone60", 60, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3078", zones.back()->m_zone, "zone485", 485,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1843", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3077", zones.back()->m_zone, "zone59", 59, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3079", zones.back()->m_zone, "zone61", 61, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3080", zones.back()->m_zone, "zone487", 487,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3081", zones.back()->m_zone, "zone64", 64, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1851", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3079", zones.back()->m_zone, "zone60", 60, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3082", zones.back()->m_zone, "zone66", 66, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3083", zones.back()->m_zone, "zone65", 65, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3084", zones.back()->m_zone, "zone62", 62, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1859", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3084", zones.back()->m_zone, "zone61", 61, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3085", zones.back()->m_zone, "zone67", 67, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3086", zones.back()->m_zone, "zone68", 68, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3087", zones.back()->m_zone, "zone532", 532, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2330", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3037", zones.back()->m_zone, "zone39", 39, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3072", zones.back()->m_zone, "zone55", 55, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3076", zones.back()->m_zone, "zone59", 59, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3088", zones.back()->m_zone, "zone64", 64, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2348", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3073", zones.back()->m_zone, "zone56", 56, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3081", zones.back()->m_zone, "zone60", 60, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3088", zones.back()->m_zone, "zone63", 63, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3089", zones.back()->m_zone, "zone65", 65, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2363", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3004", zones.back()->m_zone, "zone21", 21, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3075", zones.back()->m_zone, "zone57", 57, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3083", zones.back()->m_zone, "zone61", 61, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3089", zones.back()->m_zone, "zone64", 64, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1847", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3003", zones.back()->m_zone, "zone21", 21,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3082", zones.back()->m_zone, "zone61", 61, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3090", zones.back()->m_zone, "zone107", 107,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3091", zones.back()->m_zone, "zone67", 67, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1856", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3085", zones.back()->m_zone, "zone62", 62, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3091", zones.back()->m_zone, "zone66", 66, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3092", zones.back()->m_zone, "zone70", 70, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3093", zones.back()->m_zone, "zone108", 108,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2901", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3086", zones.back()->m_zone, "zone62", 62, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3094", zones.back()->m_zone, "zone70", 70, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3095", zones.back()->m_zone, "zone71", 71, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3096", zones.back()->m_zone, "zone536", 536, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B56", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B405", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B412", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B417", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A420", zones.back()->m_zone, "zone136", 136,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A421", zones.back()->m_zone, "zone939", 939, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2886", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3092", zones.back()->m_zone, "zone67", 67, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3094", zones.back()->m_zone, "zone68", 68, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3097", zones.back()->m_zone, "zone72", 72, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3098", zones.back()->m_zone, "zone77", 77,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2897", zones.back()->m_zone, "zone1466", 1466,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3095", zones.back()->m_zone, "zone68", 68, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3099", zones.back()->m_zone, "zone72", 72, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3100", zones.back()->m_zone, "zone73", 73, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3101", zones.back()->m_zone, "zone538", 538,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2920", zones.back()->m_zone, "zone1474", 1474, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3097", zones.back()->m_zone, "zone70", 70, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3099", zones.back()->m_zone, "zone71", 71, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3102", zones.back()->m_zone, "zone76", 76,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3103", zones.back()->m_zone, "zone73", 73, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2912", zones.back()->m_zone, "zone1471", 1471,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3100", zones.back()->m_zone, "zone71", 71, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3103", zones.back()->m_zone, "zone72", 72, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3104", zones.back()->m_zone, "zone74", 74,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3105", zones.back()->m_zone, "zone542", 542, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2916", zones.back()->m_zone, "zone1472", 1472,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3104", zones.back()->m_zone, "zone73", 73,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3106", zones.back()->m_zone, "zone75", 75, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3107", zones.back()->m_zone, "zone76", 76, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3108", zones.back()->m_zone, "zone548", 548, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2972", zones.back()->m_zone, "zone8", 8, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3106", zones.back()->m_zone, "zone74", 74, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3109", zones.back()->m_zone, "zone76", 76, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3110", zones.back()->m_zone, "zone78", 78,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3111", zones.back()->m_zone, "zone549", 549,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2919", zones.back()->m_zone, "zone1473", 1473, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3102", zones.back()->m_zone, "zone72", 72,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3107", zones.back()->m_zone, "zone74", 74, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3109", zones.back()->m_zone, "zone75", 75, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3112", zones.back()->m_zone, "zone77", 77, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2176", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3098", zones.back()->m_zone, "zone70", 70,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3112", zones.back()->m_zone, "zone76", 76, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3113", zones.back()->m_zone, "zone193", 193,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3114", zones.back()->m_zone, "zone78", 78, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2975", zones.back()->m_zone, "zone10", 10, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3110", zones.back()->m_zone, "zone75", 75, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3114", zones.back()->m_zone, "zone77", 77, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3115", zones.back()->m_zone, "zone192", 192,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3116", zones.back()->m_zone, "zone198", 198,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1780", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3117", zones.back()->m_zone, "zone81", 81,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3118", zones.back()->m_zone, "zone160", 160,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3119", zones.back()->m_zone, "zone88", 88,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3120", zones.back()->m_zone, "zone86", 86, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B79", zones.back()->m_zone, "zone576", 576, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B84", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A422", zones.back()->m_zone, "zone147", 147,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A423", zones.back()->m_zone, "zone555", 555, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A424", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A425", zones.back()->m_zone, "zone114", 114, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1759", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3117", zones.back()->m_zone, "zone79", 79,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3121", zones.back()->m_zone, "zone82", 82, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3122", zones.back()->m_zone, "zone159", 159,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3123", zones.back()->m_zone, "zone143", 143,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2098", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3121", zones.back()->m_zone, "zone81", 81, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3124", zones.back()->m_zone, "zone83", 83, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3125", zones.back()->m_zone, "zone94", 94, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3126", zones.back()->m_zone, "zone85", 85, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2113", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3124", zones.back()->m_zone, "zone82", 82, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3127", zones.back()->m_zone, "zone92", 92, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3128", zones.back()->m_zone, "zone390", 390, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3129", zones.back()->m_zone, "zone84", 84, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1742", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3129", zones.back()->m_zone, "zone83", 83, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3130", zones.back()->m_zone, "zone85", 85,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3131", zones.back()->m_zone, "zone151", 151,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3132", zones.back()->m_zone, "zone420", 420, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1756", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3126", zones.back()->m_zone, "zone82", 82, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3130", zones.back()->m_zone, "zone84", 84,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3133", zones.back()->m_zone, "zone153", 153,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3134", zones.back()->m_zone, "zone86", 86, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1774", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3120", zones.back()->m_zone, "zone79", 79, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3134", zones.back()->m_zone, "zone85", 85, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3135", zones.back()->m_zone, "zone87", 87, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3136", zones.back()->m_zone, "zone149", 149,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1823", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3135", zones.back()->m_zone, "zone86", 86, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3137", zones.back()->m_zone, "zone88", 88, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3138", zones.back()->m_zone, "zone90", 90, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3139", zones.back()->m_zone, "zone148", 148,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1833", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3119", zones.back()->m_zone, "zone79", 79,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3137", zones.back()->m_zone, "zone87", 87, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3140", zones.back()->m_zone, "zone89", 89, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3141", zones.back()->m_zone, "zone164", 164,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3039", zones.back()->m_zone, "zone40", 40, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3140", zones.back()->m_zone, "zone88", 88, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3142", zones.back()->m_zone, "zone165", 165, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3143", zones.back()->m_zone, "zone90", 90, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3047", zones.back()->m_zone, "zone43", 43,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3138", zones.back()->m_zone, "zone87", 87, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3143", zones.back()->m_zone, "zone89", 89, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3144", zones.back()->m_zone, "zone144", 144, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B63", zones.back()->m_zone, "zone335", 335, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B411", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B424", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A426", zones.back()->m_zone, "zone136", 136, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A427", zones.back()->m_zone, "zone535", 535, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A428", zones.back()->m_zone, "zone102", 102, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2114", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3127", zones.back()->m_zone, "zone83", 83, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3145", zones.back()->m_zone, "zone93", 93, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3146", zones.back()->m_zone, "zone127", 127,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3147", zones.back()->m_zone, "zone388", 388,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2101", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3145", zones.back()->m_zone, "zone92", 92, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3148", zones.back()->m_zone, "zone94", 94, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3149", zones.back()->m_zone, "zone123", 123,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3150", zones.back()->m_zone, "zone139", 139,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2103", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3125", zones.back()->m_zone, "zone82", 82, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3148", zones.back()->m_zone, "zone93", 93, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3151", zones.back()->m_zone, "zone95", 95, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3152", zones.back()->m_zone, "zone143", 143, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2085", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3151", zones.back()->m_zone, "zone94", 94, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3153", zones.back()->m_zone, "zone96", 96, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3154", zones.back()->m_zone, "zone142", 142,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3155", zones.back()->m_zone, "zone139", 139,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2071", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3153", zones.back()->m_zone, "zone95", 95, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3156", zones.back()->m_zone, "zone141", 141,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3157", zones.back()->m_zone, "zone99", 99, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3158", zones.back()->m_zone, "zone97", 97, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2073", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3158", zones.back()->m_zone, "zone96", 96, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3159", zones.back()->m_zone, "zone139", 139,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3160", zones.back()->m_zone, "zone119", 119,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3161", zones.back()->m_zone, "zone98", 98, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2059", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3161", zones.back()->m_zone, "zone97", 97, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3162", zones.back()->m_zone, "zone99", 99,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3163", zones.back()->m_zone, "zone118", 118,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3164", zones.back()->m_zone, "zone112", 112, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2062", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3157", zones.back()->m_zone, "zone96", 96, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3162", zones.back()->m_zone, "zone98", 98,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3165", zones.back()->m_zone, "zone100", 100,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3166", zones.back()->m_zone, "zone140", 140, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2043", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3165", zones.back()->m_zone, "zone99", 99, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3167", zones.back()->m_zone, "zone111", 111,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3168", zones.back()->m_zone, "zone109", 109, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3169", zones.back()->m_zone, "zone101", 101,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1695", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3169", zones.back()->m_zone, "zone100", 100,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3170", zones.back()->m_zone, "zone140", 140,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3171", zones.back()->m_zone, "zone171", 171,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3172", zones.back()->m_zone, "zone103", 103,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B204", zones.back()->m_zone, "zone898", 898,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B416", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B428", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A429", zones.back()->m_zone, "zone125", 125,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A430", zones.back()->m_zone, "zone513", 513, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A431", zones.back()->m_zone, "zone158", 158, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1803", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3172", zones.back()->m_zone, "zone101", 101,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3173", zones.back()->m_zone, "zone104", 104,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3174", zones.back()->m_zone, "zone106", 106,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3175", zones.back()->m_zone, "zone176", 176,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1816", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3173", zones.back()->m_zone, "zone103", 103,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3176", zones.back()->m_zone, "zone105", 105,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3177", zones.back()->m_zone, "zone188", 188,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3178", zones.back()->m_zone, "zone109", 109,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 6, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1854", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3176", zones.back()->m_zone, "zone104", 104,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3179", zones.back()->m_zone, "zone189", 189,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3180", zones.back()->m_zone, "zone108", 108,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3181", zones.back()->m_zone, "zone106", 106,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1848", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3174", zones.back()->m_zone, "zone103", 103,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3181", zones.back()->m_zone, "zone105", 105,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3182", zones.back()->m_zone, "zone107", 107,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3183", zones.back()->m_zone, "zone177", 177,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3090", zones.back()->m_zone, "zone66", 66,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3182", zones.back()->m_zone, "zone106", 106,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3184", zones.back()->m_zone, "zone183", 183, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3185", zones.back()->m_zone, "zone108", 108,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3093", zones.back()->m_zone, "zone67", 67, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3180", zones.back()->m_zone, "zone105", 105,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3185", zones.back()->m_zone, "zone107", 107,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3186", zones.back()->m_zone, "zone193", 193, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1685", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3168", zones.back()->m_zone, "zone100", 100, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3178", zones.back()->m_zone, "zone104", 104,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3187", zones.back()->m_zone, "zone110", 110,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3188", zones.back()->m_zone, "zone185", 185,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1673", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3187", zones.back()->m_zone, "zone109", 109,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3189", zones.back()->m_zone, "zone111", 111, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3190", zones.back()->m_zone, "zone245", 245, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3191", zones.back()->m_zone, "zone184", 184,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2046", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3167", zones.back()->m_zone, "zone100", 100,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3189", zones.back()->m_zone, "zone110", 110, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3192", zones.back()->m_zone, "zone112", 112,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3193", zones.back()->m_zone, "zone256", 256, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2048", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3164", zones.back()->m_zone, "zone98", 98, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3192", zones.back()->m_zone, "zone111", 111,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3194", zones.back()->m_zone, "zone115", 115,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3195", zones.back()->m_zone, "zone259", 259,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B41", zones.back()->m_zone, "zone1386", 1386,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A50", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A51", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A52", zones.back()->m_zone, "zone335", 335, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A53", zones.back()->m_zone, "zone690", 690, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A54", zones.back()->m_zone, "zone692", 692, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B89", zones.back()->m_zone, "zone598", 598, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B93", zones.back()->m_zone, "zone610", 610, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B425", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A432", zones.back()->m_zone, "zone169", 169,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A433", zones.back()->m_zone, "zone136", 136,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A434", zones.back()->m_zone, "zone926", 926, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1979", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3194", zones.back()->m_zone, "zone112", 112,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3196", zones.back()->m_zone, "zone116", 116,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3197", zones.back()->m_zone, "zone118", 118,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3198", zones.back()->m_zone, "zone278", 278,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1997", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3196", zones.back()->m_zone, "zone115", 115,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3199", zones.back()->m_zone, "zone117", 117,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3200", zones.back()->m_zone, "zone135", 135,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3201", zones.back()->m_zone, "zone289", 289, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1996", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3199", zones.back()->m_zone, "zone116", 116,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3202", zones.back()->m_zone, "zone134", 134,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3203", zones.back()->m_zone, "zone120", 120,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3204", zones.back()->m_zone, "zone118", 118,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1975", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3163", zones.back()->m_zone, "zone98", 98,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3197", zones.back()->m_zone, "zone115", 115,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3204", zones.back()->m_zone, "zone117", 117,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3205", zones.back()->m_zone, "zone119", 119,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1971", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3160", zones.back()->m_zone, "zone97", 97,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3205", zones.back()->m_zone, "zone118", 118,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3206", zones.back()->m_zone, "zone120", 120,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3207", zones.back()->m_zone, "zone122", 122,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1991", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3203", zones.back()->m_zone, "zone117", 117,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3206", zones.back()->m_zone, "zone119", 119,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3208", zones.back()->m_zone, "zone121", 121,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3209", zones.back()->m_zone, "zone133", 133, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1990", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3208", zones.back()->m_zone, "zone120", 120,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3210", zones.back()->m_zone, "zone132", 132, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3211", zones.back()->m_zone, "zone124", 124,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3212", zones.back()->m_zone, "zone122", 122,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1957", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3207", zones.back()->m_zone, "zone119", 119,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3212", zones.back()->m_zone, "zone121", 121,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3213", zones.back()->m_zone, "zone139", 139,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3214", zones.back()->m_zone, "zone123", 123,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1964", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3149", zones.back()->m_zone, "zone93", 93,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3214", zones.back()->m_zone, "zone122", 122,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A3215", zones.back()->m_zone, "zone124", 124,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3216", zones.back()->m_zone, "zone127", 127,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1985", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3211", zones.back()->m_zone, "zone121", 121,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3215", zones.back()->m_zone, "zone123", 123,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3217", zones.back()->m_zone, "zone126", 126,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3218", zones.back()->m_zone, "zone129", 129,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B299", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B419", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B429", zones.back()->m_zone, "zone102", 102,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A435", zones.back()->m_zone, "zone136", 136, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A436", zones.back()->m_zone, "zone180", 180, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A437", zones.back()->m_zone, "zone937", 937, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1984", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3217", zones.back()->m_zone, "zone124", 124,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3219", zones.back()->m_zone, "zone128", 128,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3220", zones.back()->m_zone, "zone367", 367, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3221", zones.back()->m_zone, "zone127", 127,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1966", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3146", zones.back()->m_zone, "zone92", 92,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3216", zones.back()->m_zone, "zone123", 123,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3221", zones.back()->m_zone, "zone126", 126,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3222", zones.back()->m_zone, "zone377", 377,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2016", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3219", zones.back()->m_zone, "zone126", 126,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3223", zones.back()->m_zone, "zone129", 129,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3224", zones.back()->m_zone, "zone346", 346,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3225", zones.back()->m_zone, "zone130", 130,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2014", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3218", zones.back()->m_zone, "zone124", 124,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3223", zones.back()->m_zone, "zone128", 128,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3226", zones.back()->m_zone, "zone132", 132,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3227", zones.back()->m_zone, "zone130", 130,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2019", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3225", zones.back()->m_zone, "zone128", 128, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3227", zones.back()->m_zone, "zone129", 129,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3228", zones.back()->m_zone, "zone345", 345, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3229", zones.back()->m_zone, "zone131", 131, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2024", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3229", zones.back()->m_zone, "zone130", 130, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3230", zones.back()->m_zone, "zone132", 132,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3231", zones.back()->m_zone, "zone332", 332, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3232", zones.back()->m_zone, "zone138", 138, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2012", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3210", zones.back()->m_zone, "zone121", 121,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3226", zones.back()->m_zone, "zone129", 129,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3230", zones.back()->m_zone, "zone131", 131,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3233", zones.back()->m_zone, "zone133", 133,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2008", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3209", zones.back()->m_zone, "zone120", 120,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3233", zones.back()->m_zone, "zone132", 132,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3234", zones.back()->m_zone, "zone138", 138,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3235", zones.back()->m_zone, "zone134", 134,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2004", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3202", zones.back()->m_zone, "zone117", 117,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3235", zones.back()->m_zone, "zone133", 133,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3236", zones.back()->m_zone, "zone135", 135,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3237", zones.back()->m_zone, "zone137", 137,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2002", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3200", zones.back()->m_zone, "zone116", 116,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3236", zones.back()->m_zone, "zone134", 134,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3238", zones.back()->m_zone, "zone299", 299, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3239", zones.back()->m_zone, "zone137", 137,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B67", zones.back()->m_zone, "zone446", 446, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B420", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B426", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B433", zones.back()->m_zone, "zone114", 114,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B435", zones.back()->m_zone, "zone125", 125, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A438", zones.back()->m_zone, "zone938", 938,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2029", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3237", zones.back()->m_zone, "zone134", 134,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3239", zones.back()->m_zone, "zone135", 135, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3240", zones.back()->m_zone, "zone311", 311, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3241", zones.back()->m_zone, "zone138", 138,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2025", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3232", zones.back()->m_zone, "zone131", 131, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3234", zones.back()->m_zone, "zone133", 133,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3241", zones.back()->m_zone, "zone137", 137,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3242", zones.back()->m_zone, "zone314", 314,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2083", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3150", zones.back()->m_zone, "zone93", 93, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3155", zones.back()->m_zone, "zone95", 95, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3159", zones.back()->m_zone, "zone97", 97, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3213", zones.back()->m_zone, "zone122", 122,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1704", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3166", zones.back()->m_zone, "zone99", 99, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3170", zones.back()->m_zone, "zone101", 101,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3243", zones.back()->m_zone, "zone141", 141,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3244", zones.back()->m_zone, "zone170", 170,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1712", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3156", zones.back()->m_zone, "zone96", 96, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3243", zones.back()->m_zone, "zone140", 140,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3245", zones.back()->m_zone, "zone142", 142,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3246", zones.back()->m_zone, "zone168", 168,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1722", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3154", zones.back()->m_zone, "zone95", 95, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3245", zones.back()->m_zone, "zone141", 141,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3247", zones.back()->m_zone, "zone143", 143,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3248", zones.back()->m_zone, "zone154", 154,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1731", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3123", zones.back()->m_zone, "zone81", 81, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3152", zones.back()->m_zone, "zone94", 94, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3247", zones.back()->m_zone, "zone142", 142,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3249", zones.back()->m_zone, "zone156", 156,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3058", zones.back()->m_zone, "zone49", 49, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3144", zones.back()->m_zone, "zone90", 90, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3250", zones.back()->m_zone, "zone145", 145, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3251", zones.back()->m_zone, "zone148", 148,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3063", zones.back()->m_zone, "zone51", 51, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3250", zones.back()->m_zone, "zone144", 144, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3252", zones.back()->m_zone, "zone452", 452,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3253", zones.back()->m_zone, "zone146", 146,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2807", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3253", zones.back()->m_zone, "zone145", 145,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3254", zones.back()->m_zone, "zone148", 148,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3255", zones.back()->m_zone, "zone451", 451,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3256", zones.back()->m_zone, "zone150", 150,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B70", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B422", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A439", zones.back()->m_zone, "zone169", 169, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A440", zones.back()->m_zone, "zone158", 158, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A441", zones.back()->m_zone, "zone400", 400, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A442", zones.back()->m_zone, "zone794", 794, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2817", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3139", zones.back()->m_zone, "zone87", 87,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3251", zones.back()->m_zone, "zone144", 144,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3254", zones.back()->m_zone, "zone146", 146,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3257", zones.back()->m_zone, "zone149", 149,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2822", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3136", zones.back()->m_zone, "zone86", 86, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3257", zones.back()->m_zone, "zone148", 148,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3258", zones.back()->m_zone, "zone150", 150,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3259", zones.back()->m_zone, "zone153", 153,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2814", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3256", zones.back()->m_zone, "zone146", 146,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3258", zones.back()->m_zone, "zone149", 149,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3260", zones.back()->m_zone, "zone152", 152,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3261", zones.back()->m_zone, "zone439", 439, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2789", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3131", zones.back()->m_zone, "zone84", 84, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3262", zones.back()->m_zone, "zone152", 152,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3263", zones.back()->m_zone, "zone153", 153, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3264", zones.back()->m_zone, "zone426", 426, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2798", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3260", zones.back()->m_zone, "zone150", 150,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3262", zones.back()->m_zone, "zone151", 151,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3265", zones.back()->m_zone, "zone153", 153,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3266", zones.back()->m_zone, "zone433", 433,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2779", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3133", zones.back()->m_zone, "zone85", 85,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3259", zones.back()->m_zone, "zone149", 149,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3263", zones.back()->m_zone, "zone151", 151, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3265", zones.back()->m_zone, "zone152", 152,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2487", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3248", zones.back()->m_zone, "zone142", 142,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3267", zones.back()->m_zone, "zone155", 155,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3268", zones.back()->m_zone, "zone168", 168, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3269", zones.back()->m_zone, "zone156", 156,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2530", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3267", zones.back()->m_zone, "zone154", 154,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3270", zones.back()->m_zone, "zone173", 173,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3271", zones.back()->m_zone, "zone161", 161,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3272", zones.back()->m_zone, "zone157", 157,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2499", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3249", zones.back()->m_zone, "zone143", 143,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3269", zones.back()->m_zone, "zone154", 154,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3273", zones.back()->m_zone, "zone157", 157,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3274", zones.back()->m_zone, "zone159", 159,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2540", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3272", zones.back()->m_zone, "zone155", 155,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3273", zones.back()->m_zone, "zone156", 156,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3275", zones.back()->m_zone, "zone159", 159,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3276", zones.back()->m_zone, "zone194", 194,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B208", zones.back()->m_zone, "zone909", 909,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B431", zones.back()->m_zone, "zone102", 102, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B440", zones.back()->m_zone, "zone147", 147, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A443", zones.back()->m_zone, "zone180", 180,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A444", zones.back()->m_zone, "zone757", 757, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A445", zones.back()->m_zone, "zone384", 384, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2277", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3122", zones.back()->m_zone, "zone81", 81,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3274", zones.back()->m_zone, "zone156", 156,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3275", zones.back()->m_zone, "zone157", 157,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3277", zones.back()->m_zone, "zone160", 160,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2553", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3118", zones.back()->m_zone, "zone79", 79, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3277", zones.back()->m_zone, "zone159", 159,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3278", zones.back()->m_zone, "zone194", 194,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3279", zones.back()->m_zone, "zone164", 164,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2535", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3271", zones.back()->m_zone, "zone155", 155,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3280", zones.back()->m_zone, "zone174", 174,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3281", zones.back()->m_zone, "zone194", 194,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3282", zones.back()->m_zone, "zone162", 162,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2419", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3282", zones.back()->m_zone, "zone161", 161,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3283", zones.back()->m_zone, "zone179", 179,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3284", zones.back()->m_zone, "zone167", 167,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3285", zones.back()->m_zone, "zone163", 163,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2427", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3285", zones.back()->m_zone, "zone162", 162,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3286", zones.back()->m_zone, "zone164", 164,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3287", zones.back()->m_zone, "zone194", 194,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3288", zones.back()->m_zone, "zone166", 166,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2546", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3141", zones.back()->m_zone, "zone88", 88, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3279", zones.back()->m_zone, "zone160", 160,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3286", zones.back()->m_zone, "zone163", 163,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3289", zones.back()->m_zone, "zone165", 165,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3010", zones.back()->m_zone, "zone24", 24, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3142", zones.back()->m_zone, "zone89", 89, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3289", zones.back()->m_zone, "zone164", 164, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3290", zones.back()->m_zone, "zone166", 166,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3008", zones.back()->m_zone, "zone23", 23, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3288", zones.back()->m_zone, "zone163", 163, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3290", zones.back()->m_zone, "zone165", 165,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3291", zones.back()->m_zone, "zone167", 167, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2995", zones.back()->m_zone, "zone18", 18, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3284", zones.back()->m_zone, "zone162", 162,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3291", zones.back()->m_zone, "zone166", 166, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3292", zones.back()->m_zone, "zone181", 181,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2474", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3246", zones.back()->m_zone, "zone141", 141,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3268", zones.back()->m_zone, "zone154", 154, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3293", zones.back()->m_zone, "zone173", 173,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3294", zones.back()->m_zone, "zone170", 170,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B76", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B432", zones.back()->m_zone, "zone114", 114,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B439", zones.back()->m_zone, "zone147", 147, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A446", zones.back()->m_zone, "zone927", 927, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A447", zones.back()->m_zone, "zone180", 180,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A448", zones.back()->m_zone, "zone399", 399, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2462", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3244", zones.back()->m_zone, "zone140", 140,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3294", zones.back()->m_zone, "zone168", 168,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3295", zones.back()->m_zone, "zone172", 172,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3296", zones.back()->m_zone, "zone171", 171, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2451", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3171", zones.back()->m_zone, "zone101", 101,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3296", zones.back()->m_zone, "zone170", 170, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3297", zones.back()->m_zone, "zone172", 172,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3298", zones.back()->m_zone, "zone176", 176,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2510", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3295", zones.back()->m_zone, "zone170", 170,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3297", zones.back()->m_zone, "zone171", 171,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3299", zones.back()->m_zone, "zone173", 173,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3300", zones.back()->m_zone, "zone175", 175,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2522", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3270", zones.back()->m_zone, "zone155", 155,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3293", zones.back()->m_zone, "zone168", 168,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3299", zones.back()->m_zone, "zone172", 172,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3301", zones.back()->m_zone, "zone174", 174,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2526", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3280", zones.back()->m_zone, "zone161", 161,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3301", zones.back()->m_zone, "zone173", 173,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3302", zones.back()->m_zone, "zone175", 175,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3303", zones.back()->m_zone, "zone179", 179,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2515", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3300", zones.back()->m_zone, "zone172", 172,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3302", zones.back()->m_zone, "zone174", 174,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3304", zones.back()->m_zone, "zone176", 176,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3305", zones.back()->m_zone, "zone178", 178,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2441", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3175", zones.back()->m_zone, "zone103", 103,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3298", zones.back()->m_zone, "zone171", 171,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3304", zones.back()->m_zone, "zone175", 175,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3306", zones.back()->m_zone, "zone177", 177,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2435", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3183", zones.back()->m_zone, "zone106", 106,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3306", zones.back()->m_zone, "zone176", 176,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3307", zones.back()->m_zone, "zone178", 178,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3308", zones.back()->m_zone, "zone183", 183,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2260", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3305", zones.back()->m_zone, "zone175", 175,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3307", zones.back()->m_zone, "zone177", 177,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3309", zones.back()->m_zone, "zone179", 179,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3310", zones.back()->m_zone, "zone182", 182,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2432", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3283", zones.back()->m_zone, "zone162", 162,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3303", zones.back()->m_zone, "zone174", 174,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3309", zones.back()->m_zone, "zone178", 178,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3311", zones.back()->m_zone, "zone181", 181,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B301", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B436", zones.back()->m_zone, "zone125", 125, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B443", zones.back()->m_zone, "zone158", 158,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B447", zones.back()->m_zone, "zone169", 169,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A449", zones.back()->m_zone, "zone386", 386,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A450", zones.back()->m_zone, "zone936", 936,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2993", zones.back()->m_zone, "zone17", 17, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3292", zones.back()->m_zone, "zone167", 167,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3311", zones.back()->m_zone, "zone179", 179,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3312", zones.back()->m_zone, "zone182", 182, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2988", zones.back()->m_zone, "zone15", 15, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3310", zones.back()->m_zone, "zone178", 178, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3312", zones.back()->m_zone, "zone181", 181, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3313", zones.back()->m_zone, "zone183", 183,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2978", zones.back()->m_zone, "zone11", 11, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3184", zones.back()->m_zone, "zone107", 107,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3308", zones.back()->m_zone, "zone177", 177, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3313", zones.back()->m_zone, "zone182", 182,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2963", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3191", zones.back()->m_zone, "zone110", 110,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3314", zones.back()->m_zone, "zone186", 186,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3315", zones.back()->m_zone, "zone185", 185, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3316", zones.back()->m_zone, "zone223", 223,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2968", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3188", zones.back()->m_zone, "zone109", 109,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3315", zones.back()->m_zone, "zone184", 184, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3317", zones.back()->m_zone, "zone186", 186,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3318", zones.back()->m_zone, "zone188", 188,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2131", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3314", zones.back()->m_zone, "zone184", 184,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3317", zones.back()->m_zone, "zone185", 185,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3319", zones.back()->m_zone, "zone187", 187,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3320", zones.back()->m_zone, "zone217", 217, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2123", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3319", zones.back()->m_zone, "zone186", 186,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3321", zones.back()->m_zone, "zone188", 188,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3322", zones.back()->m_zone, "zone210", 210,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3323", zones.back()->m_zone, "zone190", 190,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2921", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3177", zones.back()->m_zone, "zone104", 104,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3318", zones.back()->m_zone, "zone185", 185,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3321", zones.back()->m_zone, "zone187", 187,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3324", zones.back()->m_zone, "zone189", 189,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2175", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3179", zones.back()->m_zone, "zone105", 105,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3324", zones.back()->m_zone, "zone188", 188,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3325", zones.back()->m_zone, "zone190", 190,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3326", zones.back()->m_zone, "zone193", 193,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2976", zones.back()->m_zone, "zone10", 10, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3323", zones.back()->m_zone, "zone187", 187,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3325", zones.back()->m_zone, "zone189", 189,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3327", zones.back()->m_zone, "zone192", 192,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3328", zones.back()->m_zone, "zone204", 204, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A451", zones.back()->m_zone, "zone826", 826, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A452", zones.back()->m_zone, "zone853", 853, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A453", zones.back()->m_zone, "zone206", 206, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A454", zones.back()->m_zone, "zone213", 213, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A455", zones.back()->m_zone, "zone202", 202,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3115", zones.back()->m_zone, "zone78", 78, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3327", zones.back()->m_zone, "zone190", 190,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3329", zones.back()->m_zone, "zone196", 196,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3330", zones.back()->m_zone, "zone193", 193, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3113", zones.back()->m_zone, "zone77", 77, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3186", zones.back()->m_zone, "zone108", 108,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3326", zones.back()->m_zone, "zone189", 189,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3330", zones.back()->m_zone, "zone192", 192, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2544", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3276", zones.back()->m_zone, "zone157", 157,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3278", zones.back()->m_zone, "zone160", 160,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3281", zones.back()->m_zone, "zone161", 161,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3287", zones.back()->m_zone, "zone163", 163,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1664", zones.back()->m_zone, "zone1036", 1036,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3331", zones.back()->m_zone, "zone197", 197,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3332", zones.back()->m_zone, "zone205", 205,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3333", zones.back()->m_zone, "zone196", 196,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3329", zones.back()->m_zone, "zone192", 192,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3333", zones.back()->m_zone, "zone195", 195,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3334", zones.back()->m_zone, "zone204", 204,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3335", zones.back()->m_zone, "zone198", 198,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1451", zones.back()->m_zone, "zone923", 923,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3331", zones.back()->m_zone, "zone195", 195,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3336", zones.back()->m_zone, "zone547", 547, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3337", zones.back()->m_zone, "zone201", 201,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3338", zones.back()->m_zone, "zone198", 198,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3116", zones.back()->m_zone, "zone78", 78, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3335", zones.back()->m_zone, "zone196", 196, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3338", zones.back()->m_zone, "zone197", 197,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3339", zones.back()->m_zone, "zone203", 203,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3340", zones.back()->m_zone, "zone549", 549, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2973", zones.back()->m_zone, "zone9", 9, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3341", zones.back()->m_zone, "zone207", 207, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3342", zones.back()->m_zone, "zone200", 200,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3343", zones.back()->m_zone, "zone553", 553,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3344", zones.back()->m_zone, "zone203", 203,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1308", zones.back()->m_zone, "zone853", 853, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3342", zones.back()->m_zone, "zone199", 199,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3345", zones.back()->m_zone, "zone551", 551, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3346", zones.back()->m_zone, "zone206", 206,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3347", zones.back()->m_zone, "zone201", 201,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1311", zones.back()->m_zone, "zone855", 855, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3337", zones.back()->m_zone, "zone197", 197,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3347", zones.back()->m_zone, "zone200", 200,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3348", zones.back()->m_zone, "zone208", 208, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3349", zones.back()->m_zone, "zone205", 205,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3350", zones.back()->m_zone, "zone203", 203,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B455", zones.back()->m_zone, "zone191", 191,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A456", zones.back()->m_zone, "zone825", 825, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A457", zones.back()->m_zone, "zone225", 225, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A458", zones.back()->m_zone, "zone1032", 1032,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A459", zones.back()->m_zone, "zone208", 208, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A460", zones.back()->m_zone, "zone855", 855, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2977", zones.back()->m_zone, "zone10", 10, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3339", zones.back()->m_zone, "zone198", 198,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3344", zones.back()->m_zone, "zone199", 199,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3350", zones.back()->m_zone, "zone201", 201,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3351", zones.back()->m_zone, "zone204", 204,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3352", zones.back()->m_zone, "zone209", 209,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3328", zones.back()->m_zone, "zone190", 190, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3334", zones.back()->m_zone, "zone196", 196,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3351", zones.back()->m_zone, "zone203", 203,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3353", zones.back()->m_zone, "zone205", 205,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3354", zones.back()->m_zone, "zone210", 210,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1663", zones.back()->m_zone, "zone1035", 1035,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3332", zones.back()->m_zone, "zone195", 195,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3349", zones.back()->m_zone, "zone201", 201,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3353", zones.back()->m_zone, "zone204", 204,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3355", zones.back()->m_zone, "zone211", 211,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B453", zones.back()->m_zone, "zone191", 191, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3346", zones.back()->m_zone, "zone200", 200, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3356", zones.back()->m_zone, "zone207", 207,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3357", zones.back()->m_zone, "zone214", 214,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3358", zones.back()->m_zone, "zone208", 208,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2118", zones.back()->m_zone, "zone1169", 1169,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3341", zones.back()->m_zone, "zone199", 199,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3356", zones.back()->m_zone, "zone206", 206,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3359", zones.back()->m_zone, "zone212", 212, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3360", zones.back()->m_zone, "zone209", 209,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B459", zones.back()->m_zone, "zone202", 202, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3348", zones.back()->m_zone, "zone201", 201,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3358", zones.back()->m_zone, "zone206", 206,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3361", zones.back()->m_zone, "zone215", 215, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3362", zones.back()->m_zone, "zone211", 211,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3363", zones.back()->m_zone, "zone209", 209,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2124", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3352", zones.back()->m_zone, "zone203", 203, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3360", zones.back()->m_zone, "zone207", 207,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3363", zones.back()->m_zone, "zone208", 208,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3364", zones.back()->m_zone, "zone210", 210,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3365", zones.back()->m_zone, "zone216", 216,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3322", zones.back()->m_zone, "zone187", 187,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3354", zones.back()->m_zone, "zone204", 204, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3364", zones.back()->m_zone, "zone209", 209,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3366", zones.back()->m_zone, "zone211", 211,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3367", zones.back()->m_zone, "zone217", 217,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1657", zones.back()->m_zone, "zone1032", 1032,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3355", zones.back()->m_zone, "zone205", 205, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3362", zones.back()->m_zone, "zone208", 208,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3366", zones.back()->m_zone, "zone210", 210,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3368", zones.back()->m_zone, "zone218", 218,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2128", zones.back()->m_zone, "zone1171", 1171,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3359", zones.back()->m_zone, "zone207", 207, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3369", zones.back()->m_zone, "zone220", 220, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3370", zones.back()->m_zone, "zone214", 214,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3371", zones.back()->m_zone, "zone216", 216,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B454", zones.back()->m_zone, "zone191", 191, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A461", zones.back()->m_zone, "zone851", 851, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A462", zones.back()->m_zone, "zone214", 214, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A463", zones.back()->m_zone, "zone849", 849,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A464", zones.back()->m_zone, "zone225", 225,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B462", zones.back()->m_zone, "zone213", 213, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3357", zones.back()->m_zone, "zone206", 206, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3370", zones.back()->m_zone, "zone212", 212,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3372", zones.back()->m_zone, "zone219", 219, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3373", zones.back()->m_zone, "zone215", 215,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B468", zones.back()->m_zone, "zone225", 225, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3361", zones.back()->m_zone, "zone208", 208, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3373", zones.back()->m_zone, "zone214", 214,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3374", zones.back()->m_zone, "zone222", 222, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3375", zones.back()->m_zone, "zone218", 218,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3376", zones.back()->m_zone, "zone216", 216,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2132", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3365", zones.back()->m_zone, "zone209", 209, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3371", zones.back()->m_zone, "zone212", 212,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3376", zones.back()->m_zone, "zone215", 215,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3377", zones.back()->m_zone, "zone217", 217,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3378", zones.back()->m_zone, "zone221", 221, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3320", zones.back()->m_zone, "zone186", 186, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3367", zones.back()->m_zone, "zone210", 210, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3377", zones.back()->m_zone, "zone216", 216,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3379", zones.back()->m_zone, "zone218", 218,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3380", zones.back()->m_zone, "zone223", 223,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1655", zones.back()->m_zone, "zone1030", 1030,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3368", zones.back()->m_zone, "zone211", 211, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3375", zones.back()->m_zone, "zone215", 215,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3379", zones.back()->m_zone, "zone217", 217,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3381", zones.back()->m_zone, "zone226", 226,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1301", zones.back()->m_zone, "zone849", 849,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3372", zones.back()->m_zone, "zone214", 214, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3382", zones.back()->m_zone, "zone220", 220,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3383", zones.back()->m_zone, "zone227", 227,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3384", zones.back()->m_zone, "zone222", 222,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2966", zones.back()->m_zone, "zone5", 5, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3369", zones.back()->m_zone, "zone212", 212,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3382", zones.back()->m_zone, "zone219", 219,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3385", zones.back()->m_zone, "zone228", 228,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3386", zones.back()->m_zone, "zone221", 221,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2962", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3378", zones.back()->m_zone, "zone216", 216, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3386", zones.back()->m_zone, "zone220", 220,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3387", zones.back()->m_zone, "zone223", 223,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3388", zones.back()->m_zone, "zone222", 222,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3389", zones.back()->m_zone, "zone229", 229,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1297", zones.back()->m_zone, "zone848", 848,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3374", zones.back()->m_zone, "zone215", 215,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3384", zones.back()->m_zone, "zone219", 219,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3388", zones.back()->m_zone, "zone221", 221,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3390", zones.back()->m_zone, "zone230", 230,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3391", zones.back()->m_zone, "zone226", 226,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3316", zones.back()->m_zone, "zone184", 184,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3380", zones.back()->m_zone, "zone217", 217,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3387", zones.back()->m_zone, "zone221", 221,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3392", zones.back()->m_zone, "zone226", 226,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3393", zones.back()->m_zone, "zone245", 245,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B46", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B50", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A55", zones.back()->m_zone, "zone986", 986, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A56", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A57", zones.back()->m_zone, "zone446", 446, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A58", zones.back()->m_zone, "zone645", 645, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B457", zones.back()->m_zone, "zone202", 202, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B464", zones.back()->m_zone, "zone213", 213,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A465", zones.back()->m_zone, "zone850", 850, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A466", zones.back()->m_zone, "zone848", 848,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A467", zones.back()->m_zone, "zone1030", 1030,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A468", zones.back()->m_zone, "zone215", 215, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1651", zones.back()->m_zone, "zone1028", 1028,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3381", zones.back()->m_zone, "zone218", 218,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3391", zones.back()->m_zone, "zone222", 222,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3392", zones.back()->m_zone, "zone223", 223,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3394", zones.back()->m_zone, "zone244", 244,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1296", zones.back()->m_zone, "zone847", 847,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3383", zones.back()->m_zone, "zone219", 219,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3395", zones.back()->m_zone, "zone228", 228, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3396", zones.back()->m_zone, "zone233", 233,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3397", zones.back()->m_zone, "zone230", 230,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2960", zones.back()->m_zone, "zone1496", 1496,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3385", zones.back()->m_zone, "zone220", 220,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3395", zones.back()->m_zone, "zone227", 227, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3398", zones.back()->m_zone, "zone234", 234,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3399", zones.back()->m_zone, "zone229", 229,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B2958", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3389", zones.back()->m_zone, "zone221", 221,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3399", zones.back()->m_zone, "zone228", 228,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3400", zones.back()->m_zone, "zone230", 230, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3401", zones.back()->m_zone, "zone243", 243,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3402", zones.back()->m_zone, "zone232", 232,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1292", zones.back()->m_zone, "zone846", 846,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3390", zones.back()->m_zone, "zone222", 222,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3397", zones.back()->m_zone, "zone227", 227,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3400", zones.back()->m_zone, "zone229", 229, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3403", zones.back()->m_zone, "zone242", 242,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3404", zones.back()->m_zone, "zone231", 231,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1290", zones.back()->m_zone, "zone845", 845, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3404", zones.back()->m_zone, "zone230", 230,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3405", zones.back()->m_zone, "zone239", 239,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3406", zones.back()->m_zone, "zone232", 232, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3407", zones.back()->m_zone, "zone241", 241,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3408", zones.back()->m_zone, "zone233", 233,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B2956", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3402", zones.back()->m_zone, "zone229", 229,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3406", zones.back()->m_zone, "zone231", 231, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3409", zones.back()->m_zone, "zone238", 238,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3410", zones.back()->m_zone, "zone240", 240,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3411", zones.back()->m_zone, "zone234", 234,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1288", zones.back()->m_zone, "zone844", 844, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3396", zones.back()->m_zone, "zone227", 227,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3408", zones.back()->m_zone, "zone231", 231,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3412", zones.back()->m_zone, "zone237", 237,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3413", zones.back()->m_zone, "zone234", 234, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2952", zones.back()->m_zone, "zone1493", 1493,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3398", zones.back()->m_zone, "zone228", 228,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3411", zones.back()->m_zone, "zone232", 232,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3413", zones.back()->m_zone, "zone233", 233, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3414", zones.back()->m_zone, "zone235", 235,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2947", zones.back()->m_zone, "zone1491", 1491,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3414", zones.back()->m_zone, "zone234", 234,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3415", zones.back()->m_zone, "zone248", 248, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3416", zones.back()->m_zone, "zone237", 237,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3417", zones.back()->m_zone, "zone238", 238,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A469", zones.back()->m_zone, "zone280", 280, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A470", zones.back()->m_zone, "zone590", 590, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A471", zones.back()->m_zone, "zone369", 369, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A472", zones.back()->m_zone, "zone833", 833, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A473", zones.back()->m_zone, "zone247", 247,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1283", zones.back()->m_zone, "zone841", 841, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3412", zones.back()->m_zone, "zone233", 233,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3416", zones.back()->m_zone, "zone235", 235,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3418", zones.back()->m_zone, "zone246", 246, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3419", zones.back()->m_zone, "zone239", 239,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2951", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3409", zones.back()->m_zone, "zone232", 232,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3417", zones.back()->m_zone, "zone235", 235,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3420", zones.back()->m_zone, "zone240", 240,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3421", zones.back()->m_zone, "zone239", 239,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3422", zones.back()->m_zone, "zone250", 250, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1285", zones.back()->m_zone, "zone842", 842, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3405", zones.back()->m_zone, "zone231", 231,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3419", zones.back()->m_zone, "zone237", 237,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3421", zones.back()->m_zone, "zone238", 238,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3423", zones.back()->m_zone, "zone249", 249, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3424", zones.back()->m_zone, "zone241", 241,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1669", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3410", zones.back()->m_zone, "zone232", 232,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3420", zones.back()->m_zone, "zone238", 238,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3425", zones.back()->m_zone, "zone243", 243, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3426", zones.back()->m_zone, "zone252", 252, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3427", zones.back()->m_zone, "zone241", 241,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3407", zones.back()->m_zone, "zone231", 231,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3424", zones.back()->m_zone, "zone239", 239,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3427", zones.back()->m_zone, "zone240", 240,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3428", zones.back()->m_zone, "zone251", 251, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3429", zones.back()->m_zone, "zone242", 242, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B9", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3403", zones.back()->m_zone, "zone230", 230,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3429", zones.back()->m_zone, "zone241", 241, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3430", zones.back()->m_zone, "zone253", 253, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3431", zones.back()->m_zone, "zone243", 243,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3432", zones.back()->m_zone, "zone244", 244,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1675", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3401", zones.back()->m_zone, "zone229", 229,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3425", zones.back()->m_zone, "zone240", 240,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3431", zones.back()->m_zone, "zone242", 242,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3433", zones.back()->m_zone, "zone245", 245, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3434", zones.back()->m_zone, "zone254", 254, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1525", zones.back()->m_zone, "zone956", 956, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3394", zones.back()->m_zone, "zone226", 226,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3432", zones.back()->m_zone, "zone242", 242, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3435", zones.back()->m_zone, "zone255", 255, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3436", zones.back()->m_zone, "zone245", 245,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3190", zones.back()->m_zone, "zone110", 110, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3393", zones.back()->m_zone, "zone223", 223,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3433", zones.back()->m_zone, "zone243", 243,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3436", zones.back()->m_zone, "zone244", 244,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3437", zones.back()->m_zone, "zone256", 256, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B997", zones.back()->m_zone, "zone719", 719, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3418", zones.back()->m_zone, "zone237", 237, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3438", zones.back()->m_zone, "zone248", 248,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3439", zones.back()->m_zone, "zone266", 266, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3440", zones.back()->m_zone, "zone249", 249,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B143", zones.back()->m_zone, "zone732", 732, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B473", zones.back()->m_zone, "zone236", 236,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A474", zones.back()->m_zone, "zone291", 291, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A475", zones.back()->m_zone, "zone834", 834, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A476", zones.back()->m_zone, "zone380", 380, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A477", zones.back()->m_zone, "zone591", 591, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2662", zones.back()->m_zone, "zone1370", 1370,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3415", zones.back()->m_zone, "zone235", 235, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3438", zones.back()->m_zone, "zone246", 246,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3441", zones.back()->m_zone, "zone267", 267, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3442", zones.back()->m_zone, "zone250", 250,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1002", zones.back()->m_zone, "zone722", 722, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3423", zones.back()->m_zone, "zone239", 239, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3440", zones.back()->m_zone, "zone246", 246,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3443", zones.back()->m_zone, "zone265", 265, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3444", zones.back()->m_zone, "zone251", 251,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3445", zones.back()->m_zone, "zone250", 250,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2667", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3422", zones.back()->m_zone, "zone238", 238, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3442", zones.back()->m_zone, "zone248", 248,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3445", zones.back()->m_zone, "zone249", 249,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3446", zones.back()->m_zone, "zone252", 252,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3447", zones.back()->m_zone, "zone264", 264, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B370", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3428", zones.back()->m_zone, "zone241", 241, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3444", zones.back()->m_zone, "zone249", 249,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3448", zones.back()->m_zone, "zone262", 262,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3449", zones.back()->m_zone, "zone252", 252,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3450", zones.back()->m_zone, "zone253", 253, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2035", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3426", zones.back()->m_zone, "zone240", 240, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3446", zones.back()->m_zone, "zone250", 250,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3449", zones.back()->m_zone, "zone251", 251,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3451", zones.back()->m_zone, "zone254", 254, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3452", zones.back()->m_zone, "zone263", 263,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B381", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3430", zones.back()->m_zone, "zone242", 242, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3450", zones.back()->m_zone, "zone251", 251, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3453", zones.back()->m_zone, "zone260", 260,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3454", zones.back()->m_zone, "zone254", 254,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3455", zones.back()->m_zone, "zone255", 255,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2045", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3434", zones.back()->m_zone, "zone243", 243, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3451", zones.back()->m_zone, "zone252", 252,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3454", zones.back()->m_zone, "zone253", 253,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3456", zones.back()->m_zone, "zone256", 256, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3457", zones.back()->m_zone, "zone261", 261,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1528", zones.back()->m_zone, "zone957", 957, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3435", zones.back()->m_zone, "zone244", 244, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3455", zones.back()->m_zone, "zone253", 253, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3458", zones.back()->m_zone, "zone257", 257,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3459", zones.back()->m_zone, "zone256", 256,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3193", zones.back()->m_zone, "zone111", 111, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3437", zones.back()->m_zone, "zone245", 245, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3456", zones.back()->m_zone, "zone254", 254,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3459", zones.back()->m_zone, "zone255", 255,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3460", zones.back()->m_zone, "zone259", 259,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1530", zones.back()->m_zone, "zone958", 958,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3458", zones.back()->m_zone, "zone255", 255,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3461", zones.back()->m_zone, "zone259", 259,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3462", zones.back()->m_zone, "zone260", 260,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3463", zones.back()->m_zone, "zone277", 277,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A478", zones.back()->m_zone, "zone839", 839, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A479", zones.back()->m_zone, "zone280", 280, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A480", zones.back()->m_zone, "zone833", 833, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A481", zones.back()->m_zone, "zone811", 811, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A482", zones.back()->m_zone, "zone269", 269,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B3195", zones.back()->m_zone, "zone112", 112,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3460", zones.back()->m_zone, "zone256", 256,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3461", zones.back()->m_zone, "zone257", 257,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3464", zones.back()->m_zone, "zone261", 261, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3465", zones.back()->m_zone, "zone278", 278,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B385", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3453", zones.back()->m_zone, "zone253", 253,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3462", zones.back()->m_zone, "zone257", 257, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3466", zones.back()->m_zone, "zone261", 261,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3467", zones.back()->m_zone, "zone262", 262,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3468", zones.back()->m_zone, "zone276", 276,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2050", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3457", zones.back()->m_zone, "zone254", 254,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3464", zones.back()->m_zone, "zone259", 259,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3466", zones.back()->m_zone, "zone260", 260,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3469", zones.back()->m_zone, "zone263", 263, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3470", zones.back()->m_zone, "zone275", 275,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B376", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3448", zones.back()->m_zone, "zone251", 251,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3467", zones.back()->m_zone, "zone260", 260, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3471", zones.back()->m_zone, "zone263", 263,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3472", zones.back()->m_zone, "zone265", 265,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3473", zones.back()->m_zone, "zone274", 274,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2041", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3452", zones.back()->m_zone, "zone252", 252,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3469", zones.back()->m_zone, "zone261", 261, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3471", zones.back()->m_zone, "zone262", 262,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3474", zones.back()->m_zone, "zone264", 264,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3475", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2673", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3447", zones.back()->m_zone, "zone250", 250, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3474", zones.back()->m_zone, "zone263", 263,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3476", zones.back()->m_zone, "zone265", 265,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3477", zones.back()->m_zone, "zone271", 271,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3478", zones.back()->m_zone, "zone267", 267,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1010", zones.back()->m_zone, "zone724", 724,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3443", zones.back()->m_zone, "zone249", 249,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3472", zones.back()->m_zone, "zone262", 262,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3476", zones.back()->m_zone, "zone264", 264,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3479", zones.back()->m_zone, "zone272", 272, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3480", zones.back()->m_zone, "zone266", 266,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1004", zones.back()->m_zone, "zone723", 723,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3439", zones.back()->m_zone, "zone246", 246, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3480", zones.back()->m_zone, "zone265", 265,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3481", zones.back()->m_zone, "zone267", 267,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3482", zones.back()->m_zone, "zone268", 268,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2671", zones.back()->m_zone, "zone1372", 1372,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3441", zones.back()->m_zone, "zone248", 248,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3478", zones.back()->m_zone, "zone264", 264,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3481", zones.back()->m_zone, "zone266", 266,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3483", zones.back()->m_zone, "zone270", 270, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1012", zones.back()->m_zone, "zone725", 725,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3482", zones.back()->m_zone, "zone266", 266, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3484", zones.back()->m_zone, "zone270", 270,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3485", zones.back()->m_zone, "zone281", 281,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3486", zones.back()->m_zone, "zone272", 272,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B145", zones.back()->m_zone, "zone743", 743, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B482", zones.back()->m_zone, "zone258", 258,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A483", zones.back()->m_zone, "zone840", 840, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A484", zones.back()->m_zone, "zone812", 812, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A485", zones.back()->m_zone, "zone834", 834, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A486", zones.back()->m_zone, "zone291", 291, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2676", zones.back()->m_zone, "zone1374", 1374,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3483", zones.back()->m_zone, "zone267", 267,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3484", zones.back()->m_zone, "zone268", 268,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3487", zones.back()->m_zone, "zone279", 279, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3488", zones.back()->m_zone, "zone271", 271,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2680", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3477", zones.back()->m_zone, "zone264", 264, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3488", zones.back()->m_zone, "zone270", 270,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3489", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3490", zones.back()->m_zone, "zone272", 272,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3491", zones.back()->m_zone, "zone283", 283,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1014", zones.back()->m_zone, "zone726", 726,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3479", zones.back()->m_zone, "zone265", 265,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3486", zones.back()->m_zone, "zone268", 268,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3490", zones.back()->m_zone, "zone271", 271,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3492", zones.back()->m_zone, "zone282", 282, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3493", zones.back()->m_zone, "zone274", 274,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1890", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3475", zones.back()->m_zone, "zone263", 263,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3489", zones.back()->m_zone, "zone271", 271,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3494", zones.back()->m_zone, "zone275", 275, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3495", zones.back()->m_zone, "zone284", 284, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3496", zones.back()->m_zone, "zone274", 274,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B225", zones.back()->m_zone, "zone943", 943, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3473", zones.back()->m_zone, "zone262", 262,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3493", zones.back()->m_zone, "zone272", 272,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3496", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3497", zones.back()->m_zone, "zone285", 285, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3498", zones.back()->m_zone, "zone276", 276, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1980", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3470", zones.back()->m_zone, "zone261", 261,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3494", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3499", zones.back()->m_zone, "zone278", 278, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3500", zones.back()->m_zone, "zone286", 286, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3501", zones.back()->m_zone, "zone276", 276,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B315", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3468", zones.back()->m_zone, "zone260", 260,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3498", zones.back()->m_zone, "zone274", 274, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3501", zones.back()->m_zone, "zone275", 275,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3502", zones.back()->m_zone, "zone287", 287, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3503", zones.back()->m_zone, "zone277", 277,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1533", zones.back()->m_zone, "zone959", 959,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3463", zones.back()->m_zone, "zone257", 257,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3503", zones.back()->m_zone, "zone276", 276, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3504", zones.back()->m_zone, "zone288", 288, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3505", zones.back()->m_zone, "zone278", 278,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B3198", zones.back()->m_zone, "zone115", 115,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3465", zones.back()->m_zone, "zone259", 259,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3499", zones.back()->m_zone, "zone275", 275,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3505", zones.back()->m_zone, "zone277", 277,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3506", zones.back()->m_zone, "zone289", 289, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2655", zones.back()->m_zone, "zone1368", 1368,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3487", zones.back()->m_zone, "zone270", 270,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3507", zones.back()->m_zone, "zone290", 290, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3508", zones.back()->m_zone, "zone281", 281,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3509", zones.back()->m_zone, "zone283", 283,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B469", zones.back()->m_zone, "zone236", 236, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B479", zones.back()->m_zone, "zone258", 258, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A487", zones.back()->m_zone, "zone302", 302, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A488", zones.back()->m_zone, "zone571", 571, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A489", zones.back()->m_zone, "zone291", 291,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B991", zones.back()->m_zone, "zone717", 717, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3485", zones.back()->m_zone, "zone268", 268, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3508", zones.back()->m_zone, "zone279", 279,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3510", zones.back()->m_zone, "zone292", 292,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3511", zones.back()->m_zone, "zone282", 282,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B993", zones.back()->m_zone, "zone718", 718, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3492", zones.back()->m_zone, "zone272", 272,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3511", zones.back()->m_zone, "zone281", 281,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3512", zones.back()->m_zone, "zone294", 294, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3513", zones.back()->m_zone, "zone285", 285,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3514", zones.back()->m_zone, "zone283", 283,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2659", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3491", zones.back()->m_zone, "zone271", 271, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3509", zones.back()->m_zone, "zone279", 279,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3514", zones.back()->m_zone, "zone282", 282,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3515", zones.back()->m_zone, "zone284", 284,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3516", zones.back()->m_zone, "zone293", 293,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1914", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3495", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3515", zones.back()->m_zone, "zone283", 283,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3517", zones.back()->m_zone, "zone286", 286, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3518", zones.back()->m_zone, "zone296", 296, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3519", zones.back()->m_zone, "zone285", 285,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B249", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3497", zones.back()->m_zone, "zone274", 274,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3513", zones.back()->m_zone, "zone282", 282,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3519", zones.back()->m_zone, "zone284", 284,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3520", zones.back()->m_zone, "zone295", 295, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3521", zones.back()->m_zone, "zone287", 287, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1999", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3500", zones.back()->m_zone, "zone275", 275,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3517", zones.back()->m_zone, "zone284", 284,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3522", zones.back()->m_zone, "zone289", 289, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3523", zones.back()->m_zone, "zone298", 298, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3524", zones.back()->m_zone, "zone287", 287,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B333", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3502", zones.back()->m_zone, "zone276", 276,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3521", zones.back()->m_zone, "zone285", 285, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3524", zones.back()->m_zone, "zone286", 286,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3525", zones.back()->m_zone, "zone297", 297, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3526", zones.back()->m_zone, "zone288", 288,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1536", zones.back()->m_zone, "zone960", 960, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3504", zones.back()->m_zone, "zone277", 277,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3526", zones.back()->m_zone, "zone287", 287, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3527", zones.back()->m_zone, "zone300", 300,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3528", zones.back()->m_zone, "zone289", 289,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3201", zones.back()->m_zone, "zone116", 116, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3506", zones.back()->m_zone, "zone278", 278,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3522", zones.back()->m_zone, "zone286", 286,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3528", zones.back()->m_zone, "zone288", 288,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3529", zones.back()->m_zone, "zone299", 299,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2598", zones.back()->m_zone, "zone1348", 1348,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3507", zones.back()->m_zone, "zone279", 279,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3530", zones.back()->m_zone, "zone301", 301, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3531", zones.back()->m_zone, "zone292", 292,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3532", zones.back()->m_zone, "zone293", 293,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B132", zones.back()->m_zone, "zone709", 709, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B474", zones.back()->m_zone, "zone247", 247, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B486", zones.back()->m_zone, "zone269", 269, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B489", zones.back()->m_zone, "zone280", 280,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A490", zones.back()->m_zone, "zone313", 313, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A491", zones.back()->m_zone, "zone572", 572, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B933", zones.back()->m_zone, "zone697", 697, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3510", zones.back()->m_zone, "zone281", 281, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3531", zones.back()->m_zone, "zone290", 290,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3533", zones.back()->m_zone, "zone303", 303,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3534", zones.back()->m_zone, "zone294", 294,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2603", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3516", zones.back()->m_zone, "zone283", 283, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3532", zones.back()->m_zone, "zone290", 290,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3535", zones.back()->m_zone, "zone296", 296,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3536", zones.back()->m_zone, "zone294", 294,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3537", zones.back()->m_zone, "zone305", 305,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B938", zones.back()->m_zone, "zone699", 699, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3512", zones.back()->m_zone, "zone282", 282,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3534", zones.back()->m_zone, "zone292", 292,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3536", zones.back()->m_zone, "zone293", 293,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3538", zones.back()->m_zone, "zone304", 304, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3539", zones.back()->m_zone, "zone295", 295,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B253", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3520", zones.back()->m_zone, "zone285", 285,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3539", zones.back()->m_zone, "zone294", 294,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3540", zones.back()->m_zone, "zone306", 306,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3541", zones.back()->m_zone, "zone296", 296,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3542", zones.back()->m_zone, "zone297", 297, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1918", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3518", zones.back()->m_zone, "zone284", 284,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3535", zones.back()->m_zone, "zone293", 293,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3541", zones.back()->m_zone, "zone295", 295,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3543", zones.back()->m_zone, "zone298", 298, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3544", zones.back()->m_zone, "zone307", 307,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B337", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3525", zones.back()->m_zone, "zone287", 287,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3542", zones.back()->m_zone, "zone295", 295, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3545", zones.back()->m_zone, "zone309", 309,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3546", zones.back()->m_zone, "zone298", 298,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3547", zones.back()->m_zone, "zone300", 300,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2001", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3523", zones.back()->m_zone, "zone286", 286,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3543", zones.back()->m_zone, "zone296", 296,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3546", zones.back()->m_zone, "zone297", 297,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3548", zones.back()->m_zone, "zone299", 299,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3549", zones.back()->m_zone, "zone308", 308,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3238", zones.back()->m_zone, "zone135", 135, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3529", zones.back()->m_zone, "zone289", 289,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3548", zones.back()->m_zone, "zone298", 298,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3550", zones.back()->m_zone, "zone300", 300,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3551", zones.back()->m_zone, "zone311", 311,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1573", zones.back()->m_zone, "zone980", 980, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3527", zones.back()->m_zone, "zone288", 288,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3547", zones.back()->m_zone, "zone297", 297,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3550", zones.back()->m_zone, "zone299", 299,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3552", zones.back()->m_zone, "zone310", 310,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2613", zones.back()->m_zone, "zone1352", 1352,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3530", zones.back()->m_zone, "zone290", 290, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3553", zones.back()->m_zone, "zone322", 322, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3554", zones.back()->m_zone, "zone303", 303,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3555", zones.back()->m_zone, "zone305", 305,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B487", zones.back()->m_zone, "zone280", 280, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A492", zones.back()->m_zone, "zone837", 837, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A493", zones.back()->m_zone, "zone413", 413, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A494", zones.back()->m_zone, "zone560", 560, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A495", zones.back()->m_zone, "zone313", 313,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B947", zones.back()->m_zone, "zone702", 702, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3533", zones.back()->m_zone, "zone292", 292, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3554", zones.back()->m_zone, "zone301", 301,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3556", zones.back()->m_zone, "zone321", 321, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3557", zones.back()->m_zone, "zone304", 304,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B951", zones.back()->m_zone, "zone703", 703, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3538", zones.back()->m_zone, "zone294", 294, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3557", zones.back()->m_zone, "zone303", 303,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3558", zones.back()->m_zone, "zone319", 319, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3559", zones.back()->m_zone, "zone306", 306,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3560", zones.back()->m_zone, "zone305", 305,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2615", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3537", zones.back()->m_zone, "zone293", 293, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3555", zones.back()->m_zone, "zone301", 301,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3560", zones.back()->m_zone, "zone304", 304,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3561", zones.back()->m_zone, "zone307", 307,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3562", zones.back()->m_zone, "zone320", 320, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B289", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3540", zones.back()->m_zone, "zone295", 295,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3559", zones.back()->m_zone, "zone304", 304,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3563", zones.back()->m_zone, "zone309", 309, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3564", zones.back()->m_zone, "zone307", 307,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3565", zones.back()->m_zone, "zone317", 317,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1954", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3544", zones.back()->m_zone, "zone296", 296,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3561", zones.back()->m_zone, "zone305", 305,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3564", zones.back()->m_zone, "zone306", 306,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3566", zones.back()->m_zone, "zone308", 308, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3567", zones.back()->m_zone, "zone318", 318,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2028", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3549", zones.back()->m_zone, "zone298", 298,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3566", zones.back()->m_zone, "zone307", 307, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3568", zones.back()->m_zone, "zone309", 309,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3569", zones.back()->m_zone, "zone311", 311,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3570", zones.back()->m_zone, "zone316", 316,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B364", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3545", zones.back()->m_zone, "zone297", 297,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3563", zones.back()->m_zone, "zone306", 306,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3568", zones.back()->m_zone, "zone308", 308,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3571", zones.back()->m_zone, "zone310", 310, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3572", zones.back()->m_zone, "zone315", 315,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1575", zones.back()->m_zone, "zone981", 981, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3552", zones.back()->m_zone, "zone300", 300, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3571", zones.back()->m_zone, "zone309", 309,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3573", zones.back()->m_zone, "zone311", 311,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3574", zones.back()->m_zone, "zone312", 312, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3240", zones.back()->m_zone, "zone137", 137, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3551", zones.back()->m_zone, "zone299", 299, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3569", zones.back()->m_zone, "zone308", 308, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3573", zones.back()->m_zone, "zone310", 310,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3575", zones.back()->m_zone, "zone314", 314, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1577", zones.back()->m_zone, "zone982", 982,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3574", zones.back()->m_zone, "zone310", 310, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3576", zones.back()->m_zone, "zone314", 314,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3577", zones.back()->m_zone, "zone315", 315,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3578", zones.back()->m_zone, "zone333", 333,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B25", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B490", zones.back()->m_zone, "zone291", 291, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B495", zones.back()->m_zone, "zone302", 302,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A496", zones.back()->m_zone, "zone838", 838, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A497", zones.back()->m_zone, "zone561", 561, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A498", zones.back()->m_zone, "zone424", 424, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3242", zones.back()->m_zone, "zone138", 138,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3575", zones.back()->m_zone, "zone311", 311, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3576", zones.back()->m_zone, "zone312", 312,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3579", zones.back()->m_zone, "zone316", 316,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3580", zones.back()->m_zone, "zone332", 332,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B360", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3572", zones.back()->m_zone, "zone309", 309,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3577", zones.back()->m_zone, "zone312", 312,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3581", zones.back()->m_zone, "zone317", 317, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3582", zones.back()->m_zone, "zone331", 331, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3583", zones.back()->m_zone, "zone316", 316,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2026", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3570", zones.back()->m_zone, "zone308", 308,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3579", zones.back()->m_zone, "zone314", 314,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3583", zones.back()->m_zone, "zone315", 315,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3584", zones.back()->m_zone, "zone330", 330, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3585", zones.back()->m_zone, "zone318", 318,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B285", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3565", zones.back()->m_zone, "zone306", 306,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3581", zones.back()->m_zone, "zone315", 315, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3586", zones.back()->m_zone, "zone319", 319,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3587", zones.back()->m_zone, "zone329", 329, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3588", zones.back()->m_zone, "zone318", 318,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1950", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3567", zones.back()->m_zone, "zone307", 307,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3585", zones.back()->m_zone, "zone316", 316, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3588", zones.back()->m_zone, "zone317", 317,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3589", zones.back()->m_zone, "zone328", 328, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3590", zones.back()->m_zone, "zone320", 320,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B958", zones.back()->m_zone, "zone705", 705, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3558", zones.back()->m_zone, "zone304", 304, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3586", zones.back()->m_zone, "zone317", 317,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3591", zones.back()->m_zone, "zone326", 326, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3592", zones.back()->m_zone, "zone320", 320,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3593", zones.back()->m_zone, "zone321", 321,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2622", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3562", zones.back()->m_zone, "zone305", 305, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3590", zones.back()->m_zone, "zone318", 318,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3592", zones.back()->m_zone, "zone319", 319,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3594", zones.back()->m_zone, "zone327", 327, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3595", zones.back()->m_zone, "zone322", 322,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B953", zones.back()->m_zone, "zone704", 704, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3556", zones.back()->m_zone, "zone303", 303, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3593", zones.back()->m_zone, "zone319", 319,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3596", zones.back()->m_zone, "zone322", 322,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3597", zones.back()->m_zone, "zone325", 325, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2619", zones.back()->m_zone, "zone1355", 1355,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3553", zones.back()->m_zone, "zone301", 301, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3595", zones.back()->m_zone, "zone320", 320,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3596", zones.back()->m_zone, "zone321", 321,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3598", zones.back()->m_zone, "zone323", 323, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2645", zones.back()->m_zone, "zone1363", 1363,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3598", zones.back()->m_zone, "zone322", 322, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3599", zones.back()->m_zone, "zone334", 334, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3600", zones.back()->m_zone, "zone325", 325,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3601", zones.back()->m_zone, "zone327", 327,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A499", zones.back()->m_zone, "zone839", 839, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A500", zones.back()->m_zone, "zone811", 811, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A501", zones.back()->m_zone, "zone828", 828,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A502", zones.back()->m_zone, "zone844", 844, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A503", zones.back()->m_zone, "zone336", 336,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B980", zones.back()->m_zone, "zone713", 713, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3597", zones.back()->m_zone, "zone321", 321, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3600", zones.back()->m_zone, "zone323", 323,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3602", zones.back()->m_zone, "zone337", 337, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3603", zones.back()->m_zone, "zone326", 326,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B982", zones.back()->m_zone, "zone714", 714, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3591", zones.back()->m_zone, "zone319", 319, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3603", zones.back()->m_zone, "zone325", 325,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3604", zones.back()->m_zone, "zone339", 339, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3605", zones.back()->m_zone, "zone329", 329,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3606", zones.back()->m_zone, "zone327", 327,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2647", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3594", zones.back()->m_zone, "zone320", 320, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3601", zones.back()->m_zone, "zone323", 323,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3606", zones.back()->m_zone, "zone326", 326,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3607", zones.back()->m_zone, "zone328", 328,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3608", zones.back()->m_zone, "zone338", 338, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1947", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3589", zones.back()->m_zone, "zone318", 318, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3607", zones.back()->m_zone, "zone327", 327,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3609", zones.back()->m_zone, "zone330", 330, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3610", zones.back()->m_zone, "zone341", 341, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3611", zones.back()->m_zone, "zone329", 329,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B282", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3587", zones.back()->m_zone, "zone317", 317, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3605", zones.back()->m_zone, "zone326", 326,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3611", zones.back()->m_zone, "zone328", 328,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3612", zones.back()->m_zone, "zone340", 340, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3613", zones.back()->m_zone, "zone331", 331, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2023", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3584", zones.back()->m_zone, "zone316", 316, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3609", zones.back()->m_zone, "zone328", 328,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3614", zones.back()->m_zone, "zone332", 332, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3615", zones.back()->m_zone, "zone343", 343, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3616", zones.back()->m_zone, "zone331", 331,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B359", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 8, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3582", zones.back()->m_zone, "zone315", 315, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3613", zones.back()->m_zone, "zone329", 329, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3616", zones.back()->m_zone, "zone330", 330,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3617", zones.back()->m_zone, "zone342", 342, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3618", zones.back()->m_zone, "zone333", 333,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3231", zones.back()->m_zone, "zone131", 131, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3580", zones.back()->m_zone, "zone314", 314,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3614", zones.back()->m_zone, "zone330", 330,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3619", zones.back()->m_zone, "zone345", 345, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3620", zones.back()->m_zone, "zone333", 333,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1566", zones.back()->m_zone, "zone975", 975, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3578", zones.back()->m_zone, "zone312", 312,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3618", zones.back()->m_zone, "zone331", 331, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3620", zones.back()->m_zone, "zone332", 332,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3621", zones.back()->m_zone, "zone344", 344, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2638", zones.back()->m_zone, "zone1361", 1361,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3599", zones.back()->m_zone, "zone323", 323, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3622", zones.back()->m_zone, "zone355", 355, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3623", zones.back()->m_zone, "zone337", 337,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3624", zones.back()->m_zone, "zone338", 338,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B52", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A59", zones.back()->m_zone, "zone576", 576,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A60", zones.back()->m_zone, "zone446", 446, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A61", zones.back()->m_zone, "zone688", 688,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A62", zones.back()->m_zone, "zone685", 685, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A63", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B15", zones.back()->m_zone, "zone720", 720, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B503", zones.back()->m_zone, "zone324", 324,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A504", zones.back()->m_zone, "zone840", 840, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A505", zones.back()->m_zone, "zone845", 845, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A506", zones.back()->m_zone, "zone827", 827,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A507", zones.back()->m_zone, "zone812", 812, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B973", zones.back()->m_zone, "zone711", 711, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3602", zones.back()->m_zone, "zone325", 325, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3623", zones.back()->m_zone, "zone334", 334,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3625", zones.back()->m_zone, "zone356", 356, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3626", zones.back()->m_zone, "zone339", 339,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2643", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3608", zones.back()->m_zone, "zone327", 327, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3624", zones.back()->m_zone, "zone334", 334,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3627", zones.back()->m_zone, "zone341", 341,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3628", zones.back()->m_zone, "zone339", 339,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3629", zones.back()->m_zone, "zone354", 354, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B978", zones.back()->m_zone, "zone712", 712, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3604", zones.back()->m_zone, "zone326", 326, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3626", zones.back()->m_zone, "zone337", 337,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3628", zones.back()->m_zone, "zone338", 338,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3630", zones.back()->m_zone, "zone353", 353, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3631", zones.back()->m_zone, "zone340", 340,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B277", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3612", zones.back()->m_zone, "zone329", 329, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3631", zones.back()->m_zone, "zone339", 339,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3632", zones.back()->m_zone, "zone352", 352, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3633", zones.back()->m_zone, "zone341", 341,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3634", zones.back()->m_zone, "zone342", 342, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1942", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3610", zones.back()->m_zone, "zone328", 328, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3627", zones.back()->m_zone, "zone338", 338,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3633", zones.back()->m_zone, "zone340", 340,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3635", zones.back()->m_zone, "zone343", 343, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3636", zones.back()->m_zone, "zone351", 351, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B354", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3617", zones.back()->m_zone, "zone331", 331, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3634", zones.back()->m_zone, "zone340", 340, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3637", zones.back()->m_zone, "zone350", 350, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3638", zones.back()->m_zone, "zone343", 343,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3639", zones.back()->m_zone, "zone344", 344,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2020", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3615", zones.back()->m_zone, "zone330", 330, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3635", zones.back()->m_zone, "zone341", 341,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3638", zones.back()->m_zone, "zone342", 342,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3640", zones.back()->m_zone, "zone345", 345, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3641", zones.back()->m_zone, "zone349", 349, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1563", zones.back()->m_zone, "zone974", 974, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3621", zones.back()->m_zone, "zone333", 333, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3639", zones.back()->m_zone, "zone342", 342, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3642", zones.back()->m_zone, "zone348", 348,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3643", zones.back()->m_zone, "zone345", 345,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3228", zones.back()->m_zone, "zone130", 130, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3619", zones.back()->m_zone, "zone332", 332, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3640", zones.back()->m_zone, "zone343", 343,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3643", zones.back()->m_zone, "zone344", 344,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3644", zones.back()->m_zone, "zone346", 346,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3224", zones.back()->m_zone, "zone128", 128,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3644", zones.back()->m_zone, "zone345", 345,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3645", zones.back()->m_zone, "zone349", 349,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3646", zones.back()->m_zone, "zone348", 348,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3647", zones.back()->m_zone, "zone367", 367,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A508", zones.back()->m_zone, "zone825", 825,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A509", zones.back()->m_zone, "zone855", 855, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A510", zones.back()->m_zone, "zone1034", 1034,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A511", zones.back()->m_zone, "zone922", 922, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A512", zones.back()->m_zone, "zone813", 813,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A513", zones.back()->m_zone, "zone358", 358,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1559", zones.back()->m_zone, "zone972", 972,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3642", zones.back()->m_zone, "zone344", 344,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3646", zones.back()->m_zone, "zone346", 346,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3648", zones.back()->m_zone, "zone350", 350,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3649", zones.back()->m_zone, "zone366", 366,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2017", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3641", zones.back()->m_zone, "zone343", 343,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3645", zones.back()->m_zone, "zone346", 346,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3650", zones.back()->m_zone, "zone365", 365,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3651", zones.back()->m_zone, "zone350", 350,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3652", zones.back()->m_zone, "zone351", 351,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B351", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3637", zones.back()->m_zone, "zone342", 342,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3648", zones.back()->m_zone, "zone348", 348,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3651", zones.back()->m_zone, "zone349", 349,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3653", zones.back()->m_zone, "zone352", 352, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3654", zones.back()->m_zone, "zone364", 364,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1938", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3636", zones.back()->m_zone, "zone341", 341,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3652", zones.back()->m_zone, "zone349", 349, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3655", zones.back()->m_zone, "zone362", 362,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3656", zones.back()->m_zone, "zone352", 352,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3657", zones.back()->m_zone, "zone354", 354,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B273", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3632", zones.back()->m_zone, "zone340", 340,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3653", zones.back()->m_zone, "zone350", 350, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3656", zones.back()->m_zone, "zone351", 351,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3658", zones.back()->m_zone, "zone353", 353,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3659", zones.back()->m_zone, "zone363", 363,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1060", zones.back()->m_zone, "zone750", 750, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3630", zones.back()->m_zone, "zone339", 339,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3658", zones.back()->m_zone, "zone352", 352,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3660", zones.back()->m_zone, "zone361", 361, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3661", zones.back()->m_zone, "zone354", 354,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3662", zones.back()->m_zone, "zone356", 356,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2724", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3629", zones.back()->m_zone, "zone338", 338, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3657", zones.back()->m_zone, "zone351", 351,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3661", zones.back()->m_zone, "zone353", 353,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3663", zones.back()->m_zone, "zone360", 360,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3664", zones.back()->m_zone, "zone355", 355,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2722", zones.back()->m_zone, "zone1400", 1400,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3622", zones.back()->m_zone, "zone334", 334,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3664", zones.back()->m_zone, "zone354", 354,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3665", zones.back()->m_zone, "zone357", 357, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3666", zones.back()->m_zone, "zone356", 356,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1056", zones.back()->m_zone, "zone749", 749, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3625", zones.back()->m_zone, "zone337", 337, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3662", zones.back()->m_zone, "zone353", 353,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3666", zones.back()->m_zone, "zone355", 355,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3667", zones.back()->m_zone, "zone359", 359,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2727", zones.back()->m_zone, "zone1402", 1402,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3665", zones.back()->m_zone, "zone355", 355,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3668", zones.back()->m_zone, "zone368", 368, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3669", zones.back()->m_zone, "zone359", 359,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3670", zones.back()->m_zone, "zone360", 360,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B513", zones.back()->m_zone, "zone347", 347,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A514", zones.back()->m_zone, "zone826", 826,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A515", zones.back()->m_zone, "zone814", 814,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A516", zones.back()->m_zone, "zone823", 823, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A517", zones.back()->m_zone, "zone853", 853,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1061", zones.back()->m_zone, "zone751", 751, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3667", zones.back()->m_zone, "zone356", 356, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3669", zones.back()->m_zone, "zone357", 357,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3671", zones.back()->m_zone, "zone370", 370,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3672", zones.back()->m_zone, "zone361", 361,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2729", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3663", zones.back()->m_zone, "zone354", 354, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3670", zones.back()->m_zone, "zone357", 357,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3673", zones.back()->m_zone, "zone362", 362,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3674", zones.back()->m_zone, "zone361", 361,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3675", zones.back()->m_zone, "zone372", 372,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1065", zones.back()->m_zone, "zone752", 752, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3660", zones.back()->m_zone, "zone353", 353,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3672", zones.back()->m_zone, "zone359", 359,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3674", zones.back()->m_zone, "zone360", 360,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3676", zones.back()->m_zone, "zone371", 371, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3677", zones.back()->m_zone, "zone363", 363,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1894", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3655", zones.back()->m_zone, "zone351", 351,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3673", zones.back()->m_zone, "zone360", 360,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3678", zones.back()->m_zone, "zone363", 363,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3679", zones.back()->m_zone, "zone365", 365, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3680", zones.back()->m_zone, "zone373", 373,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B229", zones.back()->m_zone, "zone954", 954, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3659", zones.back()->m_zone, "zone352", 352,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3677", zones.back()->m_zone, "zone361", 361,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3678", zones.back()->m_zone, "zone362", 362,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3681", zones.back()->m_zone, "zone364", 364, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3682", zones.back()->m_zone, "zone374", 374,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B319", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3654", zones.back()->m_zone, "zone350", 350,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3681", zones.back()->m_zone, "zone363", 363,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3683", zones.back()->m_zone, "zone366", 366, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3684", zones.back()->m_zone, "zone365", 365,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3685", zones.back()->m_zone, "zone375", 375,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1983", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3650", zones.back()->m_zone, "zone349", 349,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3679", zones.back()->m_zone, "zone362", 362, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3684", zones.back()->m_zone, "zone364", 364,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3686", zones.back()->m_zone, "zone367", 367,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3687", zones.back()->m_zone, "zone376", 376,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1555", zones.back()->m_zone, "zone970", 970, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3649", zones.back()->m_zone, "zone348", 348, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3683", zones.back()->m_zone, "zone364", 364,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3688", zones.back()->m_zone, "zone367", 367,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3689", zones.back()->m_zone, "zone378", 378,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3220", zones.back()->m_zone, "zone126", 126, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3647", zones.back()->m_zone, "zone346", 346, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3686", zones.back()->m_zone, "zone365", 365, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3688", zones.back()->m_zone, "zone366", 366,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3690", zones.back()->m_zone, "zone377", 377,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2732", zones.back()->m_zone, "zone1404", 1404,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3668", zones.back()->m_zone, "zone357", 357,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3691", zones.back()->m_zone, "zone381", 381, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3692", zones.back()->m_zone, "zone370", 370,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3693", zones.back()->m_zone, "zone372", 372,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B471", zones.back()->m_zone, "zone236", 236, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A518", zones.back()->m_zone, "zone585", 585, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A519", zones.back()->m_zone, "zone391", 391,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A520", zones.back()->m_zone, "zone514", 514, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A521", zones.back()->m_zone, "zone380", 380,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1066", zones.back()->m_zone, "zone753", 753,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3671", zones.back()->m_zone, "zone359", 359, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3692", zones.back()->m_zone, "zone368", 368,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3694", zones.back()->m_zone, "zone379", 379,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3695", zones.back()->m_zone, "zone371", 371,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1070", zones.back()->m_zone, "zone755", 755,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3676", zones.back()->m_zone, "zone361", 361,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3695", zones.back()->m_zone, "zone370", 370,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3696", zones.back()->m_zone, "zone382", 382, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3697", zones.back()->m_zone, "zone374", 374,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3698", zones.back()->m_zone, "zone372", 372,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2734", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3675", zones.back()->m_zone, "zone360", 360, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3693", zones.back()->m_zone, "zone368", 368,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3698", zones.back()->m_zone, "zone371", 371,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3699", zones.back()->m_zone, "zone373", 373,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3700", zones.back()->m_zone, "zone383", 383,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1875", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3680", zones.back()->m_zone, "zone362", 362, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3699", zones.back()->m_zone, "zone372", 372,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3701", zones.back()->m_zone, "zone374", 374,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3702", zones.back()->m_zone, "zone376", 376, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3703", zones.back()->m_zone, "zone385", 385,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B210", zones.back()->m_zone, "zone909", 909, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3682", zones.back()->m_zone, "zone363", 363, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3697", zones.back()->m_zone, "zone371", 371,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3701", zones.back()->m_zone, "zone373", 373,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3704", zones.back()->m_zone, "zone375", 375, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3705", zones.back()->m_zone, "zone384", 384,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B303", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3685", zones.back()->m_zone, "zone364", 364, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3704", zones.back()->m_zone, "zone374", 374,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3706", zones.back()->m_zone, "zone378", 378, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3707", zones.back()->m_zone, "zone376", 376,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3708", zones.back()->m_zone, "zone386", 386,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1968", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3687", zones.back()->m_zone, "zone365", 365, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3702", zones.back()->m_zone, "zone373", 373, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3707", zones.back()->m_zone, "zone375", 375,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3709", zones.back()->m_zone, "zone377", 377,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3710", zones.back()->m_zone, "zone387", 387,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B3222", zones.back()->m_zone, "zone127", 127,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3690", zones.back()->m_zone, "zone367", 367, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3709", zones.back()->m_zone, "zone376", 376, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3711", zones.back()->m_zone, "zone378", 378,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3712", zones.back()->m_zone, "zone388", 388,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1557", zones.back()->m_zone, "zone971", 971,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3689", zones.back()->m_zone, "zone366", 366, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3706", zones.back()->m_zone, "zone375", 375,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3711", zones.back()->m_zone, "zone377", 377,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3713", zones.back()->m_zone, "zone389", 389,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1072", zones.back()->m_zone, "zone756", 756,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3694", zones.back()->m_zone, "zone370", 370, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3714", zones.back()->m_zone, "zone381", 381,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3715", zones.back()->m_zone, "zone393", 393,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3716", zones.back()->m_zone, "zone382", 382,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B128", zones.back()->m_zone, "zone698", 698, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B476", zones.back()->m_zone, "zone247", 247, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B521", zones.back()->m_zone, "zone369", 369,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A522", zones.back()->m_zone, "zone511", 511, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A523", zones.back()->m_zone, "zone402", 402,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A524", zones.back()->m_zone, "zone586", 586, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2737", zones.back()->m_zone, "zone1406", 1406,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3691", zones.back()->m_zone, "zone368", 368,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3714", zones.back()->m_zone, "zone379", 379,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3717", zones.back()->m_zone, "zone394", 394, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3718", zones.back()->m_zone, "zone383", 383,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1075", zones.back()->m_zone, "zone757", 757,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B3696", zones.back()->m_zone, "zone371", 371,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3716", zones.back()->m_zone, "zone379", 379,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3719", zones.back()->m_zone, "zone395", 395, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3720", zones.back()->m_zone, "zone384", 384,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3721", zones.back()->m_zone, "zone383", 383,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2740", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3700", zones.back()->m_zone, "zone372", 372, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3718", zones.back()->m_zone, "zone381", 381,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3721", zones.back()->m_zone, "zone382", 382,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3722", zones.back()->m_zone, "zone385", 385,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3723", zones.back()->m_zone, "zone396", 396,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B445", zones.back()->m_zone, "zone158", 158, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3705", zones.back()->m_zone, "zone374", 374, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3720", zones.back()->m_zone, "zone382", 382,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3724", zones.back()->m_zone, "zone386", 386, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3725", zones.back()->m_zone, "zone385", 385,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3726", zones.back()->m_zone, "zone400", 400,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2110", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3703", zones.back()->m_zone, "zone373", 373, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3722", zones.back()->m_zone, "zone383", 383,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3725", zones.back()->m_zone, "zone384", 384,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3727", zones.back()->m_zone, "zone387", 387, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3728", zones.back()->m_zone, "zone397", 397,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B449", zones.back()->m_zone, "zone180", 180,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3708", zones.back()->m_zone, "zone375", 375, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3724", zones.back()->m_zone, "zone384", 384,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3729", zones.back()->m_zone, "zone389", 389, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3730", zones.back()->m_zone, "zone387", 387,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3731", zones.back()->m_zone, "zone399", 399,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2115", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3710", zones.back()->m_zone, "zone376", 376, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3727", zones.back()->m_zone, "zone385", 385, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3730", zones.back()->m_zone, "zone386", 386,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3732", zones.back()->m_zone, "zone388", 388,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3733", zones.back()->m_zone, "zone398", 398,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3147", zones.back()->m_zone, "zone92", 92, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3712", zones.back()->m_zone, "zone377", 377, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3732", zones.back()->m_zone, "zone387", 387, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3734", zones.back()->m_zone, "zone389", 389,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3735", zones.back()->m_zone, "zone390", 390,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1482", zones.back()->m_zone, "zone936", 936,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3713", zones.back()->m_zone, "zone378", 378, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3729", zones.back()->m_zone, "zone386", 386,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3734", zones.back()->m_zone, "zone388", 388,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3736", zones.back()->m_zone, "zone392", 392,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3128", zones.back()->m_zone, "zone83", 83, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3735", zones.back()->m_zone, "zone388", 388, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3737", zones.back()->m_zone, "zone398", 398, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3738", zones.back()->m_zone, "zone392", 392,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3739", zones.back()->m_zone, "zone420", 420,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B519", zones.back()->m_zone, "zone369", 369,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A525", zones.back()->m_zone, "zone556", 556, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A526", zones.back()->m_zone, "zone504", 504, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A527", zones.back()->m_zone, "zone575", 575, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A528", zones.back()->m_zone, "zone402", 402,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1463", zones.back()->m_zone, "zone927", 927, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3736", zones.back()->m_zone, "zone389", 389, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3738", zones.back()->m_zone, "zone390", 390,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3740", zones.back()->m_zone, "zone399", 399,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3741", zones.back()->m_zone, "zone419", 419,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1182", zones.back()->m_zone, "zone793", 793, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3715", zones.back()->m_zone, "zone379", 379, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3742", zones.back()->m_zone, "zone394", 394,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3743", zones.back()->m_zone, "zone403", 403, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3744", zones.back()->m_zone, "zone395", 395,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2848", zones.back()->m_zone, "zone1444", 1444,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3717", zones.back()->m_zone, "zone381", 381, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3742", zones.back()->m_zone, "zone393", 393,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3745", zones.back()->m_zone, "zone401", 401, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3746", zones.back()->m_zone, "zone396", 396,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1186", zones.back()->m_zone, "zone794", 794, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3719", zones.back()->m_zone, "zone382", 382, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3744", zones.back()->m_zone, "zone393", 393,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3747", zones.back()->m_zone, "zone404", 404, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3748", zones.back()->m_zone, "zone400", 400,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3749", zones.back()->m_zone, "zone396", 396,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2850", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3723", zones.back()->m_zone, "zone383", 383, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3746", zones.back()->m_zone, "zone394", 394,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3749", zones.back()->m_zone, "zone395", 395,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3750", zones.back()->m_zone, "zone397", 397,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3751", zones.back()->m_zone, "zone405", 405, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2106", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3728", zones.back()->m_zone, "zone385", 385, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3750", zones.back()->m_zone, "zone396", 396,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3752", zones.back()->m_zone, "zone400", 400,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3753", zones.back()->m_zone, "zone398", 398, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3754", zones.back()->m_zone, "zone408", 408, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2112", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3733", zones.back()->m_zone, "zone387", 387, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3737", zones.back()->m_zone, "zone390", 390,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3753", zones.back()->m_zone, "zone397", 397, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3755", zones.back()->m_zone, "zone399", 399,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3756", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B448", zones.back()->m_zone, "zone169", 169, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3731", zones.back()->m_zone, "zone386", 386, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3740", zones.back()->m_zone, "zone392", 392, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3755", zones.back()->m_zone, "zone398", 398,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3757", zones.back()->m_zone, "zone400", 400,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3758", zones.back()->m_zone, "zone417", 417,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B441", zones.back()->m_zone, "zone147", 147, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3726", zones.back()->m_zone, "zone384", 384, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 31, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3748", zones.back()->m_zone, "zone395", 395,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3752", zones.back()->m_zone, "zone397", 397,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3757", zones.back()->m_zone, "zone399", 399, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3759", zones.back()->m_zone, "zone410", 410, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2853", zones.back()->m_zone, "zone1446", 1446,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3745", zones.back()->m_zone, "zone394", 394, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3760", zones.back()->m_zone, "zone406", 406,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3761", zones.back()->m_zone, "zone403", 403,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3762", zones.back()->m_zone, "zone405", 405,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B122", zones.back()->m_zone, "zone687", 687, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B523", zones.back()->m_zone, "zone380", 380,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B528", zones.back()->m_zone, "zone391", 391,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A529", zones.back()->m_zone, "zone557", 557, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A530", zones.back()->m_zone, "zone577", 577, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A531", zones.back()->m_zone, "zone501", 501, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1188", zones.back()->m_zone, "zone795", 795, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3743", zones.back()->m_zone, "zone393", 393, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3761", zones.back()->m_zone, "zone401", 401,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3763", zones.back()->m_zone, "zone407", 407,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3764", zones.back()->m_zone, "zone404", 404,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1191", zones.back()->m_zone, "zone796", 796, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3747", zones.back()->m_zone, "zone395", 395, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3764", zones.back()->m_zone, "zone403", 403,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3765", zones.back()->m_zone, "zone411", 411,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3766", zones.back()->m_zone, "zone410", 410,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3767", zones.back()->m_zone, "zone405", 405,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2856", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3751", zones.back()->m_zone, "zone396", 396, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3762", zones.back()->m_zone, "zone401", 401,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3767", zones.back()->m_zone, "zone404", 404,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3768", zones.back()->m_zone, "zone408", 408,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3769", zones.back()->m_zone, "zone409", 409,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2838", zones.back()->m_zone, "zone1439", 1439,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3760", zones.back()->m_zone, "zone401", 401,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3770", zones.back()->m_zone, "zone412", 412,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3771", zones.back()->m_zone, "zone407", 407, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3772", zones.back()->m_zone, "zone409", 409,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1171", zones.back()->m_zone, "zone789", 789, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3763", zones.back()->m_zone, "zone403", 403,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3771", zones.back()->m_zone, "zone406", 406, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3773", zones.back()->m_zone, "zone414", 414,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3774", zones.back()->m_zone, "zone411", 411,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1736", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3754", zones.back()->m_zone, "zone397", 397, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3768", zones.back()->m_zone, "zone405", 405,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3775", zones.back()->m_zone, "zone410", 410,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3776", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3777", zones.back()->m_zone, "zone409", 409,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B2841", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3769", zones.back()->m_zone, "zone405", 405,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3772", zones.back()->m_zone, "zone406", 406,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3777", zones.back()->m_zone, "zone408", 408,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3778", zones.back()->m_zone, "zone411", 411, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3779", zones.back()->m_zone, "zone415", 415,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B71", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3759", zones.back()->m_zone, "zone400", 400, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3766", zones.back()->m_zone, "zone404", 404,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3775", zones.back()->m_zone, "zone408", 408,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3780", zones.back()->m_zone, "zone417", 417,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3781", zones.back()->m_zone, "zone411", 411,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1178", zones.back()->m_zone, "zone790", 790, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3765", zones.back()->m_zone, "zone404", 404,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3774", zones.back()->m_zone, "zone407", 407,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3778", zones.back()->m_zone, "zone409", 409, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3781", zones.back()->m_zone, "zone410", 410,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3782", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2846", zones.back()->m_zone, "zone1443", 1443,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3770", zones.back()->m_zone, "zone406", 406,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3783", zones.back()->m_zone, "zone422", 422, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3784", zones.back()->m_zone, "zone414", 414,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3785", zones.back()->m_zone, "zone415", 415,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B493", zones.back()->m_zone, "zone302", 302, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A532", zones.back()->m_zone, "zone835", 835, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A533", zones.back()->m_zone, "zone693", 693,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A534", zones.back()->m_zone, "zone562", 562, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A535", zones.back()->m_zone, "zone424", 424,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1181", zones.back()->m_zone, "zone792", 792,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3773", zones.back()->m_zone, "zone407", 407,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3784", zones.back()->m_zone, "zone412", 412,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3786", zones.back()->m_zone, "zone421", 421,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3787", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2844", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3779", zones.back()->m_zone, "zone409", 409,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3785", zones.back()->m_zone, "zone412", 412,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3788", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3789", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3790", zones.back()->m_zone, "zone425", 425,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1741", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3756", zones.back()->m_zone, "zone398", 398,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3776", zones.back()->m_zone, "zone408", 408,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3788", zones.back()->m_zone, "zone415", 415,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3791", zones.back()->m_zone, "zone420", 420, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3792", zones.back()->m_zone, "zone417", 417,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B77", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3758", zones.back()->m_zone, "zone399", 399,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3780", zones.back()->m_zone, "zone410", 410,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3792", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3793", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3794", zones.back()->m_zone, "zone419", 419,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1179", zones.back()->m_zone, "zone791", 791,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3782", zones.back()->m_zone, "zone411", 411,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3787", zones.back()->m_zone, "zone414", 414,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3789", zones.back()->m_zone, "zone415", 415,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3793", zones.back()->m_zone, "zone417", 417,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3795", zones.back()->m_zone, "zone423", 423, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1467", zones.back()->m_zone, "zone928", 928, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3741", zones.back()->m_zone, "zone392", 392,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3794", zones.back()->m_zone, "zone417", 417, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3796", zones.back()->m_zone, "zone427", 427,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3797", zones.back()->m_zone, "zone420", 420,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3132", zones.back()->m_zone, "zone84", 84, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3739", zones.back()->m_zone, "zone390", 390,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3791", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3797", zones.back()->m_zone, "zone419", 419,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3798", zones.back()->m_zone, "zone426", 426,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1127", zones.back()->m_zone, "zone772", 772,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3786", zones.back()->m_zone, "zone414", 414, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3799", zones.back()->m_zone, "zone422", 422,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3800", zones.back()->m_zone, "zone428", 428,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3801", zones.back()->m_zone, "zone423", 423,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2792", zones.back()->m_zone, "zone1423", 1423,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3783", zones.back()->m_zone, "zone412", 412,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3799", zones.back()->m_zone, "zone421", 421,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3802", zones.back()->m_zone, "zone429", 429, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3803", zones.back()->m_zone, "zone425", 425,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1122", zones.back()->m_zone, "zone771", 771,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3795", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3801", zones.back()->m_zone, "zone421", 421,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3804", zones.back()->m_zone, "zone431", 431, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3805", zones.back()->m_zone, "zone427", 427, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3806", zones.back()->m_zone, "zone425", 425,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B35", zones.back()->m_zone, "zone1164", 1164,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B498", zones.back()->m_zone, "zone313", 313, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B535", zones.back()->m_zone, "zone413", 413,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A536", zones.back()->m_zone, "zone563", 563, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A537", zones.back()->m_zone, "zone694", 694,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A538", zones.back()->m_zone, "zone836", 836, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2787", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3790", zones.back()->m_zone, "zone415", 415, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3803", zones.back()->m_zone, "zone422", 422,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3806", zones.back()->m_zone, "zone423", 423,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3807", zones.back()->m_zone, "zone426", 426,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3808", zones.back()->m_zone, "zone430", 430,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3264", zones.back()->m_zone, "zone151", 151, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3798", zones.back()->m_zone, "zone420", 420,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3807", zones.back()->m_zone, "zone425", 425,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3809", zones.back()->m_zone, "zone433", 433,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3810", zones.back()->m_zone, "zone427", 427,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1599", zones.back()->m_zone, "zone995", 995, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3796", zones.back()->m_zone, "zone419", 419,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3805", zones.back()->m_zone, "zone423", 423,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3810", zones.back()->m_zone, "zone426", 426,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3811", zones.back()->m_zone, "zone432", 432,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1130", zones.back()->m_zone, "zone773", 773, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3800", zones.back()->m_zone, "zone421", 421, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3812", zones.back()->m_zone, "zone429", 429,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3813", zones.back()->m_zone, "zone436", 436, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3814", zones.back()->m_zone, "zone431", 431,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2796", zones.back()->m_zone, "zone1424", 1424,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3802", zones.back()->m_zone, "zone422", 422, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3812", zones.back()->m_zone, "zone428", 428,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3815", zones.back()->m_zone, "zone434", 434, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3816", zones.back()->m_zone, "zone430", 430,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2799", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3808", zones.back()->m_zone, "zone425", 425, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3816", zones.back()->m_zone, "zone429", 429,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3817", zones.back()->m_zone, "zone433", 433,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3818", zones.back()->m_zone, "zone431", 431,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3819", zones.back()->m_zone, "zone438", 438, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1135", zones.back()->m_zone, "zone774", 774, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3804", zones.back()->m_zone, "zone423", 423, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3814", zones.back()->m_zone, "zone428", 428,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3818", zones.back()->m_zone, "zone430", 430,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3820", zones.back()->m_zone, "zone437", 437, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3821", zones.back()->m_zone, "zone432", 432,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1601", zones.back()->m_zone, "zone996", 996,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3811", zones.back()->m_zone, "zone427", 427,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3821", zones.back()->m_zone, "zone431", 431,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3822", zones.back()->m_zone, "zone433", 433,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3823", zones.back()->m_zone, "zone440", 440, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3266", zones.back()->m_zone, "zone152", 152,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3809", zones.back()->m_zone, "zone426", 426,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3817", zones.back()->m_zone, "zone430", 430,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3822", zones.back()->m_zone, "zone432", 432,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3824", zones.back()->m_zone, "zone439", 439, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2811", zones.back()->m_zone, "zone1428", 1428,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3815", zones.back()->m_zone, "zone429", 429,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3825", zones.back()->m_zone, "zone441", 441, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3826", zones.back()->m_zone, "zone436", 436,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3827", zones.back()->m_zone, "zone438", 438,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A539", zones.back()->m_zone, "zone734", 734,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A540", zones.back()->m_zone, "zone458", 458, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A541", zones.back()->m_zone, "zone524", 524,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A542", zones.back()->m_zone, "zone689", 689, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A543", zones.back()->m_zone, "zone447", 447,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1146", zones.back()->m_zone, "zone778", 778, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3813", zones.back()->m_zone, "zone428", 428, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3826", zones.back()->m_zone, "zone434", 434,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3828", zones.back()->m_zone, "zone442", 442,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3829", zones.back()->m_zone, "zone437", 437,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1150", zones.back()->m_zone, "zone779", 779, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3820", zones.back()->m_zone, "zone431", 431,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3829", zones.back()->m_zone, "zone436", 436,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3830", zones.back()->m_zone, "zone443", 443, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3831", zones.back()->m_zone, "zone440", 440,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3832", zones.back()->m_zone, "zone438", 438,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2815", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3819", zones.back()->m_zone, "zone430", 430, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3827", zones.back()->m_zone, "zone434", 434,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3832", zones.back()->m_zone, "zone437", 437,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3833", zones.back()->m_zone, "zone439", 439,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3834", zones.back()->m_zone, "zone444", 444,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3261", zones.back()->m_zone, "zone150", 150, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3824", zones.back()->m_zone, "zone433", 433, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3833", zones.back()->m_zone, "zone438", 438,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3835", zones.back()->m_zone, "zone440", 440,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3836", zones.back()->m_zone, "zone451", 451,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1596", zones.back()->m_zone, "zone994", 994, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3823", zones.back()->m_zone, "zone432", 432, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3831", zones.back()->m_zone, "zone437", 437,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3835", zones.back()->m_zone, "zone439", 439,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3837", zones.back()->m_zone, "zone449", 449,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2802", zones.back()->m_zone, "zone1426", 1426,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3825", zones.back()->m_zone, "zone434", 434,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3838", zones.back()->m_zone, "zone453", 453, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3839", zones.back()->m_zone, "zone442", 442,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3840", zones.back()->m_zone, "zone444", 444,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1138", zones.back()->m_zone, "zone775", 775, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3828", zones.back()->m_zone, "zone436", 436, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3839", zones.back()->m_zone, "zone441", 441,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3841", zones.back()->m_zone, "zone454", 454,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3842", zones.back()->m_zone, "zone443", 443,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1143", zones.back()->m_zone, "zone777", 777, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3830", zones.back()->m_zone, "zone437", 437,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3842", zones.back()->m_zone, "zone442", 442,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3843", zones.back()->m_zone, "zone448", 448,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3844", zones.back()->m_zone, "zone449", 449,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3845", zones.back()->m_zone, "zone444", 444,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2808", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3834", zones.back()->m_zone, "zone438", 438, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3840", zones.back()->m_zone, "zone441", 441,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3845", zones.back()->m_zone, "zone443", 443,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3846", zones.back()->m_zone, "zone451", 451,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3847", zones.back()->m_zone, "zone445", 445,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3064", zones.back()->m_zone, "zone51", 51, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3847", zones.back()->m_zone, "zone444", 444,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3848", zones.back()->m_zone, "zone452", 452,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3849", zones.back()->m_zone, "zone456", 456, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3850", zones.back()->m_zone, "zone448", 448,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B57", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B60", zones.back()->m_zone, "zone335", 335, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A64", zones.back()->m_zone, "zone610", 610,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A65", zones.back()->m_zone, "zone988", 988, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A66", zones.back()->m_zone, "zone649", 649, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A67", zones.back()->m_zone, "zone136", 136, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B410", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B543", zones.back()->m_zone, "zone435", 435,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A544", zones.back()->m_zone, "zone735", 735,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A545", zones.back()->m_zone, "zone690", 690, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A546", zones.back()->m_zone, "zone535", 535,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A547", zones.back()->m_zone, "zone469", 469, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1399", zones.back()->m_zone, "zone895", 895,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3843", zones.back()->m_zone, "zone443", 443,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3850", zones.back()->m_zone, "zone445", 445,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3851", zones.back()->m_zone, "zone450", 450, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3852", zones.back()->m_zone, "zone455", 455, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1590", zones.back()->m_zone, "zone991", 991,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3837", zones.back()->m_zone, "zone440", 440, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3844", zones.back()->m_zone, "zone443", 443,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3853", zones.back()->m_zone, "zone451", 451,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3854", zones.back()->m_zone, "zone450", 450,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1587", zones.back()->m_zone, "zone990", 990,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3851", zones.back()->m_zone, "zone448", 448,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3854", zones.back()->m_zone, "zone449", 449, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3855", zones.back()->m_zone, "zone452", 452,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3255", zones.back()->m_zone, "zone146", 146,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 11, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3836", zones.back()->m_zone, "zone439", 439, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3846", zones.back()->m_zone, "zone444", 444,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3853", zones.back()->m_zone, "zone449", 449,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3856", zones.back()->m_zone, "zone452", 452,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B3252", zones.back()->m_zone, "zone145", 145,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3848", zones.back()->m_zone, "zone445", 445,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3855", zones.back()->m_zone, "zone450", 450,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3856", zones.back()->m_zone, "zone451", 451, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2774", zones.back()->m_zone, "zone1418", 1418,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3838", zones.back()->m_zone, "zone441", 441,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3857", zones.back()->m_zone, "zone457", 457, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3858", zones.back()->m_zone, "zone454", 454,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3859", zones.back()->m_zone, "zone456", 456,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1110", zones.back()->m_zone, "zone768", 768, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3841", zones.back()->m_zone, "zone442", 442, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3858", zones.back()->m_zone, "zone453", 453,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3860", zones.back()->m_zone, "zone459", 459,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3861", zones.back()->m_zone, "zone455", 455,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1402", zones.back()->m_zone, "zone896", 896,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3852", zones.back()->m_zone, "zone448", 448, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3861", zones.back()->m_zone, "zone454", 454, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3862", zones.back()->m_zone, "zone461", 461,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3863", zones.back()->m_zone, "zone456", 456,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3067", zones.back()->m_zone, "zone52", 52, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3849", zones.back()->m_zone, "zone445", 445, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3859", zones.back()->m_zone, "zone453", 453,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3863", zones.back()->m_zone, "zone455", 455,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3864", zones.back()->m_zone, "zone460", 460,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2771", zones.back()->m_zone, "zone1417", 1417,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3857", zones.back()->m_zone, "zone453", 453,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3865", zones.back()->m_zone, "zone463", 463,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3866", zones.back()->m_zone, "zone459", 459,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3867", zones.back()->m_zone, "zone460", 460,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B540", zones.back()->m_zone, "zone435", 435, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A548", zones.back()->m_zone, "zone731", 731, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A549", zones.back()->m_zone, "zone736", 736,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A550", zones.back()->m_zone, "zone502", 502, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A551", zones.back()->m_zone, "zone469", 469,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1106", zones.back()->m_zone, "zone767", 767,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3860", zones.back()->m_zone, "zone454", 454, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3866", zones.back()->m_zone, "zone457", 457,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3868", zones.back()->m_zone, "zone462", 462,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3869", zones.back()->m_zone, "zone461", 461,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3069", zones.back()->m_zone, "zone53", 53, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3864", zones.back()->m_zone, "zone456", 456, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3867", zones.back()->m_zone, "zone457", 457,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3870", zones.back()->m_zone, "zone465", 465,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3871", zones.back()->m_zone, "zone461", 461,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1404", zones.back()->m_zone, "zone897", 897, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3862", zones.back()->m_zone, "zone455", 455, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3869", zones.back()->m_zone, "zone459", 459, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3871", zones.back()->m_zone, "zone460", 460,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A3872", zones.back()->m_zone, "zone464", 464,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1103", zones.back()->m_zone, "zone766", 766, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3868", zones.back()->m_zone, "zone459", 459,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3873", zones.back()->m_zone, "zone463", 463,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3874", zones.back()->m_zone, "zone472", 472,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3875", zones.back()->m_zone, "zone464", 464,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2768", zones.back()->m_zone, "zone1416", 1416,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3865", zones.back()->m_zone, "zone457", 457,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3873", zones.back()->m_zone, "zone462", 462,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3876", zones.back()->m_zone, "zone471", 471,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3877", zones.back()->m_zone, "zone465", 465,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1405", zones.back()->m_zone, "zone899", 899, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3872", zones.back()->m_zone, "zone461", 461,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3875", zones.back()->m_zone, "zone462", 462,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3878", zones.back()->m_zone, "zone468", 468, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3879", zones.back()->m_zone, "zone465", 465,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3070", zones.back()->m_zone, "zone54", 54, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3870", zones.back()->m_zone, "zone460", 460,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B3877", zones.back()->m_zone, "zone463", 463,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3879", zones.back()->m_zone, "zone464", 464,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3880", zones.back()->m_zone, "zone466", 466, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3051", zones.back()->m_zone, "zone45", 45, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3880", zones.back()->m_zone, "zone465", 465,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3881", zones.back()->m_zone, "zone467", 467, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3882", zones.back()->m_zone, "zone471", 471, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3883", zones.back()->m_zone, "zone468", 468,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3050", zones.back()->m_zone, "zone44", 44, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3881", zones.back()->m_zone, "zone466", 466, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3884", zones.back()->m_zone, "zone474", 474,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3885", zones.back()->m_zone, "zone478", 478,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3886", zones.back()->m_zone, "zone470", 470,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1386", zones.back()->m_zone, "zone890", 890,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3878", zones.back()->m_zone, "zone464", 464,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3883", zones.back()->m_zone, "zone466", 466,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3887", zones.back()->m_zone, "zone470", 470, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A3888", zones.back()->m_zone, "zone472", 472,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B415", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B547", zones.back()->m_zone, "zone447", 447, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B551", zones.back()->m_zone, "zone458", 458,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A552", zones.back()->m_zone, "zone513", 513, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A553", zones.back()->m_zone, "zone737", 737,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A554", zones.back()->m_zone, "zone733", 733, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1385", zones.back()->m_zone, "zone889", 889, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3886", zones.back()->m_zone, "zone467", 467,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3887", zones.back()->m_zone, "zone468", 468, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3889", zones.back()->m_zone, "zone473", 473,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3890", zones.back()->m_zone, "zone477", 477,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2751", zones.back()->m_zone, "zone1411", 1411,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3876", zones.back()->m_zone, "zone463", 463,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3882", zones.back()->m_zone, "zone466", 466,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3891", zones.back()->m_zone, "zone476", 476,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3892", zones.back()->m_zone, "zone472", 472,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1086", zones.back()->m_zone, "zone760", 760, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3874", zones.back()->m_zone, "zone462", 462,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3888", zones.back()->m_zone, "zone468", 468,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3892", zones.back()->m_zone, "zone471", 471,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3893", zones.back()->m_zone, "zone475", 475,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1380", zones.back()->m_zone, "zone886", 886, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3889", zones.back()->m_zone, "zone470", 470,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3894", zones.back()->m_zone, "zone474", 474,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3895", zones.back()->m_zone, "zone488", 488,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3896", zones.back()->m_zone, "zone484", 484,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3045", zones.back()->m_zone, "zone42", 42, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3884", zones.back()->m_zone, "zone467", 467,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3894", zones.back()->m_zone, "zone473", 473,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3897", zones.back()->m_zone, "zone489", 489, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3898", zones.back()->m_zone, "zone485", 485,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1097", zones.back()->m_zone, "zone763", 763, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3893", zones.back()->m_zone, "zone472", 472,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3899", zones.back()->m_zone, "zone476", 476, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3900", zones.back()->m_zone, "zone483", 483,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3901", zones.back()->m_zone, "zone477", 477,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2763", zones.back()->m_zone, "zone1414", 1414,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3891", zones.back()->m_zone, "zone471", 471,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3899", zones.back()->m_zone, "zone475", 475, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3902", zones.back()->m_zone, "zone482", 482,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3903", zones.back()->m_zone, "zone478", 478,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1102", zones.back()->m_zone, "zone764", 764, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3890", zones.back()->m_zone, "zone470", 470, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3901", zones.back()->m_zone, "zone475", 475,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3904", zones.back()->m_zone, "zone488", 488,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3905", zones.back()->m_zone, "zone479", 479,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3906", zones.back()->m_zone, "zone478", 478, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2766", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3885", zones.back()->m_zone, "zone467", 467,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B3903", zones.back()->m_zone, "zone476", 476,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3906", zones.back()->m_zone, "zone477", 477, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3907", zones.back()->m_zone, "zone489", 489,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3908", zones.back()->m_zone, "zone481", 481,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1212", zones.back()->m_zone, "zone807", 807, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3905", zones.back()->m_zone, "zone477", 477,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3909", zones.back()->m_zone, "zone494", 494,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3910", zones.back()->m_zone, "zone481", 481, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3911", zones.back()->m_zone, "zone490", 490,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3912", zones.back()->m_zone, "zone483", 483,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A555", zones.back()->m_zone, "zone747", 747, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A556", zones.back()->m_zone, "zone753", 753, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A557", zones.back()->m_zone, "zone502", 502,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A558", zones.back()->m_zone, "zone736", 736, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A559", zones.back()->m_zone, "zone491", 491,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B2877", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3908", zones.back()->m_zone, "zone478", 478,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3910", zones.back()->m_zone, "zone479", 479, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3913", zones.back()->m_zone, "zone493", 493,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3914", zones.back()->m_zone, "zone492", 492,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3915", zones.back()->m_zone, "zone482", 482,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2874", zones.back()->m_zone, "zone1457", 1457,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3902", zones.back()->m_zone, "zone476", 476,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3915", zones.back()->m_zone, "zone481", 481,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3916", zones.back()->m_zone, "zone496", 496,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3917", zones.back()->m_zone, "zone483", 483, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1209", zones.back()->m_zone, "zone806", 806, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3900", zones.back()->m_zone, "zone475", 475,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3912", zones.back()->m_zone, "zone479", 479,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3917", zones.back()->m_zone, "zone482", 482, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3918", zones.back()->m_zone, "zone495", 495,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1413", zones.back()->m_zone, "zone903", 903,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3896", zones.back()->m_zone, "zone473", 473,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3919", zones.back()->m_zone, "zone498", 498, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3920", zones.back()->m_zone, "zone486", 486,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3921", zones.back()->m_zone, "zone485", 485,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3078", zones.back()->m_zone, "zone59", 59, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3898", zones.back()->m_zone, "zone474", 474,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3921", zones.back()->m_zone, "zone484", 484,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3922", zones.back()->m_zone, "zone487", 487,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3923", zones.back()->m_zone, "zone497", 497,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1415", zones.back()->m_zone, "zone904", 904,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3920", zones.back()->m_zone, "zone484", 484,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3924", zones.back()->m_zone, "zone487", 487,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3925", zones.back()->m_zone, "zone507", 507,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3926", zones.back()->m_zone, "zone533", 533,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B3080", zones.back()->m_zone, "zone60", 60,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3922", zones.back()->m_zone, "zone485", 485,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3924", zones.back()->m_zone, "zone486", 486,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3927", zones.back()->m_zone, "zone506", 506,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3928", zones.back()->m_zone, "zone532", 532,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B162", zones.back()->m_zone, "zone787", 787, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3895", zones.back()->m_zone, "zone473", 473, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3904", zones.back()->m_zone, "zone477", 477,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A3929", zones.back()->m_zone, "zone489", 489,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3930", zones.back()->m_zone, "zone490", 490,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3931", zones.back()->m_zone, "zone498", 498,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1827", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3897", zones.back()->m_zone, "zone474", 474,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3907", zones.back()->m_zone, "zone478", 478,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3929", zones.back()->m_zone, "zone488", 488,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3932", zones.back()->m_zone, "zone492", 492,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3933", zones.back()->m_zone, "zone497", 497,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B100", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3911", zones.back()->m_zone, "zone479", 479,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3930", zones.back()->m_zone, "zone488", 488,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3934", zones.back()->m_zone, "zone494", 494,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3935", zones.back()->m_zone, "zone500", 500, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3936", zones.back()->m_zone, "zone492", 492,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B205", zones.back()->m_zone, "zone898", 898, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B559", zones.back()->m_zone, "zone480", 480,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A560", zones.back()->m_zone, "zone748", 748, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A561", zones.back()->m_zone, "zone737", 737, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A562", zones.back()->m_zone, "zone513", 513,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A563", zones.back()->m_zone, "zone755", 755, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1765", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 5, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3914", zones.back()->m_zone, "zone481", 481,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3932", zones.back()->m_zone, "zone489", 489,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3936", zones.back()->m_zone, "zone490", 490,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3937", zones.back()->m_zone, "zone499", 499, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3938", zones.back()->m_zone, "zone493", 493,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2880", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3913", zones.back()->m_zone, "zone481", 481,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3938", zones.back()->m_zone, "zone492", 492,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3939", zones.back()->m_zone, "zone494", 494,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3940", zones.back()->m_zone, "zone503", 503, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3941", zones.back()->m_zone, "zone496", 496,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1215", zones.back()->m_zone, "zone810", 810, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3909", zones.back()->m_zone, "zone479", 479,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3934", zones.back()->m_zone, "zone490", 490,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3939", zones.back()->m_zone, "zone493", 493,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3942", zones.back()->m_zone, "zone501", 501, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3943", zones.back()->m_zone, "zone495", 495,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1213", zones.back()->m_zone, "zone808", 808, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3918", zones.back()->m_zone, "zone483", 483,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3943", zones.back()->m_zone, "zone494", 494,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3944", zones.back()->m_zone, "zone496", 496,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3945", zones.back()->m_zone, "zone504", 504, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2878", zones.back()->m_zone, "zone1459", 1459,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3916", zones.back()->m_zone, "zone482", 482,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3941", zones.back()->m_zone, "zone493", 493,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3944", zones.back()->m_zone, "zone495", 495,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3946", zones.back()->m_zone, "zone505", 505, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1837", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3923", zones.back()->m_zone, "zone485", 485, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3933", zones.back()->m_zone, "zone489", 489,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3947", zones.back()->m_zone, "zone506", 506, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3948", zones.back()->m_zone, "zone498", 498,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3949", zones.back()->m_zone, "zone499", 499,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B173", zones.back()->m_zone, "zone820", 820,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3919", zones.back()->m_zone, "zone484", 484,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3931", zones.back()->m_zone, "zone488", 488,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3948", zones.back()->m_zone, "zone497", 497,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3950", zones.back()->m_zone, "zone500", 500, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3951", zones.back()->m_zone, "zone507", 507, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1791", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3937", zones.back()->m_zone, "zone492", 492, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3949", zones.back()->m_zone, "zone497", 497, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3952", zones.back()->m_zone, "zone508", 508, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3953", zones.back()->m_zone, "zone500", 500,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3954", zones.back()->m_zone, "zone503", 503,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B126", zones.back()->m_zone, "zone687", 687, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3935", zones.back()->m_zone, "zone490", 490, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3950", zones.back()->m_zone, "zone498", 498, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3953", zones.back()->m_zone, "zone499", 499,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3955", zones.back()->m_zone, "zone501", 501,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3956", zones.back()->m_zone, "zone509", 509, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B531", zones.back()->m_zone, "zone402", 402, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3942", zones.back()->m_zone, "zone494", 494, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3955", zones.back()->m_zone, "zone500", 500,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3957", zones.back()->m_zone, "zone511", 511, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3958", zones.back()->m_zone, "zone503", 503,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3959", zones.back()->m_zone, "zone504", 504,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B550", zones.back()->m_zone, "zone458", 458, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B557", zones.back()->m_zone, "zone480", 480,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A564", zones.back()->m_zone, "zone524", 524, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A565", zones.back()->m_zone, "zone756", 756, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A566", zones.back()->m_zone, "zone513", 513,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2195", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3940", zones.back()->m_zone, "zone493", 493, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3954", zones.back()->m_zone, "zone499", 499,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3958", zones.back()->m_zone, "zone501", 501,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3960", zones.back()->m_zone, "zone510", 510, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3961", zones.back()->m_zone, "zone505", 505,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B526", zones.back()->m_zone, "zone391", 391, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3945", zones.back()->m_zone, "zone495", 495, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3959", zones.back()->m_zone, "zone501", 501,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3962", zones.back()->m_zone, "zone505", 505,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3963", zones.back()->m_zone, "zone514", 514, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2192", zones.back()->m_zone, "zone1189", 1189,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3946", zones.back()->m_zone, "zone496", 496, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3961", zones.back()->m_zone, "zone503", 503,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3962", zones.back()->m_zone, "zone504", 504,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3964", zones.back()->m_zone, "zone512", 512, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1844", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3927", zones.back()->m_zone, "zone487", 487,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3947", zones.back()->m_zone, "zone497", 497, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3965", zones.back()->m_zone, "zone528", 528, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3966", zones.back()->m_zone, "zone507", 507,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3967", zones.back()->m_zone, "zone508", 508,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B179", zones.back()->m_zone, "zone832", 832, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3925", zones.back()->m_zone, "zone486", 486,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3951", zones.back()->m_zone, "zone498", 498, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3966", zones.back()->m_zone, "zone506", 506,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 8, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3968", zones.back()->m_zone, "zone509", 509, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3969", zones.back()->m_zone, "zone529", 529, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1794", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3952", zones.back()->m_zone, "zone499", 499, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3967", zones.back()->m_zone, "zone506", 506, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3970", zones.back()->m_zone, "zone520", 520, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3971", zones.back()->m_zone, "zone509", 509,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3972", zones.back()->m_zone, "zone510", 510,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B129", zones.back()->m_zone, "zone698", 698, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3956", zones.back()->m_zone, "zone500", 500, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3968", zones.back()->m_zone, "zone507", 507, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3971", zones.back()->m_zone, "zone508", 508,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3973", zones.back()->m_zone, "zone511", 511,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3974", zones.back()->m_zone, "zone519", 519, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2189", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3960", zones.back()->m_zone, "zone503", 503, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3972", zones.back()->m_zone, "zone508", 508,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3975", zones.back()->m_zone, "zone511", 511,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3976", zones.back()->m_zone, "zone517", 517, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3977", zones.back()->m_zone, "zone512", 512,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B522", zones.back()->m_zone, "zone380", 380, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3957", zones.back()->m_zone, "zone501", 501, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3973", zones.back()->m_zone, "zone509", 509,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3975", zones.back()->m_zone, "zone510", 510,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3978", zones.back()->m_zone, "zone518", 518, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3979", zones.back()->m_zone, "zone514", 514,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2183", zones.back()->m_zone, "zone1186", 1186,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3964", zones.back()->m_zone, "zone505", 505, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3977", zones.back()->m_zone, "zone510", 510,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3980", zones.back()->m_zone, "zone516", 516, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3981", zones.back()->m_zone, "zone514", 514,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B430", zones.back()->m_zone, "zone102", 102, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B552", zones.back()->m_zone, "zone469", 469, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B562", zones.back()->m_zone, "zone491", 491,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B566", zones.back()->m_zone, "zone502", 502,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A567", zones.back()->m_zone, "zone535", 535, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A568", zones.back()->m_zone, "zone757", 757, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B520", zones.back()->m_zone, "zone369", 369, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3963", zones.back()->m_zone, "zone504", 504, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3979", zones.back()->m_zone, "zone511", 511,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3981", zones.back()->m_zone, "zone512", 512,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3982", zones.back()->m_zone, "zone515", 515, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1270", zones.back()->m_zone, "zone833", 833, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3982", zones.back()->m_zone, "zone514", 514, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3983", zones.back()->m_zone, "zone516", 516,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3984", zones.back()->m_zone, "zone523", 523,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3985", zones.back()->m_zone, "zone518", 518,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2935", zones.back()->m_zone, "zone1482", 1482,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3980", zones.back()->m_zone, "zone512", 512, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3983", zones.back()->m_zone, "zone515", 515,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3986", zones.back()->m_zone, "zone525", 525,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3987", zones.back()->m_zone, "zone517", 517,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2937", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3976", zones.back()->m_zone, "zone510", 510, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3987", zones.back()->m_zone, "zone516", 516,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3988", zones.back()->m_zone, "zone520", 520,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3989", zones.back()->m_zone, "zone518", 518,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3990", zones.back()->m_zone, "zone521", 521,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1272", zones.back()->m_zone, "zone834", 834, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3978", zones.back()->m_zone, "zone511", 511, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3985", zones.back()->m_zone, "zone515", 515,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3989", zones.back()->m_zone, "zone517", 517,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3991", zones.back()->m_zone, "zone522", 522,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A3992", zones.back()->m_zone, "zone519", 519,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B153", zones.back()->m_zone, "zone765", 765, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3974", zones.back()->m_zone, "zone509", 509, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3992", zones.back()->m_zone, "zone518", 518,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3993", zones.back()->m_zone, "zone522", 522,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3994", zones.back()->m_zone, "zone520", 520,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3995", zones.back()->m_zone, "zone529", 529, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1819", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 5, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3970", zones.back()->m_zone, "zone508", 508, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3988", zones.back()->m_zone, "zone517", 517,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3994", zones.back()->m_zone, "zone519", 519,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3996", zones.back()->m_zone, "zone528", 528, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A3997", zones.back()->m_zone, "zone521", 521,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B2908", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3990", zones.back()->m_zone, "zone517", 517,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B3997", zones.back()->m_zone, "zone520", 520,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A3998", zones.back()->m_zone, "zone522", 522, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A3999", zones.back()->m_zone, "zone530", 530,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4000", zones.back()->m_zone, "zone525", 525,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1243", zones.back()->m_zone, "zone819", 819, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3991", zones.back()->m_zone, "zone518", 518,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3993", zones.back()->m_zone, "zone519", 519,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3998", zones.back()->m_zone, "zone521", 521, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A4001", zones.back()->m_zone, "zone531", 531,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4002", zones.back()->m_zone, "zone523", 523,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1240", zones.back()->m_zone, "zone818", 818, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3984", zones.back()->m_zone, "zone515", 515,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4002", zones.back()->m_zone, "zone522", 522,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4003", zones.back()->m_zone, "zone525", 525, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A4004", zones.back()->m_zone, "zone527", 527,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B541", zones.back()->m_zone, "zone435", 435,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B564", zones.back()->m_zone, "zone502", 502, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A569", zones.back()->m_zone, "zone546", 546, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A570", zones.back()->m_zone, "zone684", 684, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A571", zones.back()->m_zone, "zone535", 535,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2905", zones.back()->m_zone, "zone1469", 1469,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B3986", zones.back()->m_zone, "zone516", 516,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4000", zones.back()->m_zone, "zone521", 521,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4003", zones.back()->m_zone, "zone523", 523, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A4005", zones.back()->m_zone, "zone526", 526,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2898", zones.back()->m_zone, "zone1467", 1467,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4005", zones.back()->m_zone, "zone525", 525,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A4006", zones.back()->m_zone, "zone539", 539,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4007", zones.back()->m_zone, "zone527", 527, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4008", zones.back()->m_zone, "zone530", 530,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1234", zones.back()->m_zone, "zone816", 816, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4004", zones.back()->m_zone, "zone523", 523,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4007", zones.back()->m_zone, "zone526", 526, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4009", zones.back()->m_zone, "zone540", 540,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4010", zones.back()->m_zone, "zone531", 531,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1858", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3965", zones.back()->m_zone, "zone506", 506, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3996", zones.back()->m_zone, "zone520", 520,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4011", zones.back()->m_zone, "zone532", 532, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4012", zones.back()->m_zone, "zone530", 530,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4013", zones.back()->m_zone, "zone529", 529,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B192", zones.back()->m_zone, "zone876", 876, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3969", zones.back()->m_zone, "zone507", 507, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3995", zones.back()->m_zone, "zone519", 519, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4013", zones.back()->m_zone, "zone528", 528,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4014", zones.back()->m_zone, "zone531", 531,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4015", zones.back()->m_zone, "zone533", 533,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2903", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B3999", zones.back()->m_zone, "zone521", 521,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B4008", zones.back()->m_zone, "zone526", 526,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4012", zones.back()->m_zone, "zone528", 528,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4016", zones.back()->m_zone, "zone536", 536, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4017", zones.back()->m_zone, "zone531", 531, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1237", zones.back()->m_zone, "zone817", 817, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4001", zones.back()->m_zone, "zone522", 522,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B4010", zones.back()->m_zone, "zone527", 527,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4014", zones.back()->m_zone, "zone529", 529,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4017", zones.back()->m_zone, "zone530", 530, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4018", zones.back()->m_zone, "zone534", 534,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3087", zones.back()->m_zone, "zone62", 62, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3928", zones.back()->m_zone, "zone487", 487,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4011", zones.back()->m_zone, "zone528", 528,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4019", zones.back()->m_zone, "zone536", 536,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4020", zones.back()->m_zone, "zone533", 533,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1422", zones.back()->m_zone, "zone906", 906, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3926", zones.back()->m_zone, "zone486", 486,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4015", zones.back()->m_zone, "zone529", 529, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4020", zones.back()->m_zone, "zone532", 532,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4021", zones.back()->m_zone, "zone534", 534,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1431", zones.back()->m_zone, "zone913", 913, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B4018", zones.back()->m_zone, "zone531", 531,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4021", zones.back()->m_zone, "zone533", 533,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4022", zones.back()->m_zone, "zone537", 537,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4023", zones.back()->m_zone, "zone536", 536,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B427", zones.back()->m_zone, "zone91", 91, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B546", zones.back()->m_zone, "zone447", 447,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B567", zones.back()->m_zone, "zone513", 513, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B571", zones.back()->m_zone, "zone524", 524,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A572", zones.back()->m_zone, "zone555", 555, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A573", zones.back()->m_zone, "zone685", 685, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3096", zones.back()->m_zone, "zone68", 68, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B4016", zones.back()->m_zone, "zone530", 530,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4019", zones.back()->m_zone, "zone532", 532,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4023", zones.back()->m_zone, "zone534", 534,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A4024", zones.back()->m_zone, "zone538", 538,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1436", zones.back()->m_zone, "zone915", 915,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4022", zones.back()->m_zone, "zone534", 534,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A4025", zones.back()->m_zone, "zone538", 538,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4026", zones.back()->m_zone, "zone541", 541,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4027", zones.back()->m_zone, "zone540", 540,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3101", zones.back()->m_zone, "zone71", 71, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4024", zones.back()->m_zone, "zone536", 536,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B4025", zones.back()->m_zone, "zone537", 537,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4028", zones.back()->m_zone, "zone539", 539,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4029", zones.back()->m_zone, "zone542", 542,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2895", zones.back()->m_zone, "zone1466", 1466,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4006", zones.back()->m_zone, "zone526", 526,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4028", zones.back()->m_zone, "zone538", 538,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4030", zones.back()->m_zone, "zone544", 544,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4031", zones.back()->m_zone, "zone540", 540,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1230", zones.back()->m_zone, "zone815", 815, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4009", zones.back()->m_zone, "zone527", 527,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4027", zones.back()->m_zone, "zone537", 537,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4031", zones.back()->m_zone, "zone539", 539,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4032", zones.back()->m_zone, "zone543", 543,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1440", zones.back()->m_zone, "zone917", 917, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4026", zones.back()->m_zone, "zone537", 537,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4033", zones.back()->m_zone, "zone543", 543,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4034", zones.back()->m_zone, "zone545", 545,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A4035", zones.back()->m_zone, "zone542", 542,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3105", zones.back()->m_zone, "zone73", 73, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4029", zones.back()->m_zone, "zone538", 538,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4035", zones.back()->m_zone, "zone541", 541,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A4036", zones.back()->m_zone, "zone548", 548,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A4037", zones.back()->m_zone, "zone544", 544,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1244", zones.back()->m_zone, "zone821", 821, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4032", zones.back()->m_zone, "zone540", 540,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4033", zones.back()->m_zone, "zone541", 541,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4038", zones.back()->m_zone, "zone544", 544,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4039", zones.back()->m_zone, "zone550", 550,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2909", zones.back()->m_zone, "zone1471", 1471,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4030", zones.back()->m_zone, "zone539", 539,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4037", zones.back()->m_zone, "zone542", 542,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4038", zones.back()->m_zone, "zone543", 543,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4040", zones.back()->m_zone, "zone552", 552,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1443", zones.back()->m_zone, "zone918", 918, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4034", zones.back()->m_zone, "zone541", 541,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4041", zones.back()->m_zone, "zone547", 547, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A4042", zones.back()->m_zone, "zone550", 550,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4043", zones.back()->m_zone, "zone548", 548,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B569", zones.back()->m_zone, "zone524", 524, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A574", zones.back()->m_zone, "zone793", 793,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A575", zones.back()->m_zone, "zone800", 800, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A576", zones.back()->m_zone, "zone680", 680, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A577", zones.back()->m_zone, "zone555", 555,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1446", zones.back()->m_zone, "zone919", 919,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3336", zones.back()->m_zone, "zone197", 197, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B4041", zones.back()->m_zone, "zone545", 545,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4044", zones.back()->m_zone, "zone551", 551,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4045", zones.back()->m_zone, "zone549", 549,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3108", zones.back()->m_zone, "zone74", 74, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4036", zones.back()->m_zone, "zone542", 542,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4043", zones.back()->m_zone, "zone545", 545,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4046", zones.back()->m_zone, "zone549", 549, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4047", zones.back()->m_zone, "zone552", 552, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3111", zones.back()->m_zone, "zone75", 75, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3340", zones.back()->m_zone, "zone198", 198, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B4045", zones.back()->m_zone, "zone547", 547,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4046", zones.back()->m_zone, "zone548", 548,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4048", zones.back()->m_zone, "zone553", 553, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1248", zones.back()->m_zone, "zone822", 822,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4039", zones.back()->m_zone, "zone543", 543,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4042", zones.back()->m_zone, "zone545", 545,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A4049", zones.back()->m_zone, "zone552", 552,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4050", zones.back()->m_zone, "zone551", 551,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1305", zones.back()->m_zone, "zone852", 852, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B3345", zones.back()->m_zone, "zone200", 200,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4044", zones.back()->m_zone, "zone547", 547,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B4050", zones.back()->m_zone, "zone550", 550, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A4051", zones.back()->m_zone, "zone553", 553,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2913", zones.back()->m_zone, "zone1472", 1472,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4040", zones.back()->m_zone, "zone544", 544,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B4047", zones.back()->m_zone, "zone548", 548,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B4049", zones.back()->m_zone, "zone550", 550,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A4052", zones.back()->m_zone, "zone553", 553, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2971", zones.back()->m_zone, "zone8", 8, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3343", zones.back()->m_zone, "zone199", 199, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4048", zones.back()->m_zone, "zone549", 549,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B4051", zones.back()->m_zone, "zone551", 551,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B4052", zones.back()->m_zone, "zone552", 552,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A68", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A69", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A70", zones.back()->m_zone, "zone147", 147, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A71", zones.back()->m_zone, "zone410", 410, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A72", zones.back()->m_zone, "zone796", 796, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A73", zones.back()->m_zone, "zone790", 790, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B423", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B572", zones.back()->m_zone, "zone535", 535, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B577", zones.back()->m_zone, "zone546", 546,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A578", zones.back()->m_zone, "zone794", 794,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A579", zones.back()->m_zone, "zone681", 681, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A580", zones.back()->m_zone, "zone801", 801, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B525", zones.back()->m_zone, "zone391", 391, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A581", zones.back()->m_zone, "zone682", 682, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A582", zones.back()->m_zone, "zone808", 808, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A583", zones.back()->m_zone, "zone573", 573, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A584", zones.back()->m_zone, "zone557", 557,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B118", zones.back()->m_zone, "zone676", 676,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B529", zones.back()->m_zone, "zone402", 402, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B584", zones.back()->m_zone, "zone556", 556,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A585", zones.back()->m_zone, "zone683", 683, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A586", zones.back()->m_zone, "zone574", 574, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A587", zones.back()->m_zone, "zone810", 810, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A588", zones.back()->m_zone, "zone624", 624, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A589", zones.back()->m_zone, "zone653", 653, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A590", zones.back()->m_zone, "zone620", 620,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A591", zones.back()->m_zone, "zone614", 614, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A592", zones.back()->m_zone, "zone559", 559,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B592", zones.back()->m_zone, "zone558", 558,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A593", zones.back()->m_zone, "zone623", 623, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A594", zones.back()->m_zone, "zone859", 859, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A595", zones.back()->m_zone, "zone1023", 1023,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A596", zones.back()->m_zone, "zone622", 622,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A597", zones.back()->m_zone, "zone655", 655, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B494", zones.back()->m_zone, "zone302", 302, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A598", zones.back()->m_zone, "zone562", 562, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A599", zones.back()->m_zone, "zone633", 633,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A600", zones.back()->m_zone, "zone571", 571, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A601", zones.back()->m_zone, "zone561", 561,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B24", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B497", zones.back()->m_zone, "zone313", 313, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B601", zones.back()->m_zone, "zone560", 560,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A602", zones.back()->m_zone, "zone572", 572, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A603", zones.back()->m_zone, "zone631", 631,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A604", zones.back()->m_zone, "zone563", 563, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B534", zones.back()->m_zone, "zone413", 413, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B598", zones.back()->m_zone, "zone560", 560, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A605", zones.back()->m_zone, "zone695", 695, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A606", zones.back()->m_zone, "zone637", 637,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A607", zones.back()->m_zone, "zone563", 563,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B36", zones.back()->m_zone, "zone1164", 1164,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B536", zones.back()->m_zone, "zone424", 424, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B604", zones.back()->m_zone, "zone561", 561, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B607", zones.back()->m_zone, "zone562", 562,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A608", zones.back()->m_zone, "zone636", 636,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A609", zones.back()->m_zone, "zone696", 696, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("A610", zones.back()->m_zone, "zone671", 671,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A611", zones.back()->m_zone, "zone567", 567, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A612", zones.back()->m_zone, "zone1003", 1003,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{51, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A613", zones.back()->m_zone, "zone647", 647,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A614", zones.back()->m_zone, "zone666", 666,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A615", zones.back()->m_zone, "zone566", 566,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B69", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A74", zones.back()->m_zone, "zone598", 598, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A75", zones.back()->m_zone, "zone928", 928, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A76", zones.back()->m_zone, "zone169", 169, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A77", zones.back()->m_zone, "zone417", 417, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A78", zones.back()->m_zone, "zone791", 791, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B615", zones.back()->m_zone, "zone564", 564,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A616", zones.back()->m_zone, "zone672", 672,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A617", zones.back()->m_zone, "zone664", 664, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A618", zones.back()->m_zone, "zone648", 648,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A619", zones.back()->m_zone, "zone568", 568, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B94", zones.back()->m_zone, "zone610", 610, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B611", zones.back()->m_zone, "zone564", 564, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A620", zones.back()->m_zone, "zone673", 673, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A621", zones.back()->m_zone, "zone679", 679, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A622", zones.back()->m_zone, "zone649", 649, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A623", zones.back()->m_zone, "zone568", 568,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B619", zones.back()->m_zone, "zone566", 566, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B623", zones.back()->m_zone, "zone567", 567,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A624", zones.back()->m_zone, "zone674", 674, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A625", zones.back()->m_zone, "zone650", 650, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A626", zones.back()->m_zone, "zone678", 678,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A627", zones.back()->m_zone, "zone682", 682, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A628", zones.back()->m_zone, "zone573", 573, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A629", zones.back()->m_zone, "zone674", 674, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A630", zones.back()->m_zone, "zone678", 678, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A631", zones.back()->m_zone, "zone570", 570,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B112", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B631", zones.back()->m_zone, "zone569", 569,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A632", zones.back()->m_zone, "zone683", 683, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A633", zones.back()->m_zone, "zone679", 679, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A634", zones.back()->m_zone, "zone673", 673, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A635", zones.back()->m_zone, "zone574", 574, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B488", zones.back()->m_zone, "zone280", 280, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B600", zones.back()->m_zone, "zone560", 560, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A636", zones.back()->m_zone, "zone628", 628, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A637", zones.back()->m_zone, "zone590", 590, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A638", zones.back()->m_zone, "zone572", 572,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B135", zones.back()->m_zone, "zone709", 709, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B491", zones.back()->m_zone, "zone291", 291, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B602", zones.back()->m_zone, "zone561", 561, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B638", zones.back()->m_zone, "zone571", 571,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A639", zones.back()->m_zone, "zone591", 591, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A640", zones.back()->m_zone, "zone627", 627, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B583", zones.back()->m_zone, "zone556", 556, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B628", zones.back()->m_zone, "zone569", 569, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A641", zones.back()->m_zone, "zone575", 575, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A642", zones.back()->m_zone, "zone579", 579,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A643", zones.back()->m_zone, "zone574", 574,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B120", zones.back()->m_zone, "zone676", 676, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B586", zones.back()->m_zone, "zone557", 557, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B635", zones.back()->m_zone, "zone570", 570, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B643", zones.back()->m_zone, "zone573", 573,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A644", zones.back()->m_zone, "zone580", 580,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A645", zones.back()->m_zone, "zone577", 577, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B527", zones.back()->m_zone, "zone391", 391, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B641", zones.back()->m_zone, "zone573", 573, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A646", zones.back()->m_zone, "zone585", 585, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A647", zones.back()->m_zone, "zone581", 581,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A648", zones.back()->m_zone, "zone577", 577,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B59", zones.back()->m_zone, "zone335", 335,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A79", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A80", zones.back()->m_zone, "zone681", 681, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A81", zones.back()->m_zone, "zone679", 679, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A82", zones.back()->m_zone, "zone610", 610, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A83", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B123", zones.back()->m_zone, "zone687", 687, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B530", zones.back()->m_zone, "zone402", 402, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B645", zones.back()->m_zone, "zone574", 574, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B648", zones.back()->m_zone, "zone575", 575,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A649", zones.back()->m_zone, "zone582", 582,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A650", zones.back()->m_zone, "zone586", 586, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A651", zones.back()->m_zone, "zone606", 606, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A652", zones.back()->m_zone, "zone579", 579, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A653", zones.back()->m_zone, "zone603", 603,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A654", zones.back()->m_zone, "zone604", 604,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A655", zones.back()->m_zone, "zone882", 882,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B642", zones.back()->m_zone, "zone573", 573,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B652", zones.back()->m_zone, "zone578", 578, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A656", zones.back()->m_zone, "zone677", 677,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A657", zones.back()->m_zone, "zone581", 581, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A658", zones.back()->m_zone, "zone580", 580,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B169", zones.back()->m_zone, "zone809", 809,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B644", zones.back()->m_zone, "zone574", 574,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B658", zones.back()->m_zone, "zone579", 579,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A659", zones.back()->m_zone, "zone883", 883, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A660", zones.back()->m_zone, "zone582", 582, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A661", zones.back()->m_zone, "zone675", 675,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B647", zones.back()->m_zone, "zone575", 575,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B657", zones.back()->m_zone, "zone579", 579, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A662", zones.back()->m_zone, "zone603", 603, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A663", zones.back()->m_zone, "zone588", 588, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A664", zones.back()->m_zone, "zone582", 582,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B175", zones.back()->m_zone, "zone820", 820, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B649", zones.back()->m_zone, "zone577", 577,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B660", zones.back()->m_zone, "zone580", 580, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B664", zones.back()->m_zone, "zone581", 581,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A665", zones.back()->m_zone, "zone907", 907,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A666", zones.back()->m_zone, "zone589", 589, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A667", zones.back()->m_zone, "zone603", 603,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A668", zones.back()->m_zone, "zone588", 588, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A669", zones.back()->m_zone, "zone584", 584,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A670", zones.back()->m_zone, "zone601", 601,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A671", zones.back()->m_zone, "zone901", 901,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B669", zones.back()->m_zone, "zone583", 583,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A672", zones.back()->m_zone, "zone597", 597, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A673", zones.back()->m_zone, "zone600", 600,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A674", zones.back()->m_zone, "zone594", 594,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A675", zones.back()->m_zone, "zone902", 902,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B518", zones.back()->m_zone, "zone369", 369, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B646", zones.back()->m_zone, "zone575", 575, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A676", zones.back()->m_zone, "zone590", 590, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A677", zones.back()->m_zone, "zone588", 588,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A678", zones.back()->m_zone, "zone586", 586,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B131", zones.back()->m_zone, "zone698", 698, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B524", zones.back()->m_zone, "zone380", 380, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B650", zones.back()->m_zone, "zone577", 577, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B678", zones.back()->m_zone, "zone585", 585,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A679", zones.back()->m_zone, "zone589", 589,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A680", zones.back()->m_zone, "zone591", 591, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B68", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A84", zones.back()->m_zone, "zone80", 80, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A85", zones.back()->m_zone, "zone598", 598, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A86", zones.back()->m_zone, "zone799", 799, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A87", zones.back()->m_zone, "zone801", 801, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A88", zones.back()->m_zone, "zone632", 632, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{6, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B663", zones.back()->m_zone, "zone581", 581, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B668", zones.back()->m_zone, "zone583", 583, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B677", zones.back()->m_zone, "zone585", 585,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A681", zones.back()->m_zone, "zone594", 594, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A682", zones.back()->m_zone, "zone589", 589,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B178", zones.back()->m_zone, "zone832", 832, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B666", zones.back()->m_zone, "zone582", 582, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B679", zones.back()->m_zone, "zone586", 586,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B682", zones.back()->m_zone, "zone588", 588,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A683", zones.back()->m_zone, "zone908", 908, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A684", zones.back()->m_zone, "zone595", 595, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B470", zones.back()->m_zone, "zone236", 236, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B637", zones.back()->m_zone, "zone571", 571, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B676", zones.back()->m_zone, "zone585", 585, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A685", zones.back()->m_zone, "zone594", 594,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A686", zones.back()->m_zone, "zone591", 591,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B142", zones.back()->m_zone, "zone732", 732,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B477", zones.back()->m_zone, "zone247", 247, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B639", zones.back()->m_zone, "zone572", 572, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B680", zones.back()->m_zone, "zone586", 586, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B686", zones.back()->m_zone, "zone590", 590,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A687", zones.back()->m_zone, "zone595", 595,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B181", zones.back()->m_zone, "zone843", 843,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A688", zones.back()->m_zone, "zone627", 627,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A689", zones.back()->m_zone, "zone595", 595,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A690", zones.back()->m_zone, "zone866", 866,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A691", zones.back()->m_zone, "zone623", 623,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A692", zones.back()->m_zone, "zone593", 593,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B692", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A693", zones.back()->m_zone, "zone628", 628,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A694", zones.back()->m_zone, "zone624", 624,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A695", zones.back()->m_zone, "zone597", 597, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A696", zones.back()->m_zone, "zone594", 594,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B674", zones.back()->m_zone, "zone584", 584, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B681", zones.back()->m_zone, "zone588", 588, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B685", zones.back()->m_zone, "zone590", 590,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B696", zones.back()->m_zone, "zone593", 593,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A697", zones.back()->m_zone, "zone595", 595,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B187", zones.back()->m_zone, "zone854", 854, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B684", zones.back()->m_zone, "zone589", 589, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B687", zones.back()->m_zone, "zone591", 591,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B689", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B697", zones.back()->m_zone, "zone594", 594,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A698", zones.back()->m_zone, "zone910", 910, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A699", zones.back()->m_zone, "zone624", 624,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A700", zones.back()->m_zone, "zone614", 614, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A701", zones.back()->m_zone, "zone599", 599, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A702", zones.back()->m_zone, "zone597", 597, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A703", zones.back()->m_zone, "zone857", 857,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B672", zones.back()->m_zone, "zone584", 584, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B695", zones.back()->m_zone, "zone593", 593, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B702", zones.back()->m_zone, "zone596", 596, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A704", zones.back()->m_zone, "zone600", 600, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A705", zones.back()->m_zone, "zone864", 864,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B74", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B85", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A89", zones.back()->m_zone, "zone114", 114, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A90", zones.back()->m_zone, "zone929", 929, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A91", zones.back()->m_zone, "zone786", 786, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A92", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B701", zones.back()->m_zone, "zone596", 596, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A706", zones.back()->m_zone, "zone614", 614, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A707", zones.back()->m_zone, "zone612", 612, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A708", zones.back()->m_zone, "zone600", 600,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A709", zones.back()->m_zone, "zone874", 874,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B673", zones.back()->m_zone, "zone584", 584,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B704", zones.back()->m_zone, "zone597", 597, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B708", zones.back()->m_zone, "zone599", 599,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A710", zones.back()->m_zone, "zone601", 601,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A711", zones.back()->m_zone, "zone875", 875,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B670", zones.back()->m_zone, "zone583", 583,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B710", zones.back()->m_zone, "zone600", 600,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A712", zones.back()->m_zone, "zone612", 612, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A713", zones.back()->m_zone, "zone602", 602,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A714", zones.back()->m_zone, "zone877", 877,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B713", zones.back()->m_zone, "zone601", 601,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A715", zones.back()->m_zone, "zone604", 604,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A716", zones.back()->m_zone, "zone603", 603,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A717", zones.back()->m_zone, "zone611", 611, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A718", zones.back()->m_zone, "zone878", 878,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B653", zones.back()->m_zone, "zone578", 578,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B662", zones.back()->m_zone, "zone581", 581,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B667", zones.back()->m_zone, "zone583", 583,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B716", zones.back()->m_zone, "zone602", 602,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A719", zones.back()->m_zone, "zone900", 900,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B654", zones.back()->m_zone, "zone578", 578,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B715", zones.back()->m_zone, "zone602", 602,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A720", zones.back()->m_zone, "zone607", 607, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A721", zones.back()->m_zone, "zone606", 606, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A722", zones.back()->m_zone, "zone879", 879,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A723", zones.back()->m_zone, "zone670", 670,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A724", zones.back()->m_zone, "zone606", 606,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A725", zones.back()->m_zone, "zone607", 607,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A726", zones.back()->m_zone, "zone608", 608,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A727", zones.back()->m_zone, "zone870", 870,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B651", zones.back()->m_zone, "zone578", 578,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B721", zones.back()->m_zone, "zone604", 604, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B724", zones.back()->m_zone, "zone605", 605,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A728", zones.back()->m_zone, "zone677", 677, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A729", zones.back()->m_zone, "zone880", 880,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B720", zones.back()->m_zone, "zone604", 604, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B725", zones.back()->m_zone, "zone605", 605,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A730", zones.back()->m_zone, "zone611", 611,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A731", zones.back()->m_zone, "zone608", 608,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A732", zones.back()->m_zone, "zone871", 871,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B726", zones.back()->m_zone, "zone605", 605, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B731", zones.back()->m_zone, "zone607", 607,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A733", zones.back()->m_zone, "zone615", 615,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A734", zones.back()->m_zone, "zone618", 618, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A735", zones.back()->m_zone, "zone867", 867,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B5", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A7", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A8", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A9", zones.back()->m_zone, "zone242", 242, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A10", zones.back()->m_zone, "zone956", 956, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A11", zones.back()->m_zone, "zone846", 846, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B64", zones.back()->m_zone, "zone446", 446,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B82", zones.back()->m_zone, "zone576", 576, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A93", zones.back()->m_zone, "zone114", 114, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A94", zones.back()->m_zone, "zone567", 567, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A95", zones.back()->m_zone, "zone925", 925,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A96", zones.back()->m_zone, "zone665", 665, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B717", zones.back()->m_zone, "zone602", 602, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B730", zones.back()->m_zone, "zone607", 607,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A736", zones.back()->m_zone, "zone612", 612,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A737", zones.back()->m_zone, "zone615", 615,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A738", zones.back()->m_zone, "zone872", 872,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B707", zones.back()->m_zone, "zone599", 599, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B712", zones.back()->m_zone, "zone601", 601, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B736", zones.back()->m_zone, "zone611", 611,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A739", zones.back()->m_zone, "zone613", 613,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A740", zones.back()->m_zone, "zone873", 873,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B739", zones.back()->m_zone, "zone612", 612,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A741", zones.back()->m_zone, "zone615", 615,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A742", zones.back()->m_zone, "zone614", 614,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A743", zones.back()->m_zone, "zone620", 620, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A744", zones.back()->m_zone, "zone860", 860,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B591", zones.back()->m_zone, "zone558", 558, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B700", zones.back()->m_zone, "zone596", 596, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B706", zones.back()->m_zone, "zone599", 599, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B742", zones.back()->m_zone, "zone613", 613,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A745", zones.back()->m_zone, "zone858", 858,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B733", zones.back()->m_zone, "zone608", 608,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B737", zones.back()->m_zone, "zone611", 611,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B741", zones.back()->m_zone, "zone613", 613,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A746", zones.back()->m_zone, "zone616", 616, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A747", zones.back()->m_zone, "zone863", 863,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B746", zones.back()->m_zone, "zone615", 615, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A748", zones.back()->m_zone, "zone620", 620,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A749", zones.back()->m_zone, "zone662", 662, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A750", zones.back()->m_zone, "zone618", 618,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A751", zones.back()->m_zone, "zone617", 617,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B751", zones.back()->m_zone, "zone616", 616,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A752", zones.back()->m_zone, "zone622", 622,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A753", zones.back()->m_zone, "zone862", 862, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A754", zones.back()->m_zone, "zone1006", 1006,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A755", zones.back()->m_zone, "zone619", 619,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A756", zones.back()->m_zone, "zone663", 663, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B734", zones.back()->m_zone, "zone608", 608, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B750", zones.back()->m_zone, "zone616", 616,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A757", zones.back()->m_zone, "zone670", 670, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A758", zones.back()->m_zone, "zone667", 667, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A759", zones.back()->m_zone, "zone619", 619,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B755", zones.back()->m_zone, "zone617", 617,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B759", zones.back()->m_zone, "zone618", 618,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A760", zones.back()->m_zone, "zone669", 669, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A761", zones.back()->m_zone, "zone668", 668, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A762", zones.back()->m_zone, "zone1007", 1007,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A763", zones.back()->m_zone, "zone868", 868,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B590", zones.back()->m_zone, "zone558", 558,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B743", zones.back()->m_zone, "zone613", 613, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B748", zones.back()->m_zone, "zone616", 616,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A764", zones.back()->m_zone, "zone658", 658, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A765", zones.back()->m_zone, "zone622", 622,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A97", zones.back()->m_zone, "zone632", 632, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A98", zones.back()->m_zone, "zone807", 807, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A99", zones.back()->m_zone, "zone810", 810, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A100", zones.back()->m_zone, "zone490", 490, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A101", zones.back()->m_zone, "zone676", 676, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A102", zones.back()->m_zone, "zone787", 787, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 5, 1}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B596", zones.back()->m_zone, "zone559", 559,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B752", zones.back()->m_zone, "zone617", 617,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B765", zones.back()->m_zone, "zone620", 620,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A766", zones.back()->m_zone, "zone659", 659, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A767", zones.back()->m_zone, "zone1024", 1024,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A768", zones.back()->m_zone, "zone861", 861,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B593", zones.back()->m_zone, "zone559", 559, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B691", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A769", zones.back()->m_zone, "zone856", 856, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A770", zones.back()->m_zone, "zone1022", 1022,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A771", zones.back()->m_zone, "zone625", 625,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A772", zones.back()->m_zone, "zone624", 624,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B588", zones.back()->m_zone, "zone558", 558,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B694", zones.back()->m_zone, "zone593", 593,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B699", zones.back()->m_zone, "zone596", 596,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B772", zones.back()->m_zone, "zone623", 623,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A773", zones.back()->m_zone, "zone626", 626, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B771", zones.back()->m_zone, "zone623", 623,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A774", zones.back()->m_zone, "zone629", 629,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A775", zones.back()->m_zone, "zone627", 627,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A776", zones.back()->m_zone, "zone1021", 1021,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A777", zones.back()->m_zone, "zone655", 655,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A778", zones.back()->m_zone, "zone626", 626,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B773", zones.back()->m_zone, "zone624", 624, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B778", zones.back()->m_zone, "zone625", 625,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A779", zones.back()->m_zone, "zone630", 630,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A780", zones.back()->m_zone, "zone653", 653, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A781", zones.back()->m_zone, "zone628", 628, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B139", zones.back()->m_zone, "zone721", 721,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B640", zones.back()->m_zone, "zone572", 572,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B688", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B775", zones.back()->m_zone, "zone625", 625,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A782", zones.back()->m_zone, "zone631", 631,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A783", zones.back()->m_zone, "zone628", 628,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B636", zones.back()->m_zone, "zone571", 571, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B693", zones.back()->m_zone, "zone593", 593,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B781", zones.back()->m_zone, "zone626", 626, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B783", zones.back()->m_zone, "zone627", 627,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A784", zones.back()->m_zone, "zone633", 633,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B774", zones.back()->m_zone, "zone625", 625,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A785", zones.back()->m_zone, "zone652", 652, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A786", zones.back()->m_zone, "zone1015", 1015,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A787", zones.back()->m_zone, "zone634", 634, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A788", zones.back()->m_zone, "zone631", 631,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A789", zones.back()->m_zone, "zone630", 630,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B779", zones.back()->m_zone, "zone626", 626,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B789", zones.back()->m_zone, "zone629", 629,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A790", zones.back()->m_zone, "zone633", 633, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A791", zones.back()->m_zone, "zone635", 635, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A792", zones.back()->m_zone, "zone651", 651,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B30", zones.back()->m_zone, "zone1053", 1053, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B603", zones.back()->m_zone, "zone561", 561,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B782", zones.back()->m_zone, "zone627", 627, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B788", zones.back()->m_zone, "zone629", 629, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A793", zones.back()->m_zone, "zone636", 636, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A794", zones.back()->m_zone, "zone633", 633,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B88", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B97", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 5, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A103", zones.back()->m_zone, "zone803", 803, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A104", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A105", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A106", zones.back()->m_zone, "zone805", 805, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B599", zones.back()->m_zone, "zone560", 560,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B784", zones.back()->m_zone, "zone628", 628, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B790", zones.back()->m_zone, "zone630", 630, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B794", zones.back()->m_zone, "zone631", 631,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A795", zones.back()->m_zone, "zone637", 637, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B787", zones.back()->m_zone, "zone629", 629, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A796", zones.back()->m_zone, "zone652", 652, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A797", zones.back()->m_zone, "zone1014", 1014,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A798", zones.back()->m_zone, "zone638", 638,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A799", zones.back()->m_zone, "zone636", 636,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A800", zones.back()->m_zone, "zone635", 635,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B791", zones.back()->m_zone, "zone630", 630, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B800", zones.back()->m_zone, "zone634", 634,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A801", zones.back()->m_zone, "zone637", 637, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A802", zones.back()->m_zone, "zone639", 639,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A803", zones.back()->m_zone, "zone651", 651, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B40", zones.back()->m_zone, "zone1275", 1275,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B608", zones.back()->m_zone, "zone563", 563,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B793", zones.back()->m_zone, "zone631", 631, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B799", zones.back()->m_zone, "zone634", 634, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A804", zones.back()->m_zone, "zone640", 640, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A805", zones.back()->m_zone, "zone637", 637,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B606", zones.back()->m_zone, "zone562", 562,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B795", zones.back()->m_zone, "zone633", 633, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B801", zones.back()->m_zone, "zone635", 635, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B805", zones.back()->m_zone, "zone636", 636,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A806", zones.back()->m_zone, "zone641", 641, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B798", zones.back()->m_zone, "zone634", 634,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A807", zones.back()->m_zone, "zone642", 642, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A808", zones.back()->m_zone, "zone640", 640, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A809", zones.back()->m_zone, "zone1013", 1013,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A810", zones.back()->m_zone, "zone657", 657, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A811", zones.back()->m_zone, "zone639", 639,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B802", zones.back()->m_zone, "zone635", 635,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B811", zones.back()->m_zone, "zone638", 638,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A812", zones.back()->m_zone, "zone644", 644, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A813", zones.back()->m_zone, "zone656", 656, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A814", zones.back()->m_zone, "zone641", 641, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B49", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B804", zones.back()->m_zone, "zone636", 636, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B808", zones.back()->m_zone, "zone638", 638, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A815", zones.back()->m_zone, "zone645", 645, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A816", zones.back()->m_zone, "zone696", 696, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A817", zones.back()->m_zone, "zone641", 641,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B806", zones.back()->m_zone, "zone637", 637, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B814", zones.back()->m_zone, "zone639", 639, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B817", zones.back()->m_zone, "zone640", 640,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A818", zones.back()->m_zone, "zone646", 646, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A819", zones.back()->m_zone, "zone695", 695,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B807", zones.back()->m_zone, "zone638", 638, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A820", zones.back()->m_zone, "zone647", 647,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A821", zones.back()->m_zone, "zone645", 645, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A822", zones.back()->m_zone, "zone999", 999,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A823", zones.back()->m_zone, "zone661", 661,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A824", zones.back()->m_zone, "zone644", 644,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B92", zones.back()->m_zone, "zone598", 598,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 6, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B105", zones.back()->m_zone, "zone632", 632,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 11, 5}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A107", zones.back()->m_zone, "zone784", 784,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A108", zones.back()->m_zone, "zone776", 776,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A109", zones.back()->m_zone, "zone930", 930,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A110", zones.back()->m_zone, "zone665", 665,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B812", zones.back()->m_zone, "zone639", 639, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B824", zones.back()->m_zone, "zone642", 642,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A825", zones.back()->m_zone, "zone648", 648,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A826", zones.back()->m_zone, "zone660", 660,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A827", zones.back()->m_zone, "zone646", 646, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B58", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B815", zones.back()->m_zone, "zone640", 640, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B821", zones.back()->m_zone, "zone642", 642, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A828", zones.back()->m_zone, "zone649", 649, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A829", zones.back()->m_zone, "zone692", 692, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A830", zones.back()->m_zone, "zone646", 646,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B818", zones.back()->m_zone, "zone641", 641, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B827", zones.back()->m_zone, "zone644", 644, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B830", zones.back()->m_zone, "zone645", 645,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A831", zones.back()->m_zone, "zone650", 650, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A832", zones.back()->m_zone, "zone691", 691,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B613", zones.back()->m_zone, "zone564", 564,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B820", zones.back()->m_zone, "zone642", 642,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A833", zones.back()->m_zone, "zone666", 666,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A834", zones.back()->m_zone, "zone1001", 1001,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A835", zones.back()->m_zone, "zone649", 649,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A836", zones.back()->m_zone, "zone648", 648,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B618", zones.back()->m_zone, "zone566", 566,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B825", zones.back()->m_zone, "zone644", 644,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B836", zones.back()->m_zone, "zone647", 647,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A837", zones.back()->m_zone, "zone650", 650, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A838", zones.back()->m_zone, "zone664", 664,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B66", zones.back()->m_zone, "zone446", 446, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B622", zones.back()->m_zone, "zone567", 567, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B828", zones.back()->m_zone, "zone645", 645, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B835", zones.back()->m_zone, "zone647", 647, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A839", zones.back()->m_zone, "zone688", 688,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A840", zones.back()->m_zone, "zone650", 650,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B625", zones.back()->m_zone, "zone568", 568, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B831", zones.back()->m_zone, "zone646", 646, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B837", zones.back()->m_zone, "zone648", 648, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B840", zones.back()->m_zone, "zone649", 649,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A841", zones.back()->m_zone, "zone686", 686,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B792", zones.back()->m_zone, "zone630", 630, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B803", zones.back()->m_zone, "zone635", 635, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A842", zones.back()->m_zone, "zone656", 656,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A843", zones.back()->m_zone, "zone653", 653, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A844", zones.back()->m_zone, "zone652", 652,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B785", zones.back()->m_zone, "zone629", 629, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B796", zones.back()->m_zone, "zone634", 634, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B844", zones.back()->m_zone, "zone651", 651,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A845", zones.back()->m_zone, "zone655", 655, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A846", zones.back()->m_zone, "zone1016", 1016,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A847", zones.back()->m_zone, "zone657", 657,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B589", zones.back()->m_zone, "zone558", 558, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B780", zones.back()->m_zone, "zone626", 626, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B843", zones.back()->m_zone, "zone651", 651, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A848", zones.back()->m_zone, "zone658", 658,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A849", zones.back()->m_zone, "zone655", 655,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B83", zones.back()->m_zone, "zone576", 576, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B104", zones.back()->m_zone, "zone632", 632, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A111", zones.back()->m_zone, "zone683", 683, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A112", zones.back()->m_zone, "zone570", 570, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A113", zones.back()->m_zone, "zone665", 665, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A114", zones.back()->m_zone, "zone676", 676,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B597", zones.back()->m_zone, "zone559", 559, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B777", zones.back()->m_zone, "zone625", 625, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B845", zones.back()->m_zone, "zone652", 652, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B849", zones.back()->m_zone, "zone653", 653,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A850", zones.back()->m_zone, "zone1019", 1019,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A851", zones.back()->m_zone, "zone659", 659,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B813", zones.back()->m_zone, "zone639", 639, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B842", zones.back()->m_zone, "zone651", 651,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A852", zones.back()->m_zone, "zone660", 660,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A853", zones.back()->m_zone, "zone658", 658, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A854", zones.back()->m_zone, "zone657", 657,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B810", zones.back()->m_zone, "zone638", 638, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B847", zones.back()->m_zone, "zone652", 652,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B854", zones.back()->m_zone, "zone656", 656,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A855", zones.back()->m_zone, "zone661", 661,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A856", zones.back()->m_zone, "zone1017", 1017,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A857", zones.back()->m_zone, "zone659", 659, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B764", zones.back()->m_zone, "zone620", 620, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B848", zones.back()->m_zone, "zone653", 653,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B853", zones.back()->m_zone, "zone656", 656, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A858", zones.back()->m_zone, "zone662", 662,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A859", zones.back()->m_zone, "zone659", 659,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B766", zones.back()->m_zone, "zone622", 622, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B851", zones.back()->m_zone, "zone655", 655,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B857", zones.back()->m_zone, "zone657", 657, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B859", zones.back()->m_zone, "zone658", 658,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A860", zones.back()->m_zone, "zone663", 663,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A861", zones.back()->m_zone, "zone1018", 1018,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B826", zones.back()->m_zone, "zone644", 644,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B852", zones.back()->m_zone, "zone656", 656,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A862", zones.back()->m_zone, "zone664", 664,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A863", zones.back()->m_zone, "zone662", 662, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A864", zones.back()->m_zone, "zone661", 661,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B823", zones.back()->m_zone, "zone642", 642,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B855", zones.back()->m_zone, "zone657", 657,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B864", zones.back()->m_zone, "zone660", 660,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A865", zones.back()->m_zone, "zone663", 663, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A866", zones.back()->m_zone, "zone1000", 1000,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A867", zones.back()->m_zone, "zone666", 666,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B749", zones.back()->m_zone, "zone616", 616, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B858", zones.back()->m_zone, "zone658", 658,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B863", zones.back()->m_zone, "zone660", 660, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A868", zones.back()->m_zone, "zone667", 667,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A869", zones.back()->m_zone, "zone663", 663,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B756", zones.back()->m_zone, "zone617", 617, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B860", zones.back()->m_zone, "zone659", 659,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B865", zones.back()->m_zone, "zone661", 661, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B869", zones.back()->m_zone, "zone662", 662,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A870", zones.back()->m_zone, "zone1005", 1005,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A871", zones.back()->m_zone, "zone668", 668,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B617", zones.back()->m_zone, "zone566", 566, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B838", zones.back()->m_zone, "zone648", 648,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B862", zones.back()->m_zone, "zone660", 660,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A872", zones.back()->m_zone, "zone667", 667, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A873", zones.back()->m_zone, "zone666", 666,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B96", zones.back()->m_zone, "zone610", 610, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B110", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B113", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A115", zones.back()->m_zone, "zone673", 673,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A116", zones.back()->m_zone, "zone924", 924, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A117", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B614", zones.back()->m_zone, "zone564", 564, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B833", zones.back()->m_zone, "zone647", 647,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B867", zones.back()->m_zone, "zone661", 661,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B873", zones.back()->m_zone, "zone664", 664,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A874", zones.back()->m_zone, "zone1002", 1002,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A875", zones.back()->m_zone, "zone668", 668, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B758", zones.back()->m_zone, "zone618", 618, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B868", zones.back()->m_zone, "zone662", 662,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B872", zones.back()->m_zone, "zone664", 664, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A876", zones.back()->m_zone, "zone672", 672, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A877", zones.back()->m_zone, "zone668", 668,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B761", zones.back()->m_zone, "zone619", 619, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B871", zones.back()->m_zone, "zone663", 663,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B875", zones.back()->m_zone, "zone666", 666, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B877", zones.back()->m_zone, "zone667", 667,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A878", zones.back()->m_zone, "zone671", 671, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A879", zones.back()->m_zone, "zone1038", 1038,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B760", zones.back()->m_zone, "zone619", 619,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A880", zones.back()->m_zone, "zone869", 869,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A881", zones.back()->m_zone, "zone675", 675,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A882", zones.back()->m_zone, "zone1008", 1008,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A883", zones.back()->m_zone, "zone671", 671,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A884", zones.back()->m_zone, "zone670", 670,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B723", zones.back()->m_zone, "zone605", 605,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B757", zones.back()->m_zone, "zone618", 618, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B884", zones.back()->m_zone, "zone669", 669,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A885", zones.back()->m_zone, "zone672", 672, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A886", zones.back()->m_zone, "zone677", 677, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B610", zones.back()->m_zone, "zone564", 564,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B878", zones.back()->m_zone, "zone668", 668, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B883", zones.back()->m_zone, "zone669", 669,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A887", zones.back()->m_zone, "zone1004", 1004,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A888", zones.back()->m_zone, "zone673", 673,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A889", zones.back()->m_zone, "zone672", 672,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B616", zones.back()->m_zone, "zone566", 566,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B876", zones.back()->m_zone, "zone667", 667,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B885", zones.back()->m_zone, "zone670", 670, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B889", zones.back()->m_zone, "zone671", 671,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A890", zones.back()->m_zone, "zone674", 674,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B115", zones.back()->m_zone, "zone665", 665,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B620", zones.back()->m_zone, "zone567", 567, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B634", zones.back()->m_zone, "zone570", 570, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B888", zones.back()->m_zone, "zone671", 671,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A891", zones.back()->m_zone, "zone675", 675,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A892", zones.back()->m_zone, "zone674", 674,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B624", zones.back()->m_zone, "zone568", 568, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B629", zones.back()->m_zone, "zone569", 569,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B890", zones.back()->m_zone, "zone672", 672,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B892", zones.back()->m_zone, "zone673", 673,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A893", zones.back()->m_zone, "zone677", 677,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B168", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B661", zones.back()->m_zone, "zone580", 580,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B881", zones.back()->m_zone, "zone669", 669,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B891", zones.back()->m_zone, "zone673", 673,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A894", zones.back()->m_zone, "zone881", 881, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A895", zones.back()->m_zone, "zone677", 677,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B101", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B114", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A118", zones.back()->m_zone, "zone557", 557,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A119", zones.back()->m_zone, "zone687", 687, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A120", zones.back()->m_zone, "zone574", 574, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A121", zones.back()->m_zone, "zone809", 809,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B656", zones.back()->m_zone, "zone579", 579,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B728", zones.back()->m_zone, "zone606", 606, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B886", zones.back()->m_zone, "zone670", 670, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B893", zones.back()->m_zone, "zone674", 674,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B895", zones.back()->m_zone, "zone675", 675,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B626", zones.back()->m_zone, "zone568", 568, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B630", zones.back()->m_zone, "zone569", 569, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A896", zones.back()->m_zone, "zone680", 680, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A897", zones.back()->m_zone, "zone686", 686, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A898", zones.back()->m_zone, "zone679", 679,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B81", zones.back()->m_zone, "zone576", 576, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B621", zones.back()->m_zone, "zone567", 567, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B633", zones.back()->m_zone, "zone570", 570, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B898", zones.back()->m_zone, "zone678", 678,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A899", zones.back()->m_zone, "zone681", 681, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A900", zones.back()->m_zone, "zone688", 688, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B576", zones.back()->m_zone, "zone546", 546, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B896", zones.back()->m_zone, "zone678", 678, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A901", zones.back()->m_zone, "zone682", 682, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A902", zones.back()->m_zone, "zone684", 684, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A903", zones.back()->m_zone, "zone681", 681,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B80", zones.back()->m_zone, "zone576", 576, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B579", zones.back()->m_zone, "zone555", 555, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B899", zones.back()->m_zone, "zone679", 679, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B903", zones.back()->m_zone, "zone680", 680,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A904", zones.back()->m_zone, "zone685", 685, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A905", zones.back()->m_zone, "zone683", 683, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B581", zones.back()->m_zone, "zone556", 556, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B627", zones.back()->m_zone, "zone569", 569, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B901", zones.back()->m_zone, "zone680", 680, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A906", zones.back()->m_zone, "zone802", 802, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A907", zones.back()->m_zone, "zone683", 683,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B111", zones.back()->m_zone, "zone654", 654, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B585", zones.back()->m_zone, "zone557", 557, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B632", zones.back()->m_zone, "zone570", 570, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B905", zones.back()->m_zone, "zone681", 681, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B907", zones.back()->m_zone, "zone682", 682,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A908", zones.back()->m_zone, "zone803", 803, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B570", zones.back()->m_zone, "zone524", 524, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B902", zones.back()->m_zone, "zone680", 680, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A909", zones.back()->m_zone, "zone686", 686, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A910", zones.back()->m_zone, "zone689", 689,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A911", zones.back()->m_zone, "zone685", 685,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B62", zones.back()->m_zone, "zone335", 335, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B573", zones.back()->m_zone, "zone535", 535, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B904", zones.back()->m_zone, "zone681", 681, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B911", zones.back()->m_zone, "zone684", 684,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A912", zones.back()->m_zone, "zone690", 690,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A913", zones.back()->m_zone, "zone688", 688, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B841", zones.back()->m_zone, "zone650", 650,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B897", zones.back()->m_zone, "zone678", 678, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B909", zones.back()->m_zone, "zone684", 684, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A914", zones.back()->m_zone, "zone691", 691, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A915", zones.back()->m_zone, "zone688", 688,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B119", zones.back()->m_zone, "zone676", 676, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A122", zones.back()->m_zone, "zone402", 402, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A123", zones.back()->m_zone, "zone577", 577, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A124", zones.back()->m_zone, "zone698", 698,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A125", zones.back()->m_zone, "zone820", 820,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A126", zones.back()->m_zone, "zone500", 500, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B61", zones.back()->m_zone, "zone335", 335,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B839", zones.back()->m_zone, "zone649", 649,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B900", zones.back()->m_zone, "zone679", 679, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B913", zones.back()->m_zone, "zone685", 685, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B915", zones.back()->m_zone, "zone686", 686,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A916", zones.back()->m_zone, "zone692", 692, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B542", zones.back()->m_zone, "zone435", 435, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B910", zones.back()->m_zone, "zone684", 684,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A917", zones.back()->m_zone, "zone693", 693,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A918", zones.back()->m_zone, "zone691", 691, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A919", zones.back()->m_zone, "zone690", 690,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B53", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B545", zones.back()->m_zone, "zone447", 447, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B912", zones.back()->m_zone, "zone685", 685,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B919", zones.back()->m_zone, "zone689", 689,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A920", zones.back()->m_zone, "zone694", 694,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A921", zones.back()->m_zone, "zone692", 692, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B832", zones.back()->m_zone, "zone646", 646, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B914", zones.back()->m_zone, "zone686", 686, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B918", zones.back()->m_zone, "zone689", 689, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A922", zones.back()->m_zone, "zone695", 695, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A923", zones.back()->m_zone, "zone692", 692,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B54", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B829", zones.back()->m_zone, "zone645", 645, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B916", zones.back()->m_zone, "zone688", 688, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B921", zones.back()->m_zone, "zone690", 690, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B923", zones.back()->m_zone, "zone691", 691,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A924", zones.back()->m_zone, "zone696", 696, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B533", zones.back()->m_zone, "zone413", 413,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B917", zones.back()->m_zone, "zone689", 689,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A925", zones.back()->m_zone, "zone695", 695, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A926", zones.back()->m_zone, "zone734", 734, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A927", zones.back()->m_zone, "zone694", 694,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B42", zones.back()->m_zone, "zone1386", 1386,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B537", zones.back()->m_zone, "zone424", 424,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B920", zones.back()->m_zone, "zone690", 690,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B927", zones.back()->m_zone, "zone693", 693,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A928", zones.back()->m_zone, "zone735", 735, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A929", zones.back()->m_zone, "zone696", 696, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B605", zones.back()->m_zone, "zone562", 562, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B819", zones.back()->m_zone, "zone641", 641, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B922", zones.back()->m_zone, "zone691", 691, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B925", zones.back()->m_zone, "zone693", 693, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A930", zones.back()->m_zone, "zone696", 696,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B45", zones.back()->m_zone, "zone1386", 1386,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B609", zones.back()->m_zone, "zone563", 563, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B816", zones.back()->m_zone, "zone640", 640, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B924", zones.back()->m_zone, "zone692", 692, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B929", zones.back()->m_zone, "zone694", 694, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B930", zones.back()->m_zone, "zone695", 695,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A931", zones.back()->m_zone, "zone700", 700,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A932", zones.back()->m_zone, "zone717", 717, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A933", zones.back()->m_zone, "zone292", 292, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A934", zones.back()->m_zone, "zone702", 702, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A935", zones.back()->m_zone, "zone699", 699,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B124", zones.back()->m_zone, "zone687", 687,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A127", zones.back()->m_zone, "zone732", 732,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A128", zones.back()->m_zone, "zone380", 380, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A129", zones.back()->m_zone, "zone509", 509, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A130", zones.back()->m_zone, "zone832", 832,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A131", zones.back()->m_zone, "zone586", 586, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B254", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B935", zones.back()->m_zone, "zone697", 697,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A936", zones.back()->m_zone, "zone701", 701,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A937", zones.back()->m_zone, "zone703", 703, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A938", zones.back()->m_zone, "zone294", 294, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A939", zones.back()->m_zone, "zone718", 718, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B931", zones.back()->m_zone, "zone697", 697,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A940", zones.back()->m_zone, "zone702", 702, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A941", zones.back()->m_zone, "zone706", 706,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A942", zones.back()->m_zone, "zone715", 715, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A943", zones.back()->m_zone, "zone701", 701,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B256", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B936", zones.back()->m_zone, "zone699", 699,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B943", zones.back()->m_zone, "zone700", 700,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A944", zones.back()->m_zone, "zone716", 716, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A945", zones.back()->m_zone, "zone707", 707,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A946", zones.back()->m_zone, "zone703", 703,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B934", zones.back()->m_zone, "zone697", 697, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B940", zones.back()->m_zone, "zone700", 700, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A947", zones.back()->m_zone, "zone303", 303, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A948", zones.back()->m_zone, "zone704", 704, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A949", zones.back()->m_zone, "zone703", 703,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B290", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B937", zones.back()->m_zone, "zone699", 699, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B946", zones.back()->m_zone, "zone701", 701, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B949", zones.back()->m_zone, "zone702", 702,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A950", zones.back()->m_zone, "zone705", 705, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A951", zones.back()->m_zone, "zone304", 304, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B948", zones.back()->m_zone, "zone702", 702, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A952", zones.back()->m_zone, "zone706", 706, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A953", zones.back()->m_zone, "zone321", 321, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A954", zones.back()->m_zone, "zone713", 713, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A955", zones.back()->m_zone, "zone705", 705,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B284", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B950", zones.back()->m_zone, "zone703", 703, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B955", zones.back()->m_zone, "zone704", 704,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A956", zones.back()->m_zone, "zone707", 707, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A957", zones.back()->m_zone, "zone714", 714, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A958", zones.back()->m_zone, "zone319", 319, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B941", zones.back()->m_zone, "zone700", 700,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B952", zones.back()->m_zone, "zone704", 704,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A959", zones.back()->m_zone, "zone708", 708,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A960", zones.back()->m_zone, "zone742", 742, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A961", zones.back()->m_zone, "zone707", 707,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B261", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B945", zones.back()->m_zone, "zone701", 701,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B956", zones.back()->m_zone, "zone705", 705, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B961", zones.back()->m_zone, "zone706", 706,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A962", zones.back()->m_zone, "zone710", 710,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A963", zones.back()->m_zone, "zone744", 744, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B959", zones.back()->m_zone, "zone706", 706,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A964", zones.back()->m_zone, "zone713", 713, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A965", zones.back()->m_zone, "zone745", 745,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A966", zones.back()->m_zone, "zone740", 740, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A967", zones.back()->m_zone, "zone710", 710,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B26", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A132", zones.back()->m_zone, "zone291", 291, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A133", zones.back()->m_zone, "zone743", 743, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A134", zones.back()->m_zone, "zone721", 721, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A135", zones.back()->m_zone, "zone572", 572, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A136", zones.back()->m_zone, "zone732", 732,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B267", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B962", zones.back()->m_zone, "zone707", 707,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B967", zones.back()->m_zone, "zone708", 708,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A968", zones.back()->m_zone, "zone741", 741, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A969", zones.back()->m_zone, "zone746", 746,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A970", zones.back()->m_zone, "zone714", 714,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A971", zones.back()->m_zone, "zone745", 745, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A972", zones.back()->m_zone, "zone713", 713, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A973", zones.back()->m_zone, "zone337", 337, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A974", zones.back()->m_zone, "zone749", 749, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A975", zones.back()->m_zone, "zone712", 712,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B276", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B975", zones.back()->m_zone, "zone711", 711,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A976", zones.back()->m_zone, "zone746", 746, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A977", zones.back()->m_zone, "zone750", 750, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A978", zones.back()->m_zone, "zone339", 339, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A979", zones.back()->m_zone, "zone714", 714, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B954", zones.back()->m_zone, "zone704", 704, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B964", zones.back()->m_zone, "zone708", 708, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B972", zones.back()->m_zone, "zone711", 711, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A980", zones.back()->m_zone, "zone325", 325, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A981", zones.back()->m_zone, "zone714", 714,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B283", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B957", zones.back()->m_zone, "zone705", 705, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B970", zones.back()->m_zone, "zone710", 710, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B979", zones.back()->m_zone, "zone712", 712, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B981", zones.back()->m_zone, "zone713", 713,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A982", zones.back()->m_zone, "zone326", 326, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B942", zones.back()->m_zone, "zone700", 700, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A983", zones.back()->m_zone, "zone717", 717,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A984", zones.back()->m_zone, "zone742", 742,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A985", zones.back()->m_zone, "zone729", 729,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A986", zones.back()->m_zone, "zone716", 716,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B246", zones.back()->m_zone, "zone998", 998, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B944", zones.back()->m_zone, "zone701", 701, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B986", zones.back()->m_zone, "zone715", 715,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A987", zones.back()->m_zone, "zone718", 718,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A988", zones.back()->m_zone, "zone730", 730, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A989", zones.back()->m_zone, "zone744", 744,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B932", zones.back()->m_zone, "zone697", 697, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B983", zones.back()->m_zone, "zone715", 715,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A990", zones.back()->m_zone, "zone725", 725, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A991", zones.back()->m_zone, "zone281", 281, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A992", zones.back()->m_zone, "zone718", 718,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B247", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B939", zones.back()->m_zone, "zone699", 699, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B987", zones.back()->m_zone, "zone716", 716,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B992", zones.back()->m_zone, "zone717", 717,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A993", zones.back()->m_zone, "zone282", 282, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A994", zones.back()->m_zone, "zone726", 726,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A995", zones.back()->m_zone, "zone837", 837, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A996", zones.back()->m_zone, "zone841", 841, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A997", zones.back()->m_zone, "zone246", 246, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A998", zones.back()->m_zone, "zone723", 723, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A999", zones.back()->m_zone, "zone722", 722,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A12", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A13", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A14", zones.back()->m_zone, "zone743", 743,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A15", zones.back()->m_zone, "zone336", 336, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A16", zones.back()->m_zone, "zone840", 840, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B31", zones.back()->m_zone, "zone1053", 1053,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B134", zones.back()->m_zone, "zone709", 709, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A137", zones.back()->m_zone, "zone754", 754,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A138", zones.back()->m_zone, "zone948", 948, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A139", zones.back()->m_zone, "zone627", 627,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A140", zones.back()->m_zone, "zone843", 843,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B371", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B999", zones.back()->m_zone, "zone719", 719,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1000", zones.back()->m_zone, "zone838", 838,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1001", zones.back()->m_zone, "zone724", 724,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1002", zones.back()->m_zone, "zone249", 249, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1003", zones.back()->m_zone, "zone842", 842, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B998", zones.back()->m_zone, "zone719", 719, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1004", zones.back()->m_zone, "zone266", 266,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1005", zones.back()->m_zone, "zone725", 725,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1006", zones.back()->m_zone, "zone727", 727, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1007", zones.back()->m_zone, "zone724", 724,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B373", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1001", zones.back()->m_zone, "zone722", 722,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1007", zones.back()->m_zone, "zone723", 723,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1008", zones.back()->m_zone, "zone728", 728, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1009", zones.back()->m_zone, "zone726", 726,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1010", zones.back()->m_zone, "zone265", 265,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B990", zones.back()->m_zone, "zone717", 717, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1005", zones.back()->m_zone, "zone723", 723,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1011", zones.back()->m_zone, "zone729", 729, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1012", zones.back()->m_zone, "zone268", 268,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1013", zones.back()->m_zone, "zone726", 726,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B224", zones.back()->m_zone, "zone943", 943, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B994", zones.back()->m_zone, "zone718", 718, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1009", zones.back()->m_zone, "zone724", 724,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1013", zones.back()->m_zone, "zone725", 725,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1014", zones.back()->m_zone, "zone272", 272,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1015", zones.back()->m_zone, "zone730", 730, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1006", zones.back()->m_zone, "zone723", 723, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1016", zones.back()->m_zone, "zone835", 835,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1017", zones.back()->m_zone, "zone729", 729,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1018", zones.back()->m_zone, "zone731", 731, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1019", zones.back()->m_zone, "zone728", 728,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B391", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1008", zones.back()->m_zone, "zone724", 724, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1019", zones.back()->m_zone, "zone727", 727,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1020", zones.back()->m_zone, "zone836", 836,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1021", zones.back()->m_zone, "zone733", 733, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1022", zones.back()->m_zone, "zone730", 730,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B985", zones.back()->m_zone, "zone715", 715, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1011", zones.back()->m_zone, "zone725", 725, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1017", zones.back()->m_zone, "zone727", 727,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1023", zones.back()->m_zone, "zone738", 738, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1024", zones.back()->m_zone, "zone730", 730,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B219", zones.back()->m_zone, "zone931", 931, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B988", zones.back()->m_zone, "zone716", 716, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1015", zones.back()->m_zone, "zone726", 726, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1022", zones.back()->m_zone, "zone728", 728,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1024", zones.back()->m_zone, "zone729", 729,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1025", zones.back()->m_zone, "zone739", 739, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B548", zones.back()->m_zone, "zone458", 458, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1018", zones.back()->m_zone, "zone727", 727, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1026", zones.back()->m_zone, "zone734", 734,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1027", zones.back()->m_zone, "zone738", 738,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1028", zones.back()->m_zone, "zone733", 733,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B127", zones.back()->m_zone, "zone698", 698,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B136", zones.back()->m_zone, "zone709", 709,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{6, 11, 5}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A141", zones.back()->m_zone, "zone854", 854, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A142", zones.back()->m_zone, "zone591", 591,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A143", zones.back()->m_zone, "zone247", 247, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A144", zones.back()->m_zone, "zone765", 765, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 5, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B402", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B554", zones.back()->m_zone, "zone469", 469, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1021", zones.back()->m_zone, "zone728", 728, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1028", zones.back()->m_zone, "zone731", 731,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1029", zones.back()->m_zone, "zone735", 735,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1030", zones.back()->m_zone, "zone739", 739,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B539", zones.back()->m_zone, "zone435", 435,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B926", zones.back()->m_zone, "zone693", 693, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1026", zones.back()->m_zone, "zone731", 731,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1031", zones.back()->m_zone, "zone835", 835,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1032", zones.back()->m_zone, "zone735", 735,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B400", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B544", zones.back()->m_zone, "zone447", 447,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B928", zones.back()->m_zone, "zone694", 694, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1029", zones.back()->m_zone, "zone733", 733,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1032", zones.back()->m_zone, "zone734", 734,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1033", zones.back()->m_zone, "zone836", 836,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B549", zones.back()->m_zone, "zone458", 458,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B558", zones.back()->m_zone, "zone480", 480, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1034", zones.back()->m_zone, "zone740", 740,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1035", zones.back()->m_zone, "zone738", 738, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1036", zones.back()->m_zone, "zone737", 737,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B196", zones.back()->m_zone, "zone887", 887, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B553", zones.back()->m_zone, "zone469", 469,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B561", zones.back()->m_zone, "zone491", 491, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1036", zones.back()->m_zone, "zone736", 736,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1037", zones.back()->m_zone, "zone741", 741,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1038", zones.back()->m_zone, "zone739", 739, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1023", zones.back()->m_zone, "zone729", 729, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1027", zones.back()->m_zone, "zone731", 731,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1035", zones.back()->m_zone, "zone736", 736, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1039", zones.back()->m_zone, "zone742", 742,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1040", zones.back()->m_zone, "zone739", 739,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B214", zones.back()->m_zone, "zone920", 920, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1025", zones.back()->m_zone, "zone730", 730, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1030", zones.back()->m_zone, "zone733", 733,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1038", zones.back()->m_zone, "zone737", 737, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1040", zones.back()->m_zone, "zone738", 738,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1041", zones.back()->m_zone, "zone744", 744,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B966", zones.back()->m_zone, "zone708", 708, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1034", zones.back()->m_zone, "zone736", 736,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1042", zones.back()->m_zone, "zone742", 742,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1043", zones.back()->m_zone, "zone747", 747,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1044", zones.back()->m_zone, "zone741", 741,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B238", zones.back()->m_zone, "zone976", 976, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B968", zones.back()->m_zone, "zone710", 710, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1037", zones.back()->m_zone, "zone737", 737,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1044", zones.back()->m_zone, "zone740", 740,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1045", zones.back()->m_zone, "zone744", 744,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1046", zones.back()->m_zone, "zone748", 748,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B960", zones.back()->m_zone, "zone706", 706, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B984", zones.back()->m_zone, "zone715", 715,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1039", zones.back()->m_zone, "zone738", 738,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1042", zones.back()->m_zone, "zone740", 740,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1047", zones.back()->m_zone, "zone744", 744,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B14", zones.back()->m_zone, "zone720", 720,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B133", zones.back()->m_zone, "zone709", 709, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A145", zones.back()->m_zone, "zone269", 269, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A146", zones.back()->m_zone, "zone812", 812, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A147", zones.back()->m_zone, "zone754", 754,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A148", zones.back()->m_zone, "zone765", 765,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B239", zones.back()->m_zone, "zone987", 987, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B963", zones.back()->m_zone, "zone707", 707, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B989", zones.back()->m_zone, "zone716", 716,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1041", zones.back()->m_zone, "zone739", 739,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1045", zones.back()->m_zone, "zone741", 741,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1047", zones.back()->m_zone, "zone742", 742,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B965", zones.back()->m_zone, "zone708", 708,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B971", zones.back()->m_zone, "zone711", 711,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1048", zones.back()->m_zone, "zone749", 749,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1049", zones.back()->m_zone, "zone747", 747, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1050", zones.back()->m_zone, "zone746", 746,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B270", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B969", zones.back()->m_zone, "zone710", 710,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B976", zones.back()->m_zone, "zone712", 712, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1050", zones.back()->m_zone, "zone745", 745,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1051", zones.back()->m_zone, "zone750", 750,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1052", zones.back()->m_zone, "zone748", 748,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B555", zones.back()->m_zone, "zone480", 480, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1043", zones.back()->m_zone, "zone740", 740,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1049", zones.back()->m_zone, "zone745", 745,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1053", zones.back()->m_zone, "zone751", 751,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1054", zones.back()->m_zone, "zone748", 748,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B231", zones.back()->m_zone, "zone965", 965, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B560", zones.back()->m_zone, "zone491", 491,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1046", zones.back()->m_zone, "zone741", 741,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1052", zones.back()->m_zone, "zone746", 746, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1054", zones.back()->m_zone, "zone747", 747,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1055", zones.back()->m_zone, "zone752", 752,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B974", zones.back()->m_zone, "zone711", 711, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1048", zones.back()->m_zone, "zone745", 745,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1056", zones.back()->m_zone, "zone356", 356, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1057", zones.back()->m_zone, "zone751", 751,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1058", zones.back()->m_zone, "zone750", 750,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B272", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B977", zones.back()->m_zone, "zone712", 712, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1051", zones.back()->m_zone, "zone746", 746,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1058", zones.back()->m_zone, "zone749", 749,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1059", zones.back()->m_zone, "zone752", 752, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1060", zones.back()->m_zone, "zone353", 353, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1053", zones.back()->m_zone, "zone747", 747,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1057", zones.back()->m_zone, "zone749", 749, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1061", zones.back()->m_zone, "zone359", 359, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1062", zones.back()->m_zone, "zone753", 753,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1063", zones.back()->m_zone, "zone752", 752,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B230", zones.back()->m_zone, "zone954", 954, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1055", zones.back()->m_zone, "zone748", 748,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1059", zones.back()->m_zone, "zone750", 750,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1063", zones.back()->m_zone, "zone751", 751,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1064", zones.back()->m_zone, "zone755", 755,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1065", zones.back()->m_zone, "zone361", 361, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B556", zones.back()->m_zone, "zone480", 480, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1062", zones.back()->m_zone, "zone751", 751,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1066", zones.back()->m_zone, "zone370", 370,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1067", zones.back()->m_zone, "zone756", 756,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1068", zones.back()->m_zone, "zone755", 755,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B19", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B137", zones.back()->m_zone, "zone721", 721,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B147", zones.back()->m_zone, "zone743", 743,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 11, 5}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A149", zones.back()->m_zone, "zone865", 865,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A150", zones.back()->m_zone, "zone949", 949,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A151", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B207", zones.back()->m_zone, "zone909", 909, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B563", zones.back()->m_zone, "zone491", 491, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1064", zones.back()->m_zone, "zone752", 752,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1068", zones.back()->m_zone, "zone753", 753,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1069", zones.back()->m_zone, "zone757", 757,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1070", zones.back()->m_zone, "zone371", 371,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B565", zones.back()->m_zone, "zone502", 502, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1067", zones.back()->m_zone, "zone753", 753,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1071", zones.back()->m_zone, "zone793", 793,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1072", zones.back()->m_zone, "zone379", 379,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1073", zones.back()->m_zone, "zone757", 757,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B444", zones.back()->m_zone, "zone158", 158, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B568", zones.back()->m_zone, "zone513", 513, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1069", zones.back()->m_zone, "zone755", 755,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1073", zones.back()->m_zone, "zone756", 756,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1074", zones.back()->m_zone, "zone794", 794,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1075", zones.back()->m_zone, "zone382", 382,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1076", zones.back()->m_zone, "zone781", 781, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1077", zones.back()->m_zone, "zone768", 768,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1078", zones.back()->m_zone, "zone767", 767,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1079", zones.back()->m_zone, "zone759", 759, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1080", zones.back()->m_zone, "zone894", 894,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1079", zones.back()->m_zone, "zone758", 758, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1081", zones.back()->m_zone, "zone766", 766, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1082", zones.back()->m_zone, "zone760", 760,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1083", zones.back()->m_zone, "zone762", 762,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1084", zones.back()->m_zone, "zone892", 892,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1082", zones.back()->m_zone, "zone759", 759,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1085", zones.back()->m_zone, "zone766", 766,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1086", zones.back()->m_zone, "zone472", 472, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1087", zones.back()->m_zone, "zone763", 763,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1088", zones.back()->m_zone, "zone890", 890,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B159", zones.back()->m_zone, "zone776", 776,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1089", zones.back()->m_zone, "zone891", 891,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1090", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1091", zones.back()->m_zone, "zone784", 784,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1092", zones.back()->m_zone, "zone764", 764,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1093", zones.back()->m_zone, "zone762", 762,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1083", zones.back()->m_zone, "zone759", 759, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1093", zones.back()->m_zone, "zone761", 761,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1094", zones.back()->m_zone, "zone763", 763, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1095", zones.back()->m_zone, "zone785", 785,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1096", zones.back()->m_zone, "zone781", 781, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1087", zones.back()->m_zone, "zone760", 760,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1094", zones.back()->m_zone, "zone762", 762, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1097", zones.back()->m_zone, "zone475", 475, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1098", zones.back()->m_zone, "zone806", 806,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1099", zones.back()->m_zone, "zone764", 764,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B161", zones.back()->m_zone, "zone787", 787,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1092", zones.back()->m_zone, "zone761", 761, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1099", zones.back()->m_zone, "zone763", 763,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1100", zones.back()->m_zone, "zone889", 889, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1101", zones.back()->m_zone, "zone807", 807,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1102", zones.back()->m_zone, "zone477", 477, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B144", zones.back()->m_zone, "zone732", 732, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B148", zones.back()->m_zone, "zone743", 743, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A152", zones.back()->m_zone, "zone834", 834,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A153", zones.back()->m_zone, "zone519", 519, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A154", zones.back()->m_zone, "zone819", 819, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A155", zones.back()->m_zone, "zone876", 876, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 5, 1}},
      Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1081", zones.back()->m_zone, "zone759", 759, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1085", zones.back()->m_zone, "zone760", 760,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1103", zones.back()->m_zone, "zone462", 462, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1104", zones.back()->m_zone, "zone767", 767,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1105", zones.back()->m_zone, "zone899", 899,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1078", zones.back()->m_zone, "zone758", 758,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1104", zones.back()->m_zone, "zone766", 766, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1106", zones.back()->m_zone, "zone459", 459,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1107", zones.back()->m_zone, "zone768", 768,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1108", zones.back()->m_zone, "zone897", 897,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1077", zones.back()->m_zone, "zone758", 758,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1107", zones.back()->m_zone, "zone767", 767,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1109", zones.back()->m_zone, "zone775", 775, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1110", zones.back()->m_zone, "zone454", 454, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1111", zones.back()->m_zone, "zone896", 896,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1112", zones.back()->m_zone, "zone771", 771, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1113", zones.back()->m_zone, "zone786", 786,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1114", zones.back()->m_zone, "zone997", 997,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{51, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1115", zones.back()->m_zone, "zone782", 782,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1116", zones.back()->m_zone, "zone774", 774,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1117", zones.back()->m_zone, "zone770", 770,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1117", zones.back()->m_zone, "zone769", 769,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1118", zones.back()->m_zone, "zone772", 772, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1119", zones.back()->m_zone, "zone773", 773,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1120", zones.back()->m_zone, "zone783", 783,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1121", zones.back()->m_zone, "zone788", 788,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1112", zones.back()->m_zone, "zone769", 769, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1122", zones.back()->m_zone, "zone423", 423,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1123", zones.back()->m_zone, "zone791", 791,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1124", zones.back()->m_zone, "zone995", 995,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1125", zones.back()->m_zone, "zone774", 774,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1126", zones.back()->m_zone, "zone772", 772,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1118", zones.back()->m_zone, "zone770", 770, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1126", zones.back()->m_zone, "zone771", 771,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1127", zones.back()->m_zone, "zone421", 421,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1128", zones.back()->m_zone, "zone773", 773,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1129", zones.back()->m_zone, "zone792", 792,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1119", zones.back()->m_zone, "zone770", 770,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1128", zones.back()->m_zone, "zone772", 772,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1130", zones.back()->m_zone, "zone428", 428, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1131", zones.back()->m_zone, "zone778", 778, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1132", zones.back()->m_zone, "zone774", 774,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1116", zones.back()->m_zone, "zone769", 769,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1125", zones.back()->m_zone, "zone771", 771,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1132", zones.back()->m_zone, "zone773", 773,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1133", zones.back()->m_zone, "zone779", 779, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1134", zones.back()->m_zone, "zone996", 996,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1135", zones.back()->m_zone, "zone431", 431, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1109", zones.back()->m_zone, "zone768", 768,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1136", zones.back()->m_zone, "zone781", 781,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1137", zones.back()->m_zone, "zone778", 778, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1138", zones.back()->m_zone, "zone442", 442, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1139", zones.back()->m_zone, "zone777", 777,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B108", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A156", zones.back()->m_zone, "zone787", 787,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A157", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A158", zones.back()->m_zone, "zone932", 932, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A159", zones.back()->m_zone, "zone761", 761,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A160", zones.back()->m_zone, "zone888", 888,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1139", zones.back()->m_zone, "zone775", 775,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1140", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1141", zones.back()->m_zone, "zone895", 895,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1142", zones.back()->m_zone, "zone991", 991,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1143", zones.back()->m_zone, "zone443", 443, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1144", zones.back()->m_zone, "zone779", 779,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1131", zones.back()->m_zone, "zone773", 773, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1137", zones.back()->m_zone, "zone775", 775,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1145", zones.back()->m_zone, "zone783", 783,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1146", zones.back()->m_zone, "zone436", 436, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1147", zones.back()->m_zone, "zone779", 779,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1133", zones.back()->m_zone, "zone774", 774,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1144", zones.back()->m_zone, "zone777", 777, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1147", zones.back()->m_zone, "zone778", 778,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1148", zones.back()->m_zone, "zone782", 782,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1149", zones.back()->m_zone, "zone994", 994,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1150", zones.back()->m_zone, "zone437", 437, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1090", zones.back()->m_zone, "zone761", 761,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1140", zones.back()->m_zone, "zone777", 777,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1151", zones.back()->m_zone, "zone782", 782,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1152", zones.back()->m_zone, "zone992", 992,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1153", zones.back()->m_zone, "zone893", 893,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1154", zones.back()->m_zone, "zone781", 781,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1076", zones.back()->m_zone, "zone758", 758,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1096", zones.back()->m_zone, "zone762", 762, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1136", zones.back()->m_zone, "zone775", 775,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1154", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1155", zones.back()->m_zone, "zone783", 783,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1115", zones.back()->m_zone, "zone769", 769,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1148", zones.back()->m_zone, "zone779", 779,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1151", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1156", zones.back()->m_zone, "zone993", 993,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1157", zones.back()->m_zone, "zone784", 784,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1158", zones.back()->m_zone, "zone783", 783,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1120", zones.back()->m_zone, "zone770", 770,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1145", zones.back()->m_zone, "zone778", 778,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1155", zones.back()->m_zone, "zone781", 781,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1158", zones.back()->m_zone, "zone782", 782,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1159", zones.back()->m_zone, "zone785", 785, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B107", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1091", zones.back()->m_zone, "zone761", 761, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1157", zones.back()->m_zone, "zone782", 782, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1160", zones.back()->m_zone, "zone786", 786,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1161", zones.back()->m_zone, "zone805", 805,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1162", zones.back()->m_zone, "zone785", 785,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1095", zones.back()->m_zone, "zone762", 762, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1159", zones.back()->m_zone, "zone783", 783,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1162", zones.back()->m_zone, "zone784", 784,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1163", zones.back()->m_zone, "zone804", 804,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1164", zones.back()->m_zone, "zone788", 788,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B91", zones.back()->m_zone, "zone598", 598, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1113", zones.back()->m_zone, "zone769", 769,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1160", zones.back()->m_zone, "zone784", 784, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1165", zones.back()->m_zone, "zone791", 791,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1166", zones.back()->m_zone, "zone799", 799,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1167", zones.back()->m_zone, "zone788", 788,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B102", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 1}}, Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B156", zones.back()->m_zone, "zone776", 776,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A161", zones.back()->m_zone, "zone764", 764,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A162", zones.back()->m_zone, "zone488", 488, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A163", zones.back()->m_zone, "zone809", 809,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A164", zones.back()->m_zone, "zone886", 886,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1121", zones.back()->m_zone, "zone770", 770,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1164", zones.back()->m_zone, "zone785", 785, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1167", zones.back()->m_zone, "zone786", 786,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1168", zones.back()->m_zone, "zone797", 797,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1169", zones.back()->m_zone, "zone792", 792,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("A1170", zones.back()->m_zone, "zone795", 795,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1171", zones.back()->m_zone, "zone407", 407, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1172", zones.back()->m_zone, "zone792", 792, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1173", zones.back()->m_zone, "zone797", 797, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1174", zones.back()->m_zone, "zone790", 790,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B73", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1174", zones.back()->m_zone, "zone789", 789,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1175", zones.back()->m_zone, "zone796", 796,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1176", zones.back()->m_zone, "zone799", 799, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1177", zones.back()->m_zone, "zone791", 791, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1178", zones.back()->m_zone, "zone411", 411, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B78", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1123", zones.back()->m_zone, "zone771", 771,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1165", zones.back()->m_zone, "zone786", 786,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1177", zones.back()->m_zone, "zone790", 790, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1179", zones.back()->m_zone, "zone418", 418,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1180", zones.back()->m_zone, "zone792", 792,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1129", zones.back()->m_zone, "zone772", 772,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1169", zones.back()->m_zone, "zone788", 788,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1172", zones.back()->m_zone, "zone789", 789, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1180", zones.back()->m_zone, "zone791", 791,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1181", zones.back()->m_zone, "zone414", 414,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B574", zones.back()->m_zone, "zone546", 546,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1071", zones.back()->m_zone, "zone756", 756,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1182", zones.back()->m_zone, "zone393", 393, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1183", zones.back()->m_zone, "zone795", 795, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1184", zones.back()->m_zone, "zone794", 794,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B442", zones.back()->m_zone, "zone147", 147, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B578", zones.back()->m_zone, "zone555", 555,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1074", zones.back()->m_zone, "zone757", 757,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1184", zones.back()->m_zone, "zone793", 793,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1185", zones.back()->m_zone, "zone796", 796, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1186", zones.back()->m_zone, "zone395", 395, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1170", zones.back()->m_zone, "zone789", 789,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1183", zones.back()->m_zone, "zone793", 793, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1187", zones.back()->m_zone, "zone800", 800,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1188", zones.back()->m_zone, "zone403", 403, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1189", zones.back()->m_zone, "zone796", 796,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B72", zones.back()->m_zone, "zone554", 554, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1175", zones.back()->m_zone, "zone790", 790,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1185", zones.back()->m_zone, "zone794", 794, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1189", zones.back()->m_zone, "zone795", 795,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1190", zones.back()->m_zone, "zone801", 801,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1191", zones.back()->m_zone, "zone404", 404, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1168", zones.back()->m_zone, "zone788", 788,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1173", zones.back()->m_zone, "zone789", 789, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1192", zones.back()->m_zone, "zone800", 800, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1193", zones.back()->m_zone, "zone804", 804, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1194", zones.back()->m_zone, "zone799", 799,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B117", zones.back()->m_zone, "zone665", 665,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B157", zones.back()->m_zone, "zone776", 776,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A165", zones.back()->m_zone, "zone809", 809,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A166", zones.back()->m_zone, "zone884", 884, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A167", zones.back()->m_zone, "zone933", 933,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A168", zones.back()->m_zone, "zone675", 675,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B86", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1166", zones.back()->m_zone, "zone786", 786,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1176", zones.back()->m_zone, "zone790", 790, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1194", zones.back()->m_zone, "zone797", 797,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1195", zones.back()->m_zone, "zone801", 801, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1196", zones.back()->m_zone, "zone805", 805, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B575", zones.back()->m_zone, "zone546", 546, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1187", zones.back()->m_zone, "zone795", 795,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1192", zones.back()->m_zone, "zone797", 797, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1197", zones.back()->m_zone, "zone802", 802,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1198", zones.back()->m_zone, "zone801", 801,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B87", zones.back()->m_zone, "zone587", 587, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B580", zones.back()->m_zone, "zone555", 555, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1190", zones.back()->m_zone, "zone796", 796,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1195", zones.back()->m_zone, "zone799", 799, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1198", zones.back()->m_zone, "zone800", 800,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1199", zones.back()->m_zone, "zone803", 803,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B906", zones.back()->m_zone, "zone682", 682, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1197", zones.back()->m_zone, "zone800", 800,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1200", zones.back()->m_zone, "zone804", 804,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1201", zones.back()->m_zone, "zone808", 808,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1202", zones.back()->m_zone, "zone803", 803,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B103", zones.back()->m_zone, "zone632", 632, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B908", zones.back()->m_zone, "zone683", 683, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1199", zones.back()->m_zone, "zone801", 801,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1202", zones.back()->m_zone, "zone802", 802,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1203", zones.back()->m_zone, "zone810", 810,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1204", zones.back()->m_zone, "zone805", 805,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1163", zones.back()->m_zone, "zone785", 785,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1193", zones.back()->m_zone, "zone797", 797, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1200", zones.back()->m_zone, "zone802", 802,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1205", zones.back()->m_zone, "zone806", 806,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1206", zones.back()->m_zone, "zone805", 805,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B106", zones.back()->m_zone, "zone632", 632, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1161", zones.back()->m_zone, "zone784", 784,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1196", zones.back()->m_zone, "zone799", 799,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1204", zones.back()->m_zone, "zone803", 803,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1206", zones.back()->m_zone, "zone804", 804,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1207", zones.back()->m_zone, "zone807", 807, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1098", zones.back()->m_zone, "zone763", 763,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1205", zones.back()->m_zone, "zone804", 804, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1208", zones.back()->m_zone, "zone808", 808,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1209", zones.back()->m_zone, "zone483", 483, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1210", zones.back()->m_zone, "zone807", 807,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B98", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 5, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1101", zones.back()->m_zone, "zone764", 764,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1207", zones.back()->m_zone, "zone805", 805, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1210", zones.back()->m_zone, "zone806", 806,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1211", zones.back()->m_zone, "zone810", 810,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1212", zones.back()->m_zone, "zone479", 479, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B582", zones.back()->m_zone, "zone556", 556, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1201", zones.back()->m_zone, "zone802", 802, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1208", zones.back()->m_zone, "zone806", 806,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1213", zones.back()->m_zone, "zone495", 495, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1214", zones.back()->m_zone, "zone810", 810,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B121", zones.back()->m_zone, "zone676", 676,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 5, 1}},
                                                Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B163", zones.back()->m_zone, "zone787", 787,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B165", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A169", zones.back()->m_zone, "zone580", 580,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A170", zones.back()->m_zone, "zone885", 885, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A171", zones.back()->m_zone, "zone820", 820, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{11, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B99", zones.back()->m_zone, "zone621", 621, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B587", zones.back()->m_zone, "zone557", 557, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1203", zones.back()->m_zone, "zone803", 803, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1211", zones.back()->m_zone, "zone807", 807,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1214", zones.back()->m_zone, "zone808", 808,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1215", zones.back()->m_zone, "zone494", 494, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B481", zones.back()->m_zone, "zone258", 258, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B500", zones.back()->m_zone, "zone324", 324, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1216", zones.back()->m_zone, "zone818", 818, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1217", zones.back()->m_zone, "zone830", 830,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1218", zones.back()->m_zone, "zone812", 812,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B146", zones.back()->m_zone, "zone743", 743, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B484", zones.back()->m_zone, "zone269", 269, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B507", zones.back()->m_zone, "zone336", 336, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1218", zones.back()->m_zone, "zone811", 811,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1219", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1220", zones.back()->m_zone, "zone819", 819,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B189", zones.back()->m_zone, "zone865", 865,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B512", zones.back()->m_zone, "zone347", 347,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1221", zones.back()->m_zone, "zone914", 914, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1222", zones.back()->m_zone, "zone817", 817, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1223", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1224", zones.back()->m_zone, "zone814", 814,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B515", zones.back()->m_zone, "zone358", 358,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1224", zones.back()->m_zone, "zone813", 813,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1225", zones.back()->m_zone, "zone824", 824, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1226", zones.back()->m_zone, "zone830", 830, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1227", zones.back()->m_zone, "zone816", 816,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A1228", zones.back()->m_zone, "zone824", 824,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1229", zones.back()->m_zone, "zone816", 816,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1230", zones.back()->m_zone, "zone540", 540, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1231", zones.back()->m_zone, "zone821", 821,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1232", zones.back()->m_zone, "zone915", 915,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1227", zones.back()->m_zone, "zone814", 814, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1229", zones.back()->m_zone, "zone815", 815,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1233", zones.back()->m_zone, "zone818", 818,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1234", zones.back()->m_zone, "zone527", 527, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1235", zones.back()->m_zone, "zone817", 817,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B193", zones.back()->m_zone, "zone876", 876,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{6, 11, 8}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1222", zones.back()->m_zone, "zone813", 813, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1235", zones.back()->m_zone, "zone816", 816,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1236", zones.back()->m_zone, "zone913", 913,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1237", zones.back()->m_zone, "zone531", 531, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1238", zones.back()->m_zone, "zone819", 819,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1216", zones.back()->m_zone, "zone811", 811, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1233", zones.back()->m_zone, "zone816", 816,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1239", zones.back()->m_zone, "zone833", 833,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1240", zones.back()->m_zone, "zone523", 523, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1241", zones.back()->m_zone, "zone819", 819,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B154", zones.back()->m_zone, "zone765", 765, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1220", zones.back()->m_zone, "zone812", 812, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1238", zones.back()->m_zone, "zone817", 817,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1241", zones.back()->m_zone, "zone818", 818,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1242", zones.back()->m_zone, "zone834", 834,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1243", zones.back()->m_zone, "zone522", 522, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B125", zones.back()->m_zone, "zone687", 687,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B171", zones.back()->m_zone, "zone809", 809, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A172", zones.back()->m_zone, "zone903", 903,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A173", zones.back()->m_zone, "zone498", 498,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A174", zones.back()->m_zone, "zone832", 832,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A175", zones.back()->m_zone, "zone582", 582, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1231", zones.back()->m_zone, "zone815", 815,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1244", zones.back()->m_zone, "zone543", 543, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1245", zones.back()->m_zone, "zone822", 822, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1246", zones.back()->m_zone, "zone824", 824, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1247", zones.back()->m_zone, "zone917", 917,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1245", zones.back()->m_zone, "zone821", 821,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1248", zones.back()->m_zone, "zone550", 550,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1249", zones.back()->m_zone, "zone852", 852,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1250", zones.back()->m_zone, "zone823", 823,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1251", zones.back()->m_zone, "zone918", 918,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B516", zones.back()->m_zone, "zone358", 358, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1250", zones.back()->m_zone, "zone822", 822,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1252", zones.back()->m_zone, "zone824", 824,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1253", zones.back()->m_zone, "zone852", 852,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1254", zones.back()->m_zone, "zone921", 921,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1225", zones.back()->m_zone, "zone814", 814,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1228", zones.back()->m_zone, "zone815", 815,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1246", zones.back()->m_zone, "zone821", 821, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1252", zones.back()->m_zone, "zone823", 823,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1255", zones.back()->m_zone, "zone916", 916,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B456", zones.back()->m_zone, "zone202", 202,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B508", zones.back()->m_zone, "zone347", 347,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1256", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1257", zones.back()->m_zone, "zone1033", 1033,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1258", zones.back()->m_zone, "zone850", 850,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1259", zones.back()->m_zone, "zone826", 826,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B451", zones.back()->m_zone, "zone191", 191, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B514", zones.back()->m_zone, "zone358", 358,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1259", zones.back()->m_zone, "zone825", 825,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1260", zones.back()->m_zone, "zone851", 851,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1261", zones.back()->m_zone, "zone830", 830, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B20", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B506", zones.back()->m_zone, "zone336", 336,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1262", zones.back()->m_zone, "zone850", 850,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1263", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1264", zones.back()->m_zone, "zone846", 846,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1265", zones.back()->m_zone, "zone828", 828,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B501", zones.back()->m_zone, "zone324", 324,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1265", zones.back()->m_zone, "zone827", 827,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1266", zones.back()->m_zone, "zone851", 851,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1267", zones.back()->m_zone, "zone847", 847,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1268", zones.back()->m_zone, "zone830", 830,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B151", zones.back()->m_zone, "zone754", 754,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1219", zones.back()->m_zone, "zone812", 812,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1223", zones.back()->m_zone, "zone813", 813,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1256", zones.back()->m_zone, "zone825", 825,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1263", zones.back()->m_zone, "zone827", 827,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1269", zones.back()->m_zone, "zone830", 830,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1217", zones.back()->m_zone, "zone811", 811,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1226", zones.back()->m_zone, "zone814", 814, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1261", zones.back()->m_zone, "zone826", 826, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1268", zones.back()->m_zone, "zone828", 828,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1269", zones.back()->m_zone, "zone829", 829,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B7", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B13", zones.back()->m_zone, "zone720", 720, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A17", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A18", zones.back()->m_zone, "zone955", 955, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A19", zones.back()->m_zone, "zone754", 754, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A20", zones.back()->m_zone, "zone827", 827, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B130", zones.back()->m_zone, "zone698", 698,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B174", zones.back()->m_zone, "zone820", 820,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A176", zones.back()->m_zone, "zone854", 854, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A177", zones.back()->m_zone, "zone904", 904,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A178", zones.back()->m_zone, "zone589", 589, Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A179", zones.back()->m_zone, "zone507", 507, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B472", zones.back()->m_zone, "zone236", 236, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B480", zones.back()->m_zone, "zone258", 258, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1239", zones.back()->m_zone, "zone818", 818,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1270", zones.back()->m_zone, "zone515", 515, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1271", zones.back()->m_zone, "zone834", 834,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B152", zones.back()->m_zone, "zone765", 765,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B475", zones.back()->m_zone, "zone247", 247, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B485", zones.back()->m_zone, "zone269", 269, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1242", zones.back()->m_zone, "zone819", 819,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1271", zones.back()->m_zone, "zone833", 833,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1272", zones.back()->m_zone, "zone518", 518, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B532", zones.back()->m_zone, "zone413", 413, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1016", zones.back()->m_zone, "zone727", 727,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1031", zones.back()->m_zone, "zone734", 734,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1273", zones.back()->m_zone, "zone837", 837,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1274", zones.back()->m_zone, "zone836", 836,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B389", zones.back()->m_zone, "zone1442", 1442,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B538", zones.back()->m_zone, "zone424", 424, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1020", zones.back()->m_zone, "zone728", 728,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1033", zones.back()->m_zone, "zone735", 735,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1274", zones.back()->m_zone, "zone835", 835,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1275", zones.back()->m_zone, "zone838", 838,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B492", zones.back()->m_zone, "zone302", 302, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B995", zones.back()->m_zone, "zone719", 719, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1273", zones.back()->m_zone, "zone835", 835,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1276", zones.back()->m_zone, "zone839", 839, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1277", zones.back()->m_zone, "zone838", 838,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B366", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B496", zones.back()->m_zone, "zone313", 313, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1000", zones.back()->m_zone, "zone722", 722,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1275", zones.back()->m_zone, "zone836", 836,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1277", zones.back()->m_zone, "zone837", 837,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1278", zones.back()->m_zone, "zone840", 840, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B478", zones.back()->m_zone, "zone258", 258, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B499", zones.back()->m_zone, "zone324", 324, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1276", zones.back()->m_zone, "zone837", 837, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1279", zones.back()->m_zone, "zone841", 841,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1280", zones.back()->m_zone, "zone840", 840,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B16", zones.back()->m_zone, "zone720", 720, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B483", zones.back()->m_zone, "zone269", 269, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B504", zones.back()->m_zone, "zone336", 336, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1278", zones.back()->m_zone, "zone838", 838, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1280", zones.back()->m_zone, "zone839", 839,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1281", zones.back()->m_zone, "zone842", 842,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B996", zones.back()->m_zone, "zone719", 719, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1279", zones.back()->m_zone, "zone839", 839,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1282", zones.back()->m_zone, "zone844", 844,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1283", zones.back()->m_zone, "zone237", 237, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1284", zones.back()->m_zone, "zone842", 842,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1003", zones.back()->m_zone, "zone722", 722, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1281", zones.back()->m_zone, "zone840", 840,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1284", zones.back()->m_zone, "zone841", 841,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1285", zones.back()->m_zone, "zone239", 239, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1286", zones.back()->m_zone, "zone845", 845,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B140", zones.back()->m_zone, "zone721", 721,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A180", zones.back()->m_zone, "zone854", 854,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A181", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A182", zones.back()->m_zone, "zone911", 911, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A183", zones.back()->m_zone, "zone951", 951, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A184", zones.back()->m_zone, "zone865", 865,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B502", zones.back()->m_zone, "zone324", 324, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1282", zones.back()->m_zone, "zone841", 841,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1287", zones.back()->m_zone, "zone847", 847,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1288", zones.back()->m_zone, "zone233", 233, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1289", zones.back()->m_zone, "zone845", 845,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B6", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B505", zones.back()->m_zone, "zone336", 336, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1286", zones.back()->m_zone, "zone842", 842,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1289", zones.back()->m_zone, "zone844", 844,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1290", zones.back()->m_zone, "zone231", 231, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1291", zones.back()->m_zone, "zone846", 846,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B11", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1264", zones.back()->m_zone, "zone827", 827,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1291", zones.back()->m_zone, "zone845", 845,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1292", zones.back()->m_zone, "zone230", 230,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1293", zones.back()->m_zone, "zone848", 848,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1294", zones.back()->m_zone, "zone847", 847,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1267", zones.back()->m_zone, "zone828", 828,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1287", zones.back()->m_zone, "zone844", 844,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B1294", zones.back()->m_zone, "zone846", 846,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1295", zones.back()->m_zone, "zone849", 849,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1296", zones.back()->m_zone, "zone227", 227,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B466", zones.back()->m_zone, "zone225", 225,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1293", zones.back()->m_zone, "zone846", 846,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1297", zones.back()->m_zone, "zone222", 222,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1298", zones.back()->m_zone, "zone1028", 1028,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1299", zones.back()->m_zone, "zone850", 850, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1300", zones.back()->m_zone, "zone849", 849,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B463", zones.back()->m_zone, "zone213", 213,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1295", zones.back()->m_zone, "zone847", 847,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1300", zones.back()->m_zone, "zone848", 848,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1301", zones.back()->m_zone, "zone219", 219,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1302", zones.back()->m_zone, "zone851", 851, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B465", zones.back()->m_zone, "zone225", 225, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1258", zones.back()->m_zone, "zone825", 825,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1262", zones.back()->m_zone, "zone827", 827,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1299", zones.back()->m_zone, "zone848", 848, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1303", zones.back()->m_zone, "zone1029", 1029,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1304", zones.back()->m_zone, "zone851", 851,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B461", zones.back()->m_zone, "zone213", 213,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1260", zones.back()->m_zone, "zone826", 826,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1266", zones.back()->m_zone, "zone828", 828,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1302", zones.back()->m_zone, "zone849", 849, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1304", zones.back()->m_zone, "zone850", 850,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1249", zones.back()->m_zone, "zone822", 822,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1253", zones.back()->m_zone, "zone823", 823,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1305", zones.back()->m_zone, "zone551", 551, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1306", zones.back()->m_zone, "zone853", 853,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1307", zones.back()->m_zone, "zone919", 919,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B452", zones.back()->m_zone, "zone191", 191, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B517", zones.back()->m_zone, "zone358", 358, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1306", zones.back()->m_zone, "zone852", 852, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1308", zones.back()->m_zone, "zone200", 200, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1309", zones.back()->m_zone, "zone855", 855,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B141", zones.back()->m_zone, "zone732", 732,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B176", zones.back()->m_zone, "zone832", 832, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B180", zones.back()->m_zone, "zone843", 843,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A185", zones.back()->m_zone, "zone876", 876,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A186", zones.back()->m_zone, "zone905", 905, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A187", zones.back()->m_zone, "zone595", 595, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B460", zones.back()->m_zone, "zone202", 202, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B509", zones.back()->m_zone, "zone347", 347, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1309", zones.back()->m_zone, "zone853", 853,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1310", zones.back()->m_zone, "zone1035", 1035,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1311", zones.back()->m_zone, "zone201", 201, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1312", zones.back()->m_zone, "zone923", 923,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B769", zones.back()->m_zone, "zone623", 623, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1313", zones.back()->m_zone, "zone1027", 1027,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1314", zones.back()->m_zone, "zone859", 859,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1315", zones.back()->m_zone, "zone857", 857,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1316", zones.back()->m_zone, "zone866", 866,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B703", zones.back()->m_zone, "zone596", 596, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1315", zones.back()->m_zone, "zone856", 856,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1317", zones.back()->m_zone, "zone858", 858,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1318", zones.back()->m_zone, "zone864", 864,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1319", zones.back()->m_zone, "zone874", 874,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B745", zones.back()->m_zone, "zone614", 614, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1317", zones.back()->m_zone, "zone857", 857,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1320", zones.back()->m_zone, "zone859", 859, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1321", zones.back()->m_zone, "zone874", 874,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1322", zones.back()->m_zone, "zone860", 860,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B594", zones.back()->m_zone, "zone559", 559, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1314", zones.back()->m_zone, "zone856", 856,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1320", zones.back()->m_zone, "zone858", 858, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1323", zones.back()->m_zone, "zone1026", 1026,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1324", zones.back()->m_zone, "zone861", 861, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B744", zones.back()->m_zone, "zone613", 613,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1322", zones.back()->m_zone, "zone858", 858,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1325", zones.back()->m_zone, "zone861", 861,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1326", zones.back()->m_zone, "zone863", 863, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1327", zones.back()->m_zone, "zone873", 873,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B768", zones.back()->m_zone, "zone622", 622, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1324", zones.back()->m_zone, "zone859", 859, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1325", zones.back()->m_zone, "zone860", 860,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1328", zones.back()->m_zone, "zone1025", 1025,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1329", zones.back()->m_zone, "zone862", 862,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B753", zones.back()->m_zone, "zone617", 617,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1329", zones.back()->m_zone, "zone861", 861,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1330", zones.back()->m_zone, "zone1012", 1012,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1331", zones.back()->m_zone, "zone868", 868, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1332", zones.back()->m_zone, "zone863", 863, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B747", zones.back()->m_zone, "zone615", 615,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1326", zones.back()->m_zone, "zone860", 860, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1332", zones.back()->m_zone, "zone862", 862, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1333", zones.back()->m_zone, "zone867", 867, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1334", zones.back()->m_zone, "zone872", 872,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B705", zones.back()->m_zone, "zone597", 597, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1318", zones.back()->m_zone, "zone857", 857,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1335", zones.back()->m_zone, "zone902", 902,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1336", zones.back()->m_zone, "zone866", 866,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 12, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1337", zones.back()->m_zone, "zone875", 875,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B149", zones.back()->m_zone, "zone754", 754,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B184", zones.back()->m_zone, "zone843", 843,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A188", zones.back()->m_zone, "zone876", 876,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A189", zones.back()->m_zone, "zone813", 813,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A190", zones.back()->m_zone, "zone950", 950, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A191", zones.back()->m_zone, "zone912", 912,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B690", zones.back()->m_zone, "zone592", 592,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1316", zones.back()->m_zone, "zone856", 856, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1336", zones.back()->m_zone, "zone864", 864,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1338", zones.back()->m_zone, "zone911", 911,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1339", zones.back()->m_zone, "zone910", 910,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B735", zones.back()->m_zone, "zone608", 608,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1333", zones.back()->m_zone, "zone863", 863, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1340", zones.back()->m_zone, "zone868", 868, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1341", zones.back()->m_zone, "zone870", 870,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1342", zones.back()->m_zone, "zone871", 871,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B763", zones.back()->m_zone, "zone619", 619,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1331", zones.back()->m_zone, "zone862", 862, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1340", zones.back()->m_zone, "zone867", 867, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1343", zones.back()->m_zone, "zone1011", 1011,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1344", zones.back()->m_zone, "zone869", 869,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B880", zones.back()->m_zone, "zone669", 669,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1344", zones.back()->m_zone, "zone868", 868,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1345", zones.back()->m_zone, "zone1010", 1010,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1346", zones.back()->m_zone, "zone870", 870,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1347", zones.back()->m_zone, "zone881", 881,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B727", zones.back()->m_zone, "zone605", 605, Ioss::IJK_t{{-2, 3, -1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{12, 101, 1}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1341", zones.back()->m_zone, "zone867", 867,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1346", zones.back()->m_zone, "zone869", 869,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1348", zones.back()->m_zone, "zone880", 880,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1349", zones.back()->m_zone, "zone871", 871,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B732", zones.back()->m_zone, "zone607", 607,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1342", zones.back()->m_zone, "zone867", 867, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1349", zones.back()->m_zone, "zone870", 870, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1350", zones.back()->m_zone, "zone879", 879,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1351", zones.back()->m_zone, "zone872", 872,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B738", zones.back()->m_zone, "zone611", 611,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1334", zones.back()->m_zone, "zone863", 863,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1351", zones.back()->m_zone, "zone871", 871,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1352", zones.back()->m_zone, "zone873", 873,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1353", zones.back()->m_zone, "zone878", 878,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B740", zones.back()->m_zone, "zone612", 612,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1327", zones.back()->m_zone, "zone860", 860, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1352", zones.back()->m_zone, "zone872", 872,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1354", zones.back()->m_zone, "zone877", 877,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1355", zones.back()->m_zone, "zone874", 874,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B709", zones.back()->m_zone, "zone599", 599, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1319", zones.back()->m_zone, "zone857", 857, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1321", zones.back()->m_zone, "zone858", 858, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1355", zones.back()->m_zone, "zone873", 873,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1356", zones.back()->m_zone, "zone875", 875,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B711", zones.back()->m_zone, "zone600", 600,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1337", zones.back()->m_zone, "zone864", 864, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1356", zones.back()->m_zone, "zone874", 874,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1357", zones.back()->m_zone, "zone902", 902,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1358", zones.back()->m_zone, "zone877", 877,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B155", zones.back()->m_zone, "zone765", 765, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 1}}, Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B185", zones.back()->m_zone, "zone854", 854,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B188", zones.back()->m_zone, "zone865", 865,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A192", zones.back()->m_zone, "zone529", 529, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A193", zones.back()->m_zone, "zone817", 817,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A194", zones.back()->m_zone, "zone906", 906,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B714", zones.back()->m_zone, "zone601", 601,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1354", zones.back()->m_zone, "zone873", 873,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1358", zones.back()->m_zone, "zone875", 875,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1359", zones.back()->m_zone, "zone901", 901,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1360", zones.back()->m_zone, "zone878", 878,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B718", zones.back()->m_zone, "zone602", 602,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1353", zones.back()->m_zone, "zone872", 872,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1360", zones.back()->m_zone, "zone877", 877,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1361", zones.back()->m_zone, "zone900", 900,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1362", zones.back()->m_zone, "zone879", 879,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B722", zones.back()->m_zone, "zone604", 604,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1350", zones.back()->m_zone, "zone871", 871,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1362", zones.back()->m_zone, "zone878", 878,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1363", zones.back()->m_zone, "zone882", 882, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1364", zones.back()->m_zone, "zone880", 880, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B729", zones.back()->m_zone, "zone606", 606, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1348", zones.back()->m_zone, "zone870", 870,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1364", zones.back()->m_zone, "zone879", 879,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1365", zones.back()->m_zone, "zone882", 882,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1366", zones.back()->m_zone, "zone881", 881,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 12, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B894", zones.back()->m_zone, "zone675", 675, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1347", zones.back()->m_zone, "zone869", 869, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1366", zones.back()->m_zone, "zone880", 880,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1367", zones.back()->m_zone, "zone884", 884,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1368", zones.back()->m_zone, "zone883", 883,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B655", zones.back()->m_zone, "zone578", 578, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1363", zones.back()->m_zone, "zone879", 879,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1365", zones.back()->m_zone, "zone880", 880,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1369", zones.back()->m_zone, "zone883", 883,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1370", zones.back()->m_zone, "zone900", 900,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B659", zones.back()->m_zone, "zone580", 580, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1368", zones.back()->m_zone, "zone881", 881,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1369", zones.back()->m_zone, "zone882", 882,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1371", zones.back()->m_zone, "zone885", 885,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1372", zones.back()->m_zone, "zone907", 907,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B166", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1367", zones.back()->m_zone, "zone881", 881,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1373", zones.back()->m_zone, "zone885", 885,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1374", zones.back()->m_zone, "zone934", 934,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1375", zones.back()->m_zone, "zone888", 888,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B170", zones.back()->m_zone, "zone809", 809, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1371", zones.back()->m_zone, "zone883", 883,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1373", zones.back()->m_zone, "zone884", 884,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1376", zones.back()->m_zone, "zone903", 903, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1377", zones.back()->m_zone, "zone886", 886,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B164", zones.back()->m_zone, "zone787", 787,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1377", zones.back()->m_zone, "zone885", 885,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1378", zones.back()->m_zone, "zone888", 888,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1379", zones.back()->m_zone, "zone889", 889,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1380", zones.back()->m_zone, "zone473", 473, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "A195", zones.back()->m_zone, "zone976", 976, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A196", zones.back()->m_zone, "zone737", 737, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A197", zones.back()->m_zone, "zone920", 920,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A198", zones.back()->m_zone, "zone47", 47, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A199", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A200", zones.back()->m_zone, "zone898", 898,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B160", zones.back()->m_zone, "zone776", 776, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1375", zones.back()->m_zone, "zone884", 884,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1378", zones.back()->m_zone, "zone886", 886,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1381", zones.back()->m_zone, "zone891", 891,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1382", zones.back()->m_zone, "zone935", 935,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1100", zones.back()->m_zone, "zone764", 764,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1379", zones.back()->m_zone, "zone886", 886,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1383", zones.back()->m_zone, "zone891", 891,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1384", zones.back()->m_zone, "zone890", 890,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1385", zones.back()->m_zone, "zone470", 470, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1088", zones.back()->m_zone, "zone760", 760,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1384", zones.back()->m_zone, "zone889", 889,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1386", zones.back()->m_zone, "zone468", 468,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1387", zones.back()->m_zone, "zone899", 899,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1388", zones.back()->m_zone, "zone892", 892,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1089", zones.back()->m_zone, "zone761", 761,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1381", zones.back()->m_zone, "zone888", 888,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1383", zones.back()->m_zone, "zone889", 889,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1389", zones.back()->m_zone, "zone892", 892,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1390", zones.back()->m_zone, "zone893", 893,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1084", zones.back()->m_zone, "zone759", 759, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1388", zones.back()->m_zone, "zone890", 890,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1389", zones.back()->m_zone, "zone891", 891,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1391", zones.back()->m_zone, "zone894", 894,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1392", zones.back()->m_zone, "zone899", 899,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1153", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1390", zones.back()->m_zone, "zone891", 891,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1393", zones.back()->m_zone, "zone989", 989,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1394", zones.back()->m_zone, "zone895", 895, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1395", zones.back()->m_zone, "zone894", 894,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1080", zones.back()->m_zone, "zone758", 758, Ioss::IJK_t{{-2, -3, 1}},
      Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 101, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1391", zones.back()->m_zone, "zone892", 892,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1395", zones.back()->m_zone, "zone893", 893,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1396", zones.back()->m_zone, "zone896", 896,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1397", zones.back()->m_zone, "zone897", 897,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1141", zones.back()->m_zone, "zone777", 777,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1394", zones.back()->m_zone, "zone893", 893, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1398", zones.back()->m_zone, "zone990", 990,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1399", zones.back()->m_zone, "zone448", 448,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1400", zones.back()->m_zone, "zone896", 896,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1111", zones.back()->m_zone, "zone768", 768,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1396", zones.back()->m_zone, "zone894", 894,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1400", zones.back()->m_zone, "zone895", 895,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1401", zones.back()->m_zone, "zone897", 897,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1402", zones.back()->m_zone, "zone455", 455,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1108", zones.back()->m_zone, "zone767", 767,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1397", zones.back()->m_zone, "zone894", 894,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1401", zones.back()->m_zone, "zone896", 896,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1403", zones.back()->m_zone, "zone899", 899,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1404", zones.back()->m_zone, "zone461", 461, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B200", zones.back()->m_zone, "zone887", 887,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A201", zones.back()->m_zone, "zone965", 965, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A202", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A203", zones.back()->m_zone, "zone909", 909,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A204", zones.back()->m_zone, "zone102", 102,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A205", zones.back()->m_zone, "zone491", 491, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1105", zones.back()->m_zone, "zone766", 766,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1387", zones.back()->m_zone, "zone890", 890,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1392", zones.back()->m_zone, "zone892", 892, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1403", zones.back()->m_zone, "zone897", 897,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1405", zones.back()->m_zone, "zone464", 464, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B719", zones.back()->m_zone, "zone603", 603, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1361", zones.back()->m_zone, "zone878", 878,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1370", zones.back()->m_zone, "zone882", 882,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1406", zones.back()->m_zone, "zone901", 901,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1407", zones.back()->m_zone, "zone907", 907,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B671", zones.back()->m_zone, "zone583", 583,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1359", zones.back()->m_zone, "zone877", 877,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1406", zones.back()->m_zone, "zone900", 900,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1408", zones.back()->m_zone, "zone908", 908,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1409", zones.back()->m_zone, "zone902", 902,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B675", zones.back()->m_zone, "zone584", 584,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{11, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1335", zones.back()->m_zone, "zone864", 864,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1357", zones.back()->m_zone, "zone875", 875,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1409", zones.back()->m_zone, "zone901", 901,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{11, 8, 11}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1410", zones.back()->m_zone, "zone910", 910,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 8, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B172", zones.back()->m_zone, "zone820", 820,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1376", zones.back()->m_zone, "zone885", 885, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1411", zones.back()->m_zone, "zone907", 907,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1412", zones.back()->m_zone, "zone904", 904,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1413", zones.back()->m_zone, "zone484", 484,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B177", zones.back()->m_zone, "zone832", 832,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1412", zones.back()->m_zone, "zone903", 903,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1414", zones.back()->m_zone, "zone905", 905,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1415", zones.back()->m_zone, "zone486", 486,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1416", zones.back()->m_zone, "zone908", 908,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B186", zones.back()->m_zone, "zone854", 854,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1414", zones.back()->m_zone, "zone904", 904,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1417", zones.back()->m_zone, "zone911", 911,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1418", zones.back()->m_zone, "zone910", 910,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1419", zones.back()->m_zone, "zone906", 906,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B194", zones.back()->m_zone, "zone876", 876, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1419", zones.back()->m_zone, "zone905", 905,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1420", zones.back()->m_zone, "zone912", 912,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1421", zones.back()->m_zone, "zone913", 913,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1422", zones.back()->m_zone, "zone533", 533, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B665", zones.back()->m_zone, "zone582", 582,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1372", zones.back()->m_zone, "zone883", 883,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1407", zones.back()->m_zone, "zone900", 900,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1411", zones.back()->m_zone, "zone903", 903,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1423", zones.back()->m_zone, "zone908", 908,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B683", zones.back()->m_zone, "zone589", 589, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1408", zones.back()->m_zone, "zone901", 901,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1416", zones.back()->m_zone, "zone904", 904,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1423", zones.back()->m_zone, "zone907", 907,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1424", zones.back()->m_zone, "zone910", 910, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B203", zones.back()->m_zone, "zone898", 898,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A206", zones.back()->m_zone, "zone954", 954, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A207", zones.back()->m_zone, "zone755", 755, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A208", zones.back()->m_zone, "zone158", 158,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A209", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A210", zones.back()->m_zone, "zone374", 374, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B698", zones.back()->m_zone, "zone595", 595, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1339", zones.back()->m_zone, "zone866", 866, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1410", zones.back()->m_zone, "zone902", 902,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1418", zones.back()->m_zone, "zone905", 905,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1424", zones.back()->m_zone, "zone908", 908, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B182", zones.back()->m_zone, "zone843", 843, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1338", zones.back()->m_zone, "zone866", 866,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1417", zones.back()->m_zone, "zone905", 905,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1425", zones.back()->m_zone, "zone952", 952,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1426", zones.back()->m_zone, "zone912", 912,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B191", zones.back()->m_zone, "zone865", 865,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1420", zones.back()->m_zone, "zone906", 906,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1426", zones.back()->m_zone, "zone911", 911,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1427", zones.back()->m_zone, "zone914", 914,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1428", zones.back()->m_zone, "zone953", 953,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1236", zones.back()->m_zone, "zone817", 817,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1421", zones.back()->m_zone, "zone906", 906,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1429", zones.back()->m_zone, "zone914", 914,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1430", zones.back()->m_zone, "zone915", 915, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1431", zones.back()->m_zone, "zone534", 534, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1221", zones.back()->m_zone, "zone813", 813,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1427", zones.back()->m_zone, "zone912", 912,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1429", zones.back()->m_zone, "zone913", 913,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1432", zones.back()->m_zone, "zone916", 916,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1433", zones.back()->m_zone, "zone922", 922,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1232", zones.back()->m_zone, "zone815", 815, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1430", zones.back()->m_zone, "zone913", 913, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1434", zones.back()->m_zone, "zone916", 916,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1435", zones.back()->m_zone, "zone917", 917,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1436", zones.back()->m_zone, "zone537", 537,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1255", zones.back()->m_zone, "zone824", 824, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1432", zones.back()->m_zone, "zone914", 914,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 6}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1434", zones.back()->m_zone, "zone915", 915,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1437", zones.back()->m_zone, "zone921", 921,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1438", zones.back()->m_zone, "zone917", 917,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1247", zones.back()->m_zone, "zone821", 821,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{12, 101, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1435", zones.back()->m_zone, "zone915", 915, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{11, 8, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1438", zones.back()->m_zone, "zone916", 916, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1439", zones.back()->m_zone, "zone918", 918,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1440", zones.back()->m_zone, "zone541", 541, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1251", zones.back()->m_zone, "zone822", 822,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 101, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1439", zones.back()->m_zone, "zone917", 917,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1441", zones.back()->m_zone, "zone919", 919,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1442", zones.back()->m_zone, "zone921", 921,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1443", zones.back()->m_zone, "zone545", 545, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1307", zones.back()->m_zone, "zone852", 852, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 11}}, Ioss::IJK_t{{1, 101, 1}}, Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1441", zones.back()->m_zone, "zone918", 918,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1444", zones.back()->m_zone, "zone921", 921,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1445", zones.back()->m_zone, "zone923", 923,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{1, 8, 6}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1446", zones.back()->m_zone, "zone547", 547,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{11, 8, 11}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B197", zones.back()->m_zone, "zone887", 887,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A211", zones.back()->m_zone, "zone987", 987, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A212", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A213", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A214", zones.back()->m_zone, "zone739", 739, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A215", zones.back()->m_zone, "zone931", 931, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1254", zones.back()->m_zone, "zone823", 823, Ioss::IJK_t{{2, -3, -1}},
      Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{12, 101, 11}},
      Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1437", zones.back()->m_zone, "zone916", 916,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1442", zones.back()->m_zone, "zone918", 918,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1444", zones.back()->m_zone, "zone919", 919,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1447", zones.back()->m_zone, "zone922", 922,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x11x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B511", zones.back()->m_zone, "zone347", 347, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 12, 6}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1433", zones.back()->m_zone, "zone914", 914,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 12, 1}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1447", zones.back()->m_zone, "zone921", 921,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 12, 6}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{11, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1448", zones.back()->m_zone, "zone1037", 1037,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1449", zones.back()->m_zone, "zone923", 923, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x7x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1312", zones.back()->m_zone, "zone855", 855,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 6}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1445", zones.back()->m_zone, "zone919", 919,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1449", zones.back()->m_zone, "zone922", 922, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{11, 8, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1450", zones.back()->m_zone, "zone1036", 1036,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1451", zones.back()->m_zone, "zone197", 197,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 6}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B116", zones.back()->m_zone, "zone665", 665, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1452", zones.back()->m_zone, "zone925", 925,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1453", zones.back()->m_zone, "zone1004", 1004,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1454", zones.back()->m_zone, "zone933", 933,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1455", zones.back()->m_zone, "zone930", 930, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B95", zones.back()->m_zone, "zone610", 610,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1452", zones.back()->m_zone, "zone924", 924,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1456", zones.back()->m_zone, "zone926", 926,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1457", zones.back()->m_zone, "zone1003", 1003,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1458", zones.back()->m_zone, "zone988", 988,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B434", zones.back()->m_zone, "zone114", 114, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1456", zones.back()->m_zone, "zone925", 925,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1459", zones.back()->m_zone, "zone927", 927,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1460", zones.back()->m_zone, "zone938", 938,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1461", zones.back()->m_zone, "zone929", 929,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B446", zones.back()->m_zone, "zone169", 169, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1459", zones.back()->m_zone, "zone926", 926,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1462", zones.back()->m_zone, "zone936", 936,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1463", zones.back()->m_zone, "zone392", 392, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1464", zones.back()->m_zone, "zone928", 928,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B75", zones.back()->m_zone, "zone565", 565, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1464", zones.back()->m_zone, "zone927", 927,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1465", zones.back()->m_zone, "zone929", 929,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1466", zones.back()->m_zone, "zone995", 995,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1467", zones.back()->m_zone, "zone419", 419, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B90", zones.back()->m_zone, "zone598", 598, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1461", zones.back()->m_zone, "zone926", 926,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1465", zones.back()->m_zone, "zone928", 928,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1468", zones.back()->m_zone, "zone997", 997,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1469", zones.back()->m_zone, "zone930", 930,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B109", zones.back()->m_zone, "zone643", 643,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1455", zones.back()->m_zone, "zone924", 924, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1469", zones.back()->m_zone, "zone929", 929,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1470", zones.back()->m_zone, "zone932", 932,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1471", zones.back()->m_zone, "zone993", 993,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B215", zones.back()->m_zone, "zone920", 920, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A216", zones.back()->m_zone, "zone998", 998, Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A217", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A218", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A219", zones.back()->m_zone, "zone730", 730, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A220", zones.back()->m_zone, "zone943", 943, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B158", zones.back()->m_zone, "zone776", 776, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1470", zones.back()->m_zone, "zone930", 930, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1472", zones.back()->m_zone, "zone933", 933,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1473", zones.back()->m_zone, "zone935", 935,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1474", zones.back()->m_zone, "zone992", 992,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B167", zones.back()->m_zone, "zone798", 798,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1454", zones.back()->m_zone, "zone924", 924,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1472", zones.back()->m_zone, "zone932", 932,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1475", zones.back()->m_zone, "zone934", 934,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1476", zones.back()->m_zone, "zone1008", 1008,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1374", zones.back()->m_zone, "zone884", 884,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1475", zones.back()->m_zone, "zone933", 933,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1477", zones.back()->m_zone, "zone1010", 1010,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1478", zones.back()->m_zone, "zone935", 935,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1382", zones.back()->m_zone, "zone888", 888, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1473", zones.back()->m_zone, "zone932", 932,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1478", zones.back()->m_zone, "zone934", 934,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1479", zones.back()->m_zone, "zone989", 989, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B450", zones.back()->m_zone, "zone180", 180,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1462", zones.back()->m_zone, "zone927", 927,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1480", zones.back()->m_zone, "zone937", 937,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1481", zones.back()->m_zone, "zone971", 971,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1482", zones.back()->m_zone, "zone389", 389,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B437", zones.back()->m_zone, "zone125", 125, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1480", zones.back()->m_zone, "zone936", 936,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1483", zones.back()->m_zone, "zone938", 938,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1484", zones.back()->m_zone, "zone968", 968,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1485", zones.back()->m_zone, "zone983", 983,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B438", zones.back()->m_zone, "zone136", 136,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1460", zones.back()->m_zone, "zone926", 926,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1483", zones.back()->m_zone, "zone937", 937,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1486", zones.back()->m_zone, "zone939", 939,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1487", zones.back()->m_zone, "zone988", 988, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B421", zones.back()->m_zone, "zone69", 69, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1486", zones.back()->m_zone, "zone938", 938,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1488", zones.back()->m_zone, "zone940", 940,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1489", zones.back()->m_zone, "zone986", 986,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1490", zones.back()->m_zone, "zone983", 983,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B406", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1488", zones.back()->m_zone, "zone939", 939,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1491", zones.back()->m_zone, "zone985", 985,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1492", zones.back()->m_zone, "zone945", 945,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1493", zones.back()->m_zone, "zone941", 941,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B409", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1493", zones.back()->m_zone, "zone940", 940,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1494", zones.back()->m_zone, "zone983", 983,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1495", zones.back()->m_zone, "zone963", 963,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1496", zones.back()->m_zone, "zone944", 944, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("A21", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A22", zones.back()->m_zone, "zone1053", 1053, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A23", zones.back()->m_zone, "zone1164", 1164,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A24", zones.back()->m_zone, "zone561", 561, Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A25", zones.back()->m_zone, "zone313", 313, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A26", zones.back()->m_zone, "zone709", 709, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{6, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B220", zones.back()->m_zone, "zone931", 931, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A221", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A222", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A223", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A224", zones.back()->m_zone, "zone726", 726, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A225", zones.back()->m_zone, "zone274", 274, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B395", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1496", zones.back()->m_zone, "zone941", 941, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1497", zones.back()->m_zone, "zone945", 945,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1498", zones.back()->m_zone, "zone962", 962,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1499", zones.back()->m_zone, "zone958", 958, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B396", zones.back()->m_zone, "zone1475", 1475,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1492", zones.back()->m_zone, "zone940", 940,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1497", zones.back()->m_zone, "zone944", 944,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1500", zones.back()->m_zone, "zone946", 946,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1501", zones.back()->m_zone, "zone984", 984, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B379", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1500", zones.back()->m_zone, "zone945", 945,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1502", zones.back()->m_zone, "zone957", 957,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1503", zones.back()->m_zone, "zone955", 955, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1504", zones.back()->m_zone, "zone947", 947,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B28", zones.back()->m_zone, "zone1053", 1053,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1504", zones.back()->m_zone, "zone946", 946,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1505", zones.back()->m_zone, "zone984", 984,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1506", zones.back()->m_zone, "zone1015", 1015,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1507", zones.back()->m_zone, "zone948", 948,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B138", zones.back()->m_zone, "zone721", 721, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1507", zones.back()->m_zone, "zone947", 947,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1508", zones.back()->m_zone, "zone949", 949,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1509", zones.back()->m_zone, "zone951", 951,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1510", zones.back()->m_zone, "zone1021", 1021,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B150", zones.back()->m_zone, "zone754", 754,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1508", zones.back()->m_zone, "zone948", 948,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1511", zones.back()->m_zone, "zone950", 950,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1512", zones.back()->m_zone, "zone1033", 1033,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1513", zones.back()->m_zone, "zone955", 955,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 6, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B190", zones.back()->m_zone, "zone865", 865, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1511", zones.back()->m_zone, "zone949", 949,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1514", zones.back()->m_zone, "zone1034", 1034,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1515", zones.back()->m_zone, "zone953", 953,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1516", zones.back()->m_zone, "zone951", 951,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x10 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B183", zones.back()->m_zone, "zone843", 843, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1509", zones.back()->m_zone, "zone948", 948,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1516", zones.back()->m_zone, "zone950", 950,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}}, Ioss::IJK_t{{1, 6, 11}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1517", zones.back()->m_zone, "zone952", 952,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1518", zones.back()->m_zone, "zone1022", 1022,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 11}}, Ioss::IJK_t{{1, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1425", zones.back()->m_zone, "zone911", 911, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{11, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1517", zones.back()->m_zone, "zone951", 951,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1519", zones.back()->m_zone, "zone1027", 1027,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1520", zones.back()->m_zone, "zone953", 953,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1428", zones.back()->m_zone, "zone912", 912,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1515", zones.back()->m_zone, "zone950", 950,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1520", zones.back()->m_zone, "zone952", 952,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 11, 6}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 11, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1521", zones.back()->m_zone, "zone1037", 1037,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B206", zones.back()->m_zone, "zone909", 909,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A226", zones.back()->m_zone, "zone965", 965,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A227", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A228", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A229", zones.back()->m_zone, "zone363", 363, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A230", zones.back()->m_zone, "zone752", 752, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x5 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B18", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1503", zones.back()->m_zone, "zone946", 946, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 6}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1513", zones.back()->m_zone, "zone949", 949,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1522", zones.back()->m_zone, "zone956", 956,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1523", zones.back()->m_zone, "zone1029", 1029,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{51, 6, 6}}, Ioss::IJK_t{{1, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B10", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1522", zones.back()->m_zone, "zone955", 955,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1524", zones.back()->m_zone, "zone957", 957, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1525", zones.back()->m_zone, "zone244", 244, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1526", zones.back()->m_zone, "zone1028", 1028,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B380", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1502", zones.back()->m_zone, "zone946", 946,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1524", zones.back()->m_zone, "zone956", 956, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1527", zones.back()->m_zone, "zone958", 958,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1528", zones.back()->m_zone, "zone255", 255, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B384", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1499", zones.back()->m_zone, "zone944", 944, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1527", zones.back()->m_zone, "zone957", 957,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1529", zones.back()->m_zone, "zone959", 959,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1530", zones.back()->m_zone, "zone257", 257,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B313", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1529", zones.back()->m_zone, "zone958", 958,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1531", zones.back()->m_zone, "zone960", 960,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1532", zones.back()->m_zone, "zone962", 962,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1533", zones.back()->m_zone, "zone277", 277,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B334", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1531", zones.back()->m_zone, "zone959", 959,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1534", zones.back()->m_zone, "zone961", 961,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1535", zones.back()->m_zone, "zone980", 980,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1536", zones.back()->m_zone, "zone288", 288, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B330", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1534", zones.back()->m_zone, "zone960", 960,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1537", zones.back()->m_zone, "zone979", 979,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1538", zones.back()->m_zone, "zone964", 964,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1539", zones.back()->m_zone, "zone962", 962,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B309", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1498", zones.back()->m_zone, "zone944", 944,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1532", zones.back()->m_zone, "zone959", 959,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1539", zones.back()->m_zone, "zone961", 961,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1540", zones.back()->m_zone, "zone963", 963,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B305", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1495", zones.back()->m_zone, "zone941", 941,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1540", zones.back()->m_zone, "zone962", 962,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1541", zones.back()->m_zone, "zone964", 964,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1542", zones.back()->m_zone, "zone967", 967,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B328", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1538", zones.back()->m_zone, "zone961", 961,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1541", zones.back()->m_zone, "zone963", 963,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1543", zones.back()->m_zone, "zone966", 966,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1544", zones.back()->m_zone, "zone978", 978, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B201", zones.back()->m_zone, "zone898", 898,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B226", zones.back()->m_zone, "zone954", 954,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A231", zones.back()->m_zone, "zone748", 748, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A232", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A233", zones.back()->m_zone, "zone976", 976,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A234", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B324", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1543", zones.back()->m_zone, "zone964", 964,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1545", zones.back()->m_zone, "zone977", 977, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1546", zones.back()->m_zone, "zone969", 969,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1547", zones.back()->m_zone, "zone967", 967,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B294", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1542", zones.back()->m_zone, "zone963", 963,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1547", zones.back()->m_zone, "zone966", 966,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1548", zones.back()->m_zone, "zone983", 983,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1549", zones.back()->m_zone, "zone968", 968,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B297", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1484", zones.back()->m_zone, "zone937", 937,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1549", zones.back()->m_zone, "zone967", 967,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1550", zones.back()->m_zone, "zone969", 969,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1551", zones.back()->m_zone, "zone971", 971,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B322", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1546", zones.back()->m_zone, "zone966", 966,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1550", zones.back()->m_zone, "zone968", 968,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1552", zones.back()->m_zone, "zone970", 970,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1553", zones.back()->m_zone, "zone973", 973,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B317", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1552", zones.back()->m_zone, "zone969", 969,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 31, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1554", zones.back()->m_zone, "zone972", 972,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1555", zones.back()->m_zone, "zone366", 366, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1556", zones.back()->m_zone, "zone971", 971,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{51, 31, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B302", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1481", zones.back()->m_zone, "zone936", 936,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1551", zones.back()->m_zone, "zone968", 968,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1556", zones.back()->m_zone, "zone970", 970,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1557", zones.back()->m_zone, "zone378", 378,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B352", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1554", zones.back()->m_zone, "zone970", 970,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1558", zones.back()->m_zone, "zone973", 973,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1559", zones.back()->m_zone, "zone348", 348,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1560", zones.back()->m_zone, "zone974", 974,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B349", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1553", zones.back()->m_zone, "zone969", 969,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1558", zones.back()->m_zone, "zone972", 972,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1561", zones.back()->m_zone, "zone977", 977,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1562", zones.back()->m_zone, "zone974", 974,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B355", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1560", zones.back()->m_zone, "zone972", 972, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1562", zones.back()->m_zone, "zone973", 973,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1563", zones.back()->m_zone, "zone344", 344, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1564", zones.back()->m_zone, "zone975", 975, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B357", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1564", zones.back()->m_zone, "zone974", 974, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1565", zones.back()->m_zone, "zone977", 977,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1566", zones.back()->m_zone, "zone333", 333, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1567", zones.back()->m_zone, "zone982", 982, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B195", zones.back()->m_zone, "zone887", 887,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B233", zones.back()->m_zone, "zone965", 965,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A235", zones.back()->m_zone, "zone987", 987,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A236", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A237", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A238", zones.back()->m_zone, "zone741", 741, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B345", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1545", zones.back()->m_zone, "zone966", 966,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1561", zones.back()->m_zone, "zone973", 973,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1565", zones.back()->m_zone, "zone975", 975,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1568", zones.back()->m_zone, "zone978", 978,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B344", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1544", zones.back()->m_zone, "zone964", 964,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1568", zones.back()->m_zone, "zone977", 977,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1569", zones.back()->m_zone, "zone982", 982,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1570", zones.back()->m_zone, "zone979", 979,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B340", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1537", zones.back()->m_zone, "zone961", 961,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1570", zones.back()->m_zone, "zone978", 978,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1571", zones.back()->m_zone, "zone980", 980,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1572", zones.back()->m_zone, "zone981", 981,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B336", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1535", zones.back()->m_zone, "zone960", 960,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1571", zones.back()->m_zone, "zone979", 979,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1573", zones.back()->m_zone, "zone300", 300, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1574", zones.back()->m_zone, "zone981", 981,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B363", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1572", zones.back()->m_zone, "zone979", 979,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1574", zones.back()->m_zone, "zone980", 980, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1575", zones.back()->m_zone, "zone310", 310, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1576", zones.back()->m_zone, "zone982", 982,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B361", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1567", zones.back()->m_zone, "zone975", 975, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1569", zones.back()->m_zone, "zone978", 978,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1576", zones.back()->m_zone, "zone981", 981,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1577", zones.back()->m_zone, "zone312", 312,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B418", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1485", zones.back()->m_zone, "zone937", 937,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1490", zones.back()->m_zone, "zone939", 939,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1494", zones.back()->m_zone, "zone941", 941,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 31}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1548", zones.back()->m_zone, "zone967", 967,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{51, 8, 31}}, Ioss::IJK_t{{51, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B37", zones.back()->m_zone, "zone1275", 1275,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1501", zones.back()->m_zone, "zone945", 945, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1505", zones.back()->m_zone, "zone947", 947,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1578", zones.back()->m_zone, "zone985", 985,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1579", zones.back()->m_zone, "zone1014", 1014,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B48", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1491", zones.back()->m_zone, "zone940", 940,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1578", zones.back()->m_zone, "zone984", 984,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1580", zones.back()->m_zone, "zone986", 986,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1581", zones.back()->m_zone, "zone1013", 1013,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B55", zones.back()->m_zone, "zone224", 224, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{51, 1, 1}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1489", zones.back()->m_zone, "zone939", 939,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1580", zones.back()->m_zone, "zone985", 985,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1582", zones.back()->m_zone, "zone988", 988,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1583", zones.back()->m_zone, "zone999", 999,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B211", zones.back()->m_zone, "zone920", 920,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B235", zones.back()->m_zone, "zone976", 976,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A239", zones.back()->m_zone, "zone744", 744, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A240", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A241", zones.back()->m_zone, "zone998", 998,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A242", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B65", zones.back()->m_zone, "zone446", 446, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1458", zones.back()->m_zone, "zone925", 925,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1487", zones.back()->m_zone, "zone938", 938, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1582", zones.back()->m_zone, "zone986", 986,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1584", zones.back()->m_zone, "zone1001", 1001,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1393", zones.back()->m_zone, "zone893", 893,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1479", zones.back()->m_zone, "zone935", 935,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1585", zones.back()->m_zone, "zone990", 990, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1586", zones.back()->m_zone, "zone992", 992,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1398", zones.back()->m_zone, "zone895", 895,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1585", zones.back()->m_zone, "zone989", 989, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1587", zones.back()->m_zone, "zone450", 450,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1588", zones.back()->m_zone, "zone991", 991,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1142", zones.back()->m_zone, "zone777", 777,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1588", zones.back()->m_zone, "zone990", 990,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1589", zones.back()->m_zone, "zone992", 992,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1590", zones.back()->m_zone, "zone449", 449,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1591", zones.back()->m_zone, "zone994", 994,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1152", zones.back()->m_zone, "zone780", 780,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1474", zones.back()->m_zone, "zone932", 932,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1586", zones.back()->m_zone, "zone989", 989,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1589", zones.back()->m_zone, "zone991", 991,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1592", zones.back()->m_zone, "zone993", 993,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1156", zones.back()->m_zone, "zone782", 782,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1471", zones.back()->m_zone, "zone930", 930,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B1592", zones.back()->m_zone, "zone992", 992,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1593", zones.back()->m_zone, "zone994", 994,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1594", zones.back()->m_zone, "zone997", 997,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1149", zones.back()->m_zone, "zone779", 779,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1591", zones.back()->m_zone, "zone991", 991,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1593", zones.back()->m_zone, "zone993", 993,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1595", zones.back()->m_zone, "zone996", 996,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1596", zones.back()->m_zone, "zone440", 440, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1124", zones.back()->m_zone, "zone771", 771,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1466", zones.back()->m_zone, "zone928", 928,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1597", zones.back()->m_zone, "zone996", 996,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1598", zones.back()->m_zone, "zone997", 997, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{51, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1599", zones.back()->m_zone, "zone427", 427, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1134", zones.back()->m_zone, "zone774", 774,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1595", zones.back()->m_zone, "zone994", 994,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1597", zones.back()->m_zone, "zone995", 995,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1600", zones.back()->m_zone, "zone997", 997,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1601", zones.back()->m_zone, "zone432", 432,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1114", zones.back()->m_zone, "zone769", 769,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1468", zones.back()->m_zone, "zone929", 929,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1594", zones.back()->m_zone, "zone993", 993,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1598", zones.back()->m_zone, "zone995", 995, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1600", zones.back()->m_zone, "zone996", 996,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B216", zones.back()->m_zone, "zone931", 931,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B241", zones.back()->m_zone, "zone987", 987,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A243", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A244", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A245", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A246", zones.back()->m_zone, "zone716", 716, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B822", zones.back()->m_zone, "zone642", 642,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1583", zones.back()->m_zone, "zone986", 986,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1602", zones.back()->m_zone, "zone1000", 1000,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1603", zones.back()->m_zone, "zone1013", 1013,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1604", zones.back()->m_zone, "zone1001", 1001,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B866", zones.back()->m_zone, "zone661", 661, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1602", zones.back()->m_zone, "zone999", 999,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1605", zones.back()->m_zone, "zone1017", 1017,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1606", zones.back()->m_zone, "zone1005", 1005,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1607", zones.back()->m_zone, "zone1002", 1002,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B834", zones.back()->m_zone, "zone647", 647,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1584", zones.back()->m_zone, "zone988", 988,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1604", zones.back()->m_zone, "zone999", 999,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1608", zones.back()->m_zone, "zone1002", 1002,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1609", zones.back()->m_zone, "zone1003", 1003,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B874", zones.back()->m_zone, "zone666", 666, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1607", zones.back()->m_zone, "zone1000", 1000,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1608", zones.back()->m_zone, "zone1001", 1001,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1610", zones.back()->m_zone, "zone1003", 1003,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1611", zones.back()->m_zone, "zone1038", 1038,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B612", zones.back()->m_zone, "zone564", 564,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1457", zones.back()->m_zone, "zone925", 925,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1609", zones.back()->m_zone, "zone1001", 1001,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1610", zones.back()->m_zone, "zone1002", 1002,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1612", zones.back()->m_zone, "zone1004", 1004,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B887", zones.back()->m_zone, "zone671", 671,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1453", zones.back()->m_zone, "zone924", 924,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1612", zones.back()->m_zone, "zone1003", 1003,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1613", zones.back()->m_zone, "zone1038", 1038,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1614", zones.back()->m_zone, "zone1008", 1008,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B870", zones.back()->m_zone, "zone663", 663,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1606", zones.back()->m_zone, "zone1000", 1000,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1615", zones.back()->m_zone, "zone1018", 1018,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1616", zones.back()->m_zone, "zone1038", 1038,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1617", zones.back()->m_zone, "zone1006", 1006,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B754", zones.back()->m_zone, "zone617", 617, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1617", zones.back()->m_zone, "zone1005", 1005,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1618", zones.back()->m_zone, "zone1024", 1024,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1619", zones.back()->m_zone, "zone1012", 1012,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1620", zones.back()->m_zone, "zone1007", 1007,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B762", zones.back()->m_zone, "zone619", 619, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1620", zones.back()->m_zone, "zone1006", 1006,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1621", zones.back()->m_zone, "zone1008", 1008,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1622", zones.back()->m_zone, "zone1038", 1038,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1623", zones.back()->m_zone, "zone1011", 1011,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B882", zones.back()->m_zone, "zone669", 669,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1476", zones.back()->m_zone, "zone933", 933,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1614", zones.back()->m_zone, "zone1004", 1004,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1621", zones.back()->m_zone, "zone1007", 1007,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1624", zones.back()->m_zone, "zone1010", 1010,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B221", zones.back()->m_zone, "zone943", 943,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B243", zones.back()->m_zone, "zone998", 998,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A247", zones.back()->m_zone, "zone718", 718, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A248", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A249", zones.back()->m_zone, "zone285", 285, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A250", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1345", zones.back()->m_zone, "zone869", 869,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1477", zones.back()->m_zone, "zone934", 934,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1624", zones.back()->m_zone, "zone1008", 1008,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1625", zones.back()->m_zone, "zone1011", 1011,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1343", zones.back()->m_zone, "zone868", 868,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1623", zones.back()->m_zone, "zone1007", 1007,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1625", zones.back()->m_zone, "zone1010", 1010,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1626", zones.back()->m_zone, "zone1012", 1012,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1330", zones.back()->m_zone, "zone862", 862,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1619", zones.back()->m_zone, "zone1006", 1006,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1626", zones.back()->m_zone, "zone1011", 1011,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1627", zones.back()->m_zone, "zone1025", 1025,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B809", zones.back()->m_zone, "zone638", 638,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1581", zones.back()->m_zone, "zone985", 985,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B1603", zones.back()->m_zone, "zone999", 999, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1628", zones.back()->m_zone, "zone1017", 1017,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1629", zones.back()->m_zone, "zone1014", 1014,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B797", zones.back()->m_zone, "zone634", 634,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1579", zones.back()->m_zone, "zone984", 984,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1629", zones.back()->m_zone, "zone1013", 1013,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1630", zones.back()->m_zone, "zone1016", 1016,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1631", zones.back()->m_zone, "zone1015", 1015,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B786", zones.back()->m_zone, "zone629", 629,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1506", zones.back()->m_zone, "zone947", 947,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1631", zones.back()->m_zone, "zone1014", 1014,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1632", zones.back()->m_zone, "zone1016", 1016,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1633", zones.back()->m_zone, "zone1021", 1021,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B846", zones.back()->m_zone, "zone652", 652,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1630", zones.back()->m_zone, "zone1014", 1014,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1632", zones.back()->m_zone, "zone1015", 1015,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1634", zones.back()->m_zone, "zone1017", 1017,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1635", zones.back()->m_zone, "zone1019", 1019,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B856", zones.back()->m_zone, "zone657", 657,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1605", zones.back()->m_zone, "zone1000", 1000,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1628", zones.back()->m_zone, "zone1013", 1013,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1634", zones.back()->m_zone, "zone1016", 1016,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1636", zones.back()->m_zone, "zone1018", 1018,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B861", zones.back()->m_zone, "zone659", 659, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1615", zones.back()->m_zone, "zone1005", 1005,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1636", zones.back()->m_zone, "zone1017", 1017,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1637", zones.back()->m_zone, "zone1019", 1019,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1638", zones.back()->m_zone, "zone1024", 1024,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B850", zones.back()->m_zone, "zone655", 655, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1635", zones.back()->m_zone, "zone1016", 1016,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1637", zones.back()->m_zone, "zone1018", 1018,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1639", zones.back()->m_zone, "zone1021", 1021,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1640", zones.back()->m_zone, "zone1023", 1023,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B248", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A251", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A252", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A253", zones.back()->m_zone, "zone295", 295,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A254", zones.back()->m_zone, "zone699", 699, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A255", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B776", zones.back()->m_zone, "zone625", 625,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1510", zones.back()->m_zone, "zone948", 948,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1633", zones.back()->m_zone, "zone1015", 1015,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1639", zones.back()->m_zone, "zone1019", 1019,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1641", zones.back()->m_zone, "zone1022", 1022,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B770", zones.back()->m_zone, "zone623", 623,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1518", zones.back()->m_zone, "zone951", 951,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1641", zones.back()->m_zone, "zone1021", 1021,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1642", zones.back()->m_zone, "zone1023", 1023,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1643", zones.back()->m_zone, "zone1027", 1027,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B595", zones.back()->m_zone, "zone559", 559,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1640", zones.back()->m_zone, "zone1019", 1019,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1642", zones.back()->m_zone, "zone1022", 1022,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1644", zones.back()->m_zone, "zone1024", 1024,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1645", zones.back()->m_zone, "zone1026", 1026,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B767", zones.back()->m_zone, "zone622", 622,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1618", zones.back()->m_zone, "zone1006", 1006,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1638", zones.back()->m_zone, "zone1018", 1018,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1644", zones.back()->m_zone, "zone1023", 1023,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1646", zones.back()->m_zone, "zone1025", 1025,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1328", zones.back()->m_zone, "zone861", 861,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1627", zones.back()->m_zone, "zone1012", 1012,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1646", zones.back()->m_zone, "zone1024", 1024,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1647", zones.back()->m_zone, "zone1026", 1026,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1323", zones.back()->m_zone, "zone859", 859,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1645", zones.back()->m_zone, "zone1023", 1023,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1647", zones.back()->m_zone, "zone1025", 1025,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1648", zones.back()->m_zone, "zone1027", 1027,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1313", zones.back()->m_zone, "zone856", 856,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1519", zones.back()->m_zone, "zone952", 952,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1643", zones.back()->m_zone, "zone1022", 1022,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1648", zones.back()->m_zone, "zone1026", 1026,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x7x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1298", zones.back()->m_zone, "zone848", 848,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1526", zones.back()->m_zone, "zone956", 956,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1649", zones.back()->m_zone, "zone1030", 1030,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 8, 1}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1650", zones.back()->m_zone, "zone1029", 1029,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1651", zones.back()->m_zone, "zone226", 226,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{51, 8, 12}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1303", zones.back()->m_zone, "zone850", 850,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1523", zones.back()->m_zone, "zone955", 955,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1650", zones.back()->m_zone, "zone1028", 1028,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1652", zones.back()->m_zone, "zone1030", 1030,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1653", zones.back()->m_zone, "zone1033", 1033,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B467", zones.back()->m_zone, "zone225", 225,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1649", zones.back()->m_zone, "zone1028", 1028,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1652", zones.back()->m_zone, "zone1029", 1029,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 6, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1654", zones.back()->m_zone, "zone1032", 1032,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1655", zones.back()->m_zone, "zone218", 218, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B245", zones.back()->m_zone, "zone998", 998,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B251", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A256", zones.back()->m_zone, "zone701", 701, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A257", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A258", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A259", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B458", zones.back()->m_zone, "zone202", 202, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1654", zones.back()->m_zone, "zone1030", 1030,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1656", zones.back()->m_zone, "zone1033", 1033,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1657", zones.back()->m_zone, "zone211", 211,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1658", zones.back()->m_zone, "zone1035", 1035,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1257", zones.back()->m_zone, "zone825", 825,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1512", zones.back()->m_zone, "zone949", 949,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1653", zones.back()->m_zone, "zone1029", 1029,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1656", zones.back()->m_zone, "zone1032", 1032,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1659", zones.back()->m_zone, "zone1034", 1034,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B510", zones.back()->m_zone, "zone347", 347,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1514", zones.back()->m_zone, "zone950", 950,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1659", zones.back()->m_zone, "zone1033", 1033,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1660", zones.back()->m_zone, "zone1035", 1035,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1661", zones.back()->m_zone, "zone1037", 1037,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1310", zones.back()->m_zone, "zone855", 855,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1658", zones.back()->m_zone, "zone1032", 1032,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1660", zones.back()->m_zone, "zone1034", 1034,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1662", zones.back()->m_zone, "zone1036", 1036,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1663", zones.back()->m_zone, "zone205", 205, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1450", zones.back()->m_zone, "zone923", 923,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1662", zones.back()->m_zone, "zone1035", 1035,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1664", zones.back()->m_zone, "zone195", 195,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1665", zones.back()->m_zone, "zone1037", 1037,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1448", zones.back()->m_zone, "zone922", 922,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 12, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1521", zones.back()->m_zone, "zone953", 953,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{51, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1661", zones.back()->m_zone, "zone1034", 1034,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 12}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1665", zones.back()->m_zone, "zone1036", 1036,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "50x10x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B879", zones.back()->m_zone, "zone668", 668,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1611", zones.back()->m_zone, "zone1002", 1002,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1613", zones.back()->m_zone, "zone1004", 1004,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 11, 1}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1616", zones.back()->m_zone, "zone1005", 1005,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{51, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1622", zones.back()->m_zone, "zone1007", 1007,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{51, 1, 8}}, Ioss::IJK_t{{51, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("A1666", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1667", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1668", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1669", zones.back()->m_zone, "zone240", 240, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1670", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1671", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1667", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1672", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1673", zones.back()->m_zone, "zone110", 110, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1674", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1675", zones.back()->m_zone, "zone243", 243, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1676", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1666", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1677", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1678", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1679", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1680", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1681", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B240", zones.back()->m_zone, "zone987", 987, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B257", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A260", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A261", zones.back()->m_zone, "zone707", 707, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A262", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A263", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1672", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1681", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1682", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1683", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1684", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1685", zones.back()->m_zone, "zone109", 109,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("A1686", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1687", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1688", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1689", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1690", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1691", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1690", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1692", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1693", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1694", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1695", zones.back()->m_zone, "zone101", 101,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1696", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1688", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1697", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1698", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1699", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1700", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1701", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1693", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1700", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1702", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1703", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1704", zones.back()->m_zone, "zone140", 140,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1705", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1699", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1706", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1707", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1708", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1709", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1710", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1703", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1707", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1711", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1712", zones.back()->m_zone, "zone141", 141, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1713", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1714", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1706", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1715", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1716", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1717", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1718", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1719", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1711", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1718", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1720", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1721", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1722", zones.back()->m_zone, "zone142", 142,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1723", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1717", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1724", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1725", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1726", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1727", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1728", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B22", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A27", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A28", zones.back()->m_zone, "zone947", 947, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A29", zones.back()->m_zone, "zone1275", 1275,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A30", zones.back()->m_zone, "zone631", 631, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A31", zones.back()->m_zone, "zone721", 721, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 12}}, Ioss::IJK_t{{6, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B237", zones.back()->m_zone, "zone976", 976, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B260", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A264", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A265", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A266", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A267", zones.back()->m_zone, "zone710", 710, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1721", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1727", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1729", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1730", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1731", zones.back()->m_zone, "zone143", 143,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1732", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("A1733", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1734", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1735", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1736", zones.back()->m_zone, "zone408", 408, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1737", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1738", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1737", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1739", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1740", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1741", zones.back()->m_zone, "zone416", 416,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1742", zones.back()->m_zone, "zone84", 84,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1743", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1724", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1744", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1745", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1746", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1747", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1748", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1733", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1749", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1750", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1751", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1752", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1753", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1739", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1752", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1754", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1755", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1756", zones.back()->m_zone, "zone85", 85, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1757", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1729", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1745", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1758", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1759", zones.back()->m_zone, "zone81", 81, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A1760", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1761", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back("A1762", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1763", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1764", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1765", zones.back()->m_zone, "zone492", 492,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1766", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1767", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1753", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1762", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1768", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1769", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1770", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1771", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1757", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1770", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 11, 5}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1772", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1773", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1774", zones.back()->m_zone, "zone86", 86,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 1, 11}},
                                                Ioss::IJK_t{{51, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1775", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B232", zones.back()->m_zone, "zone965", 965,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B266", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A268", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A269", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A270", zones.back()->m_zone, "zone746", 746, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A271", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1748", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1771", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1776", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1777", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1778", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1779", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1761", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1772", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1777", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1780", zones.back()->m_zone, "zone79", 79,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1781", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1782", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1763", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1779", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1783", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1784", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1785", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1786", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1785", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 5, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1787", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1788", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1789", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1790", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1791", zones.back()->m_zone, "zone499", 499,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1789", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1792", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 6, 8}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1793", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1794", zones.back()->m_zone, "zone508", 508,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1795", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1796", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1691", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1797", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1798", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1799", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1800", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1801", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1696", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1799", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1802", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1803", zones.back()->m_zone, "zone103", 103,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1804", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1805", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1792", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1801", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1806", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1807", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1808", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1809", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1679", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B1800", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1810", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1811", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1812", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1813", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1683", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1804", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1811", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 11, 5}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1814", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1815", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1816", zones.back()->m_zone, "zone104", 104,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B228", zones.back()->m_zone, "zone954", 954,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B268", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A272", zones.back()->m_zone, "zone750", 750, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A273", zones.back()->m_zone, "zone352", 352, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A274", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A275", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x4x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1809", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1813", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 1}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1817", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 5, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A1818", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1819", zones.back()->m_zone, "zone520", 520,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1820", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1773", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1821", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1822", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1823", zones.back()->m_zone, "zone87", 87,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1824", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1825", zones.back()->m_zone, "zone43", 43, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1767", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1821", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1826", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1827", zones.back()->m_zone, "zone489", 489,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1828", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1829", zones.back()->m_zone, "zone42", 42, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1782", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1824", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1830", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1831", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1832", zones.back()->m_zone, "zone40", 40, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1833", zones.back()->m_zone, "zone88", 88, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 12}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1786", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 5, 1}},
                                                Ioss::IJK_t{{1, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1826", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1830", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1834", zones.back()->m_zone, "zone41", 41, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1835", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1836", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1787", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1835", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1837", zones.back()->m_zone, "zone497", 497,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1838", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1839", zones.back()->m_zone, "zone59", 59,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1840", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1793", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1838", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1841", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1842", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1843", zones.back()->m_zone, "zone60", 60,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1844", zones.back()->m_zone, "zone506", 506,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1805", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1845", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1846", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1847", zones.back()->m_zone, "zone66", 66, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1848", zones.back()->m_zone, "zone106", 106,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1849", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1808", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1841", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1845", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1850", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1851", zones.back()->m_zone, "zone61", 61, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1852", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1815", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B1846", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A1853", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1854", zones.back()->m_zone, "zone105", 105,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A1855", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{2, -3, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1856", zones.back()->m_zone, "zone67", 67, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 12}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B271", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B275", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A276", zones.back()->m_zone, "zone712", 712, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A277", zones.back()->m_zone, "zone340", 340, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A278", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A279", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1820", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 5, 1}},
                                                Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1852", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1853", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1857", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{-2, -3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1858", zones.back()->m_zone, "zone528", 528,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1859", zones.back()->m_zone, "zone62", 62, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 11, 1}},
      Ioss::IJK_t{{6, 11, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("A1860", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1861", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1862", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1863", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1864", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1865", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1865", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1866", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1867", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1868", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1869", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1870", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1868", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1871", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1872", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1873", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1874", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1875", zones.back()->m_zone, "zone373", 373,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1862", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1876", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1877", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1878", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1879", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1880", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1880", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1881", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 31, 8}},
                                                Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1882", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1883", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1884", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1885", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1885", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1886", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1887", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1888", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1889", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1890", zones.back()->m_zone, "zone273", 273,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1871", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1891", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1892", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1893", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1894", zones.back()->m_zone, "zone362", 362, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1895", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1866", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1891", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1896", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1897", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1898", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1899", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1860", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1898", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1900", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1901", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1902", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1903", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B265", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B279", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A280", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A281", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A282", zones.back()->m_zone, "zone329", 329,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A283", zones.back()->m_zone, "zone714", 714, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1876", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1900", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1904", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1905", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1906", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1907", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1881", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1906", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1908", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1909", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1910", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1911", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x30x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1886", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1908", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1912", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1913", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1914", zones.back()->m_zone, "zone284", 284, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1915", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1913", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1916", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1917", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1918", zones.back()->m_zone, "zone296", 296,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1919", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1920", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1910", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1916", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1921", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1922", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1923", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1924", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1905", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1922", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1925", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1926", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1927", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1928", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1902", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1925", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1929", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1930", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1931", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1932", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1897", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1931", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1933", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1934", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1935", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1936", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1893", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1933", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1937", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1938", zones.back()->m_zone, "zone351", 351, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1939", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1940", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1936", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1940", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1941", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1942", zones.back()->m_zone, "zone341", 341, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1943", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1944", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B262", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B281", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A284", zones.back()->m_zone, "zone705", 705, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A285", zones.back()->m_zone, "zone317", 317, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A286", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A287", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1930", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1944", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1945", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1946", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1947", zones.back()->m_zone, "zone328", 328,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1948", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1927", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1946", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1949", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1950", zones.back()->m_zone, "zone318", 318, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1951", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1952", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1920", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1924", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1952", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1953", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1954", zones.back()->m_zone, "zone307", 307, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1955", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1861", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1956", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1957", zones.back()->m_zone, "zone122", 122, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1958", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1959", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1960", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1870", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1960", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1961", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1962", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1963", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1964", zones.back()->m_zone, "zone123", 123,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1872", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1962", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1965", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1966", zones.back()->m_zone, "zone127", 127, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1967", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1968", zones.back()->m_zone, "zone376", 376,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1879", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1958", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1969", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1970", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1971", zones.back()->m_zone, "zone119", 119,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1972", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1884", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1972", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1973", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 31, 8}},
                                                Ioss::IJK_t{{12, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1974", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1975", zones.back()->m_zone, "zone118", 118,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1976", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1889", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B1976", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1977", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1978", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1979", zones.back()->m_zone, "zone115", 115,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A1980", zones.back()->m_zone, "zone275", 275,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1895", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1965", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1981", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1982", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1983", zones.back()->m_zone, "zone365", 365, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1984", zones.back()->m_zone, "zone126", 126,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B255", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B259", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B287", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A288", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A289", zones.back()->m_zone, "zone306", 306, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A290", zones.back()->m_zone, "zone703", 703, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1896", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1961", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1981", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1985", zones.back()->m_zone, "zone124", 124, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1986", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1987", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1903", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1956", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1987", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1988", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1989", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1990", zones.back()->m_zone, "zone121", 121,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1904", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1969", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1988", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1991", zones.back()->m_zone, "zone120", 120, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1992", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1993", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1911", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1973", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1993", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1994", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A1995", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1996", zones.back()->m_zone, "zone117", 117,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1912", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1977", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1994", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1997", zones.back()->m_zone, "zone116", 116, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A1998", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A1999", zones.back()->m_zone, "zone286", 286, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1919", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1998", zones.back()->m_zone, "zone1125", 1125,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2000", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2001", zones.back()->m_zone, "zone298", 298,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2002", zones.back()->m_zone, "zone135", 135,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2003", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1921", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1995", zones.back()->m_zone, "zone1124", 1124,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2000", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2004", zones.back()->m_zone, "zone134", 134, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2005", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2006", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1926", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1992", zones.back()->m_zone, "zone1123", 1123,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2005", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2007", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2008", zones.back()->m_zone, "zone133", 133, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2009", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1932", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1989", zones.back()->m_zone, "zone1122", 1122,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2007", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2010", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2011", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2012", zones.back()->m_zone, "zone132", 132,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1935", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1986", zones.back()->m_zone, "zone1121", 1121,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2011", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2013", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2014", zones.back()->m_zone, "zone129", 129,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2015", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B199", zones.back()->m_zone, "zone887", 887, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A291", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A292", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A293", zones.back()->m_zone, "zone58", 58, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A294", zones.back()->m_zone, "zone967", 967, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A295", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1937", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B1982", zones.back()->m_zone, "zone1119", 1119,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2013", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2016", zones.back()->m_zone, "zone128", 128, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2017", zones.back()->m_zone, "zone349", 349, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2018", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1941", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2015", zones.back()->m_zone, "zone1130", 1130,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2018", zones.back()->m_zone, "zone1132", 1132,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2019", zones.back()->m_zone, "zone130", 130, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2020", zones.back()->m_zone, "zone343", 343, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2021", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1948", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2010", zones.back()->m_zone, "zone1129", 1129,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2021", zones.back()->m_zone, "zone1133", 1133,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2022", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2023", zones.back()->m_zone, "zone330", 330,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2024", zones.back()->m_zone, "zone131", 131,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{51, 8, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1949", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2009", zones.back()->m_zone, "zone1128", 1128,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2022", zones.back()->m_zone, "zone1134", 1134,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2025", zones.back()->m_zone, "zone138", 138, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2026", zones.back()->m_zone, "zone316", 316, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2027", zones.back()->m_zone, "zone1136", 1136,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1955", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2003", zones.back()->m_zone, "zone1126", 1126,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2006", zones.back()->m_zone, "zone1127", 1127,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2027", zones.back()->m_zone, "zone1135", 1135,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2028", zones.back()->m_zone, "zone308", 308, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2029", zones.back()->m_zone, "zone137", 137,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1677", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1686", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2030", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2031", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2032", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2033", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1668", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2032", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2034", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2035", zones.back()->m_zone, "zone252", 252,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2036", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2037", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1888", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2037", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2038", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2039", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2040", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2041", zones.back()->m_zone, "zone263", 263,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1682", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1692", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2031", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2042", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2043", zones.back()->m_zone, "zone100", 100, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2044", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1674", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2036", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2044", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2045", zones.back()->m_zone, "zone254", 254,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2046", zones.back()->m_zone, "zone111", 111,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2047", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B202", zones.back()->m_zone, "zone898", 898, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B295", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A296", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A297", zones.back()->m_zone, "zone968", 968, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A298", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A299", zones.back()->m_zone, "zone125", 125,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1978", zones.back()->m_zone, "zone1118", 1118,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2038", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2047", zones.back()->m_zone, "zone1141", 1141,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2048", zones.back()->m_zone, "zone112", 112, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2049", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2050", zones.back()->m_zone, "zone261", 261,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1698", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2030", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2051", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2052", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2053", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2054", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1883", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2039", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2052", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2055", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2056", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2057", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1974", zones.back()->m_zone, "zone1117", 1117,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2049", zones.back()->m_zone, "zone1143", 1143,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2056", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2058", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2059", zones.back()->m_zone, "zone98", 98, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2060", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1702", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2042", zones.back()->m_zone, "zone1140", 1140,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2054", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2058", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2061", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2062", zones.back()->m_zone, "zone99", 99, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1708", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2053", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2063", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2064", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2065", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2066", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1878", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2057", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2066", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2067", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2068", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2069", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1713", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2061", zones.back()->m_zone, "zone1147", 1147,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2065", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2070", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2071", zones.back()->m_zone, "zone96", 96, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2072", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1970", zones.back()->m_zone, "zone1116", 1116,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2060", zones.back()->m_zone, "zone1146", 1146,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2067", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2072", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2073", zones.back()->m_zone, "zone97", 97, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2074", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1716", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2063", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2075", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2076", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2077", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2078", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B209", zones.back()->m_zone, "zone909", 909, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B298", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A300", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A301", zones.back()->m_zone, "zone180", 180,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A302", zones.back()->m_zone, "zone971", 971, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A303", zones.back()->m_zone, "zone375", 375, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1863", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2068", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2078", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2079", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2080", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2081", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1959", zones.back()->m_zone, "zone1113", 1113,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2074", zones.back()->m_zone, "zone1151", 1151,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2080", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2082", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2083", zones.back()->m_zone, "zone139", 139,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2084", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1720", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2070", zones.back()->m_zone, "zone1150", 1150,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2075", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2082", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2085", zones.back()->m_zone, "zone95", 95, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2086", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1744", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1749", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2087", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2088", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2089", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2090", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1728", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2076", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2089", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2091", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2092", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2093", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1869", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2081", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2093", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2094", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2095", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2096", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1754", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B1758", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2088", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2097", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2098", zones.back()->m_zone, "zone82", 82, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2099", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1963", zones.back()->m_zone, "zone1114", 1114,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2084", zones.back()->m_zone, "zone1155", 1155,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2095", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2100", zones.back()->m_zone, "zone1162", 1162,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2101", zones.back()->m_zone, "zone93", 93, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2102", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1732", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2086", zones.back()->m_zone, "zone1156", 1156,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2092", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2099", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2100", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2103", zones.back()->m_zone, "zone94", 94,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1735", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2087", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2104", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2105", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2106", zones.back()->m_zone, "zone397", 397, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2107", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B23", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A32", zones.back()->m_zone, "zone1275", 1275, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A33", zones.back()->m_zone, "zone1442", 1442, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A34", zones.back()->m_zone, "zone1386", 1386,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A35", zones.back()->m_zone, "zone424", 424, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A36", zones.back()->m_zone, "zone563", 563, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 5}}, Ioss::IJK_t{{8, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B212", zones.back()->m_zone, "zone920", 920, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B292", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A304", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A305", zones.back()->m_zone, "zone963", 963, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A306", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A307", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B1873", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2096", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2105", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2108", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2109", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2110", zones.back()->m_zone, "zone385", 385,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1740", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2097", zones.back()->m_zone, "zone1160", 1160,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2107", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2111", zones.back()->m_zone, "zone1168", 1168,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2112", zones.back()->m_zone, "zone398", 398, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2113", zones.back()->m_zone, "zone83", 83, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1967", zones.back()->m_zone, "zone1115", 1115,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2102", zones.back()->m_zone, "zone1161", 1161,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2109", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2111", zones.back()->m_zone, "zone1167", 1167,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2114", zones.back()->m_zone, "zone92", 92, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2115", zones.back()->m_zone, "zone387", 387,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2116", zones.back()->m_zone, "zone1477", 1477,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2117", zones.back()->m_zone, "zone1171", 1171,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2118", zones.back()->m_zone, "zone207", 207, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2119", zones.back()->m_zone, "zone9", 9, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2120", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2120", zones.back()->m_zone, "zone1169", 1169,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2121", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2122", zones.back()->m_zone, "zone10", 10, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2123", zones.back()->m_zone, "zone187", 187,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2124", zones.back()->m_zone, "zone209", 209, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2125", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2117", zones.back()->m_zone, "zone1169", 1169,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2126", zones.back()->m_zone, "zone7", 7, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2127", zones.back()->m_zone, "zone5", 5, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2128", zones.back()->m_zone, "zone212", 212, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2129", zones.back()->m_zone, "zone1172", 1172,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2125", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2129", zones.back()->m_zone, "zone1171", 1171,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2130", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2131", zones.back()->m_zone, "zone186", 186,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2132", zones.back()->m_zone, "zone216", 216, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2133", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2134", zones.back()->m_zone, "zone1178", 1178,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2135", zones.back()->m_zone, "zone1482", 1482,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2136", zones.back()->m_zone, "zone1186", 1186,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2137", zones.back()->m_zone, "zone1240", 1240,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2138", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1806", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2138", zones.back()->m_zone, "zone1173", 1173,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2139", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2140", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2141", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2142", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2143", zones.back()->m_zone, "zone1489", 1489,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2144", zones.back()->m_zone, "zone1461", 1461,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2145", zones.back()->m_zone, "zone1482", 1482,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2146", zones.back()->m_zone, "zone1178", 1178,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2147", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B217", zones.back()->m_zone, "zone931", 931, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B307", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A308", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 31, 8}},
                                                Ioss::IJK_t{{12, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A309", zones.back()->m_zone, "zone962", 962,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A310", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A311", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1810", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2147", zones.back()->m_zone, "zone1175", 1175,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2148", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2149", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2150", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2151", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2134", zones.back()->m_zone, "zone1173", 1173,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2146", zones.back()->m_zone, "zone1175", 1175,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2152", zones.back()->m_zone, "zone1180", 1180,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2153", zones.back()->m_zone, "zone1222", 1222,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2154", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1797", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2139", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2149", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2154", zones.back()->m_zone, "zone1178", 1178,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2155", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2156", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2152", zones.back()->m_zone, "zone1178", 1178,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2157", zones.back()->m_zone, "zone1487", 1487,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2158", zones.back()->m_zone, "zone1211", 1211,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2159", zones.back()->m_zone, "zone1191", 1191,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2160", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1687", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2155", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2160", zones.back()->m_zone, "zone1180", 1180,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2161", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2162", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2163", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2164", zones.back()->m_zone, "zone1489", 1489,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2165", zones.back()->m_zone, "zone1493", 1493,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2166", zones.back()->m_zone, "zone1479", 1479,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2167", zones.back()->m_zone, "zone1461", 1461,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2168", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1680", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2168", zones.back()->m_zone, "zone1182", 1182,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2169", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2170", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2171", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2172", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A2173", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2174", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2175", zones.back()->m_zone, "zone189", 189,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2176", zones.back()->m_zone, "zone77", 77,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2177", zones.back()->m_zone, "zone10", 10, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2178", zones.back()->m_zone, "zone1185", 1185,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2178", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2179", zones.back()->m_zone, "zone1477", 1477,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2180", zones.back()->m_zone, "zone9", 9, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2181", zones.back()->m_zone, "zone1473", 1473,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2182", zones.back()->m_zone, "zone1465", 1465, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2136", zones.back()->m_zone, "zone1173", 1173,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2183", zones.back()->m_zone, "zone512", 512,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2184", zones.back()->m_zone, "zone1189", 1189,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2185", zones.back()->m_zone, "zone1236", 1236,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2186", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B222", zones.back()->m_zone, "zone943", 943, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B311", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A312", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 31, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A313", zones.back()->m_zone, "zone959", 959, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A314", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A315", zones.back()->m_zone, "zone276", 276, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1795", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2141", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2186", zones.back()->m_zone, "zone1186", 1186,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2187", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2188", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2189", zones.back()->m_zone, "zone510", 510,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2184", zones.back()->m_zone, "zone1186", 1186,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2190", zones.back()->m_zone, "zone1206", 1206,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2191", zones.back()->m_zone, "zone1226", 1226,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2192", zones.back()->m_zone, "zone505", 505,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2193", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1790", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2188", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2193", zones.back()->m_zone, "zone1189", 1189,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2194", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2195", zones.back()->m_zone, "zone503", 503,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2196", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2159", zones.back()->m_zone, "zone1180", 1180,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2197", zones.back()->m_zone, "zone1213", 1213,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2198", zones.back()->m_zone, "zone1344", 1344,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2199", zones.back()->m_zone, "zone1484", 1484,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2200", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1697", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2162", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2200", zones.back()->m_zone, "zone1191", 1191,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2201", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2202", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2203", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2204", zones.back()->m_zone, "zone1383", 1383,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2205", zones.back()->m_zone, "zone1339", 1339,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2206", zones.back()->m_zone, "zone1202", 1202,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2207", zones.back()->m_zone, "zone1195", 1195,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2208", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2077", zones.back()->m_zone, "zone1152", 1152,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2208", zones.back()->m_zone, "zone1193", 1193,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2209", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2210", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2211", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2212", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2207", zones.back()->m_zone, "zone1193", 1193,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2213", zones.back()->m_zone, "zone1200", 1200,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2214", zones.back()->m_zone, "zone1385", 1385,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2215", zones.back()->m_zone, "zone1381", 1381,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2216", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2079", zones.back()->m_zone, "zone1154", 1154,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2210", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2216", zones.back()->m_zone, "zone1195", 1195,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2217", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2218", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2219", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("A2220", zones.back()->m_zone, "zone1397", 1397,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2221", zones.back()->m_zone, "zone1385", 1385,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2222", zones.back()->m_zone, "zone1200", 1200,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2223", zones.back()->m_zone, "zone1404", 1404,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2224", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B227", zones.back()->m_zone, "zone954", 954, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B300", zones.back()->m_zone, "zone1153", 1153,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A316", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A317", zones.back()->m_zone, "zone970", 970, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A318", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A319", zones.back()->m_zone, "zone364", 364, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1867", zones.back()->m_zone, "zone1090", 1090,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2224", zones.back()->m_zone, "zone1197", 1197,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2225", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2226", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2227", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2228", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2213", zones.back()->m_zone, "zone1195", 1195,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2222", zones.back()->m_zone, "zone1197", 1197,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2229", zones.back()->m_zone, "zone1202", 1202,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2230", zones.back()->m_zone, "zone1406", 1406,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2231", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2094", zones.back()->m_zone, "zone1159", 1159,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2219", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2227", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2231", zones.back()->m_zone, "zone1200", 1200,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2232", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2233", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2206", zones.back()->m_zone, "zone1193", 1193,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2229", zones.back()->m_zone, "zone1200", 1200,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2234", zones.back()->m_zone, "zone1204", 1204,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2235", zones.back()->m_zone, "zone1335", 1335,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2236", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2091", zones.back()->m_zone, "zone1158", 1158,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2211", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2232", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2236", zones.back()->m_zone, "zone1202", 1202,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2237", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2238", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2234", zones.back()->m_zone, "zone1202", 1202,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2239", zones.back()->m_zone, "zone1444", 1444,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2240", zones.back()->m_zone, "zone1330", 1330,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2241", zones.back()->m_zone, "zone1450", 1450,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2242", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2090", zones.back()->m_zone, "zone1157", 1157,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2237", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2242", zones.back()->m_zone, "zone1204", 1204,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2243", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2244", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2245", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2190", zones.back()->m_zone, "zone1189", 1189,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2246", zones.back()->m_zone, "zone1333", 1333,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2247", zones.back()->m_zone, "zone1224", 1224,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2248", zones.back()->m_zone, "zone1459", 1459,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2249", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1784", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2194", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2249", zones.back()->m_zone, "zone1206", 1206,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2250", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2251", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2252", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2253", zones.back()->m_zone, "zone1273", 1273,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2254", zones.back()->m_zone, "zone1263", 1263,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2255", zones.back()->m_zone, "zone1270", 1270,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2256", zones.back()->m_zone, "zone1304", 1304,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2257", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B234", zones.back()->m_zone, "zone965", 965, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B296", zones.back()->m_zone, "zone1142", 1142,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B316", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A320", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A321", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A322", zones.back()->m_zone, "zone969", 969, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2257", zones.back()->m_zone, "zone1208", 1208,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2258", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2259", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2260", zones.back()->m_zone, "zone178", 178,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2261", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2262", zones.back()->m_zone, "zone15", 15, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2158", zones.back()->m_zone, "zone1180", 1180,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2263", zones.back()->m_zone, "zone1222", 1222,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2264", zones.back()->m_zone, "zone1283", 1283,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2265", zones.back()->m_zone, "zone1213", 1213,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2266", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1689", zones.back()->m_zone, "zone1044", 1044,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2163", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2266", zones.back()->m_zone, "zone1211", 1211,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2267", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2268", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2269", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2197", zones.back()->m_zone, "zone1191", 1191,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2265", zones.back()->m_zone, "zone1211", 1211,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2270", zones.back()->m_zone, "zone1288", 1288,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2271", zones.back()->m_zone, "zone1346", 1346,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2272", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1701", zones.back()->m_zone, "zone1046", 1046,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2203", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2267", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2272", zones.back()->m_zone, "zone1213", 1213,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2273", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2274", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("A2275", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2276", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2277", zones.back()->m_zone, "zone159", 159,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2278", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2279", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2280", zones.back()->m_zone, "zone1216", 1216,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2280", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2281", zones.back()->m_zone, "zone1323", 1323,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2282", zones.back()->m_zone, "zone1218", 1218,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2283", zones.back()->m_zone, "zone1299", 1299,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2284", zones.back()->m_zone, "zone1315", 1315,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1760", zones.back()->m_zone, "zone1061", 1061,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2279", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2285", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2286", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2287", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2288", zones.back()->m_zone, "zone1218", 1218,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2282", zones.back()->m_zone, "zone1216", 1216,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2288", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2289", zones.back()->m_zone, "zone1325", 1325,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2290", zones.back()->m_zone, "zone1328", 1328,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2291", zones.back()->m_zone, "zone1301", 1301,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2292", zones.back()->m_zone, "zone1333", 1333,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2293", zones.back()->m_zone, "zone1328", 1328,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2294", zones.back()->m_zone, "zone1325", 1325,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2295", zones.back()->m_zone, "zone1224", 1224,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2296", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B236", zones.back()->m_zone, "zone976", 976, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B291", zones.back()->m_zone, "zone1131", 1131,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B321", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A323", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A324", zones.back()->m_zone, "zone966", 966, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A325", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1778", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2296", zones.back()->m_zone, "zone1219", 1219,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2297", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2298", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2299", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2300", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2153", zones.back()->m_zone, "zone1178", 1178,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{6, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2263", zones.back()->m_zone, "zone1211", 1211,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2301", zones.back()->m_zone, "zone1240", 1240,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2302", zones.back()->m_zone, "zone1279", 1279,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2303", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1798", zones.back()->m_zone, "zone1071", 1071,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2156", zones.back()->m_zone, "zone1179", 1179,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2269", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2303", zones.back()->m_zone, "zone1222", 1222,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2304", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2305", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2247", zones.back()->m_zone, "zone1206", 1206,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2295", zones.back()->m_zone, "zone1219", 1219,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2306", zones.back()->m_zone, "zone1229", 1229,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2307", zones.back()->m_zone, "zone1226", 1226,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2308", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1783", zones.back()->m_zone, "zone1068", 1068,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2252", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2298", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2308", zones.back()->m_zone, "zone1224", 1224,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2309", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2310", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2191", zones.back()->m_zone, "zone1189", 1189,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2307", zones.back()->m_zone, "zone1224", 1224,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2311", zones.back()->m_zone, "zone1232", 1232,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2312", zones.back()->m_zone, "zone1236", 1236,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2313", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1788", zones.back()->m_zone, "zone1069", 1069,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2196", zones.back()->m_zone, "zone1190", 1190,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2309", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2313", zones.back()->m_zone, "zone1226", 1226,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2314", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2315", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2316", zones.back()->m_zone, "zone1257", 1257,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2317", zones.back()->m_zone, "zone1255", 1255,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2318", zones.back()->m_zone, "zone1254", 1254,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2319", zones.back()->m_zone, "zone1229", 1229,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2320", zones.back()->m_zone, "zone38", 38,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2306", zones.back()->m_zone, "zone1224", 1224,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2319", zones.back()->m_zone, "zone1228", 1228,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2321", zones.back()->m_zone, "zone1232", 1232,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2322", zones.back()->m_zone, "zone1327", 1327,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2323", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1836", zones.back()->m_zone, "zone1081", 1081,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2310", zones.back()->m_zone, "zone1225", 1225,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2323", zones.back()->m_zone, "zone1229", 1229,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2324", zones.back()->m_zone, "zone39", 39, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2325", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2326", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B242", zones.back()->m_zone, "zone987", 987, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B304", zones.back()->m_zone, "zone1165", 1165,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B323", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A326", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A327", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A328", zones.back()->m_zone, "zone964", 964, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2311", zones.back()->m_zone, "zone1226", 1226,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2321", zones.back()->m_zone, "zone1229", 1229,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2327", zones.back()->m_zone, "zone1254", 1254,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2328", zones.back()->m_zone, "zone1238", 1238,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2329", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1840", zones.back()->m_zone, "zone1082", 1082,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2315", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2326", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2329", zones.back()->m_zone, "zone1232", 1232,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2330", zones.back()->m_zone, "zone63", 63, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2331", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2332", zones.back()->m_zone, "zone1254", 1254,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2333", zones.back()->m_zone, "zone1251", 1251,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2334", zones.back()->m_zone, "zone1235", 1235,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2335", zones.back()->m_zone, "zone1238", 1238,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2336", zones.back()->m_zone, "zone56", 56, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2334", zones.back()->m_zone, "zone1234", 1234,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2337", zones.back()->m_zone, "zone1248", 1248,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2338", zones.back()->m_zone, "zone1245", 1245,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2339", zones.back()->m_zone, "zone1250", 1250,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2340", zones.back()->m_zone, "zone57", 57,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2185", zones.back()->m_zone, "zone1186", 1186,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2312", zones.back()->m_zone, "zone1226", 1226,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2341", zones.back()->m_zone, "zone1238", 1238,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2342", zones.back()->m_zone, "zone1240", 1240,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2343", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1796", zones.back()->m_zone, "zone1070", 1070,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2187", zones.back()->m_zone, "zone1188", 1188,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2314", zones.back()->m_zone, "zone1227", 1227,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2343", zones.back()->m_zone, "zone1236", 1236,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2344", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2345", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2328", zones.back()->m_zone, "zone1232", 1232,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2335", zones.back()->m_zone, "zone1234", 1234,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2341", zones.back()->m_zone, "zone1236", 1236,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2346", zones.back()->m_zone, "zone1245", 1245,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2347", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1842", zones.back()->m_zone, "zone1083", 1083,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2331", zones.back()->m_zone, "zone1233", 1233,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2345", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2347", zones.back()->m_zone, "zone1238", 1238,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2348", zones.back()->m_zone, "zone64", 64, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2349", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2137", zones.back()->m_zone, "zone1173", 1173,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2301", zones.back()->m_zone, "zone1222", 1222,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2342", zones.back()->m_zone, "zone1236", 1236,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2350", zones.back()->m_zone, "zone1245", 1245,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2351", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1807", zones.back()->m_zone, "zone1073", 1073,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2140", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2305", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2344", zones.back()->m_zone, "zone1237", 1237,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2351", zones.back()->m_zone, "zone1240", 1240,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2352", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B244", zones.back()->m_zone, "zone998", 998, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B308", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B327", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A329", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A330", zones.back()->m_zone, "zone961", 961, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A331", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1849", zones.back()->m_zone, "zone1084", 1084,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2353", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2354", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2355", zones.back()->m_zone, "zone21", 21, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2356", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2357", zones.back()->m_zone, "zone1244", 1244,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2357", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2358", zones.back()->m_zone, "zone1279", 1279,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2359", zones.back()->m_zone, "zone1245", 1245,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2360", zones.back()->m_zone, "zone1248", 1248, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2361", zones.back()->m_zone, "zone1273", 1273, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2338", zones.back()->m_zone, "zone1235", 1235,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2346", zones.back()->m_zone, "zone1238", 1238,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2350", zones.back()->m_zone, "zone1240", 1240,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2359", zones.back()->m_zone, "zone1244", 1244,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2362", zones.back()->m_zone, "zone1246", 1246,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1850", zones.back()->m_zone, "zone1085", 1085,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2349", zones.back()->m_zone, "zone1239", 1239,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2352", zones.back()->m_zone, "zone1241", 1241,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2356", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2362", zones.back()->m_zone, "zone1245", 1245,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2363", zones.back()->m_zone, "zone65", 65, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A2364", zones.back()->m_zone, "zone1273", 1273,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2365", zones.back()->m_zone, "zone1248", 1248,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2366", zones.back()->m_zone, "zone1249", 1249,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2367", zones.back()->m_zone, "zone1263", 1263,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2368", zones.back()->m_zone, "zone12", 12,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 1, 12}}, Ioss::IJK_t{{11, 11, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2337", zones.back()->m_zone, "zone1235", 1235,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2360", zones.back()->m_zone, "zone1244", 1244, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2365", zones.back()->m_zone, "zone1247", 1247,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2369", zones.back()->m_zone, "zone1250", 1250,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2370", zones.back()->m_zone, "zone20", 20,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2366", zones.back()->m_zone, "zone1247", 1247,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2371", zones.back()->m_zone, "zone1250", 1250,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2372", zones.back()->m_zone, "zone1261", 1261,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2373", zones.back()->m_zone, "zone1263", 1263,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2374", zones.back()->m_zone, "zone30", 30,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 12}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2339", zones.back()->m_zone, "zone1235", 1235,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2369", zones.back()->m_zone, "zone1248", 1248,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2371", zones.back()->m_zone, "zone1249", 1249,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2375", zones.back()->m_zone, "zone1251", 1251,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2376", zones.back()->m_zone, "zone31", 31,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2333", zones.back()->m_zone, "zone1234", 1234,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2375", zones.back()->m_zone, "zone1250", 1250,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2377", zones.back()->m_zone, "zone1252", 1252,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2378", zones.back()->m_zone, "zone1261", 1261,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2379", zones.back()->m_zone, "zone32", 32,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2377", zones.back()->m_zone, "zone1251", 1251,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2380", zones.back()->m_zone, "zone1255", 1255,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2381", zones.back()->m_zone, "zone1260", 1260,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2382", zones.back()->m_zone, "zone1254", 1254,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2383", zones.back()->m_zone, "zone33", 33,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x30x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B250", zones.back()->m_zone, "zone1009", 1009,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B312", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 31, 1}},
                                                Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B329", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 31, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A332", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A333", zones.back()->m_zone, "zone287", 287, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 31, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A334", zones.back()->m_zone, "zone960", 960, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 31, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 31, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2318", zones.back()->m_zone, "zone1228", 1228,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2327", zones.back()->m_zone, "zone1232", 1232,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2332", zones.back()->m_zone, "zone1234", 1234,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2382", zones.back()->m_zone, "zone1252", 1252,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2384", zones.back()->m_zone, "zone55", 55,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2317", zones.back()->m_zone, "zone1228", 1228,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2380", zones.back()->m_zone, "zone1252", 1252,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2385", zones.back()->m_zone, "zone1257", 1257,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2386", zones.back()->m_zone, "zone1258", 1258,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2387", zones.back()->m_zone, "zone34", 34,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A2388", zones.back()->m_zone, "zone1321", 1321,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2389", zones.back()->m_zone, "zone1259", 1259,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2390", zones.back()->m_zone, "zone1258", 1258,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2391", zones.back()->m_zone, "zone1257", 1257, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2392", zones.back()->m_zone, "zone26", 26, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2316", zones.back()->m_zone, "zone1228", 1228,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2385", zones.back()->m_zone, "zone1255", 1255,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2391", zones.back()->m_zone, "zone1256", 1256, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2393", zones.back()->m_zone, "zone1327", 1327, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2394", zones.back()->m_zone, "zone35", 35,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2386", zones.back()->m_zone, "zone1255", 1255,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2390", zones.back()->m_zone, "zone1256", 1256,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2395", zones.back()->m_zone, "zone1260", 1260,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2396", zones.back()->m_zone, "zone1259", 1259,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2397", zones.back()->m_zone, "zone27", 27,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2389", zones.back()->m_zone, "zone1256", 1256,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2396", zones.back()->m_zone, "zone1258", 1258,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2398", zones.back()->m_zone, "zone1268", 1268,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2399", zones.back()->m_zone, "zone1265", 1265,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2400", zones.back()->m_zone, "zone22", 22, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2381", zones.back()->m_zone, "zone1252", 1252,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2395", zones.back()->m_zone, "zone1258", 1258,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2401", zones.back()->m_zone, "zone1265", 1265,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2402", zones.back()->m_zone, "zone1261", 1261,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2403", zones.back()->m_zone, "zone28", 28,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2372", zones.back()->m_zone, "zone1249", 1249,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2378", zones.back()->m_zone, "zone1251", 1251,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2402", zones.back()->m_zone, "zone1260", 1260,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2404", zones.back()->m_zone, "zone1262", 1262,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2405", zones.back()->m_zone, "zone29", 29,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2404", zones.back()->m_zone, "zone1261", 1261,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2406", zones.back()->m_zone, "zone1265", 1265,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2407", zones.back()->m_zone, "zone1270", 1270,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2408", zones.back()->m_zone, "zone1263", 1263,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2409", zones.back()->m_zone, "zone16", 16, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2254", zones.back()->m_zone, "zone1208", 1208,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2367", zones.back()->m_zone, "zone1247", 1247,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2373", zones.back()->m_zone, "zone1249", 1249,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2408", zones.back()->m_zone, "zone1262", 1262,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2410", zones.back()->m_zone, "zone13", 13,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B252", zones.back()->m_zone, "zone1020", 1020,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B332", zones.back()->m_zone, "zone1253", 1253,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A335", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A336", zones.back()->m_zone, "zone980", 980, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A337", zones.back()->m_zone, "zone297", 297,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A338", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2399", zones.back()->m_zone, "zone1259", 1259,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2401", zones.back()->m_zone, "zone1260", 1260,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2406", zones.back()->m_zone, "zone1262", 1262,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2411", zones.back()->m_zone, "zone1266", 1266,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2412", zones.back()->m_zone, "zone19", 19,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 8, 11}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2411", zones.back()->m_zone, "zone1265", 1265,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2413", zones.back()->m_zone, "zone1270", 1270,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2414", zones.back()->m_zone, "zone1268", 1268,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2415", zones.back()->m_zone, "zone1313", 1313,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2416", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2416", zones.back()->m_zone, "zone1266", 1266,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2417", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2418", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2419", zones.back()->m_zone, "zone162", 162,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2420", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2421", zones.back()->m_zone, "zone18", 18, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2398", zones.back()->m_zone, "zone1259", 1259,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2414", zones.back()->m_zone, "zone1266", 1266,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2422", zones.back()->m_zone, "zone1321", 1321,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2423", zones.back()->m_zone, "zone1317", 1317,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2424", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2420", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2424", zones.back()->m_zone, "zone1268", 1268,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2425", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2426", zones.back()->m_zone, "zone23", 23, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2427", zones.back()->m_zone, "zone163", 163,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2428", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2255", zones.back()->m_zone, "zone1208", 1208,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2407", zones.back()->m_zone, "zone1262", 1262,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2413", zones.back()->m_zone, "zone1266", 1266,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2429", zones.back()->m_zone, "zone1308", 1308,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2430", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2261", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2417", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2430", zones.back()->m_zone, "zone1270", 1270,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2431", zones.back()->m_zone, "zone17", 17, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 8, 1}}, Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2432", zones.back()->m_zone, "zone179", 179,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2433", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2258", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2354", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2434", zones.back()->m_zone, "zone11", 11, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2435", zones.back()->m_zone, "zone177", 177,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2436", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2437", zones.back()->m_zone, "zone1273", 1273,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2253", zones.back()->m_zone, "zone1208", 1208,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2361", zones.back()->m_zone, "zone1244", 1244, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2364", zones.back()->m_zone, "zone1247", 1247,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2437", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2438", zones.back()->m_zone, "zone1277", 1277, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2436", zones.back()->m_zone, "zone1272", 1272,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2439", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2440", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2441", zones.back()->m_zone, "zone176", 176,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2442", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2443", zones.back()->m_zone, "zone1277", 1277,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B29", zones.back()->m_zone, "zone1053", 1053,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B32", zones.back()->m_zone, "zone1164", 1164,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A37", zones.back()->m_zone, "zone984", 984, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A38", zones.back()->m_zone, "zone1475", 1475, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A39", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A40", zones.back()->m_zone, "zone636", 636, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B258", zones.back()->m_zone, "zone1031", 1031,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B331", zones.back()->m_zone, "zone1242", 1242,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B335", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A339", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A340", zones.back()->m_zone, "zone979", 979, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A341", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2438", zones.back()->m_zone, "zone1273", 1273, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2443", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2444", zones.back()->m_zone, "zone1281", 1281,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2445", zones.back()->m_zone, "zone1279", 1279,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2446", zones.back()->m_zone, "zone1304", 1304,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1802", zones.back()->m_zone, "zone1072", 1072,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2304", zones.back()->m_zone, "zone1223", 1223,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2353", zones.back()->m_zone, "zone1243", 1243,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2442", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2447", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2448", zones.back()->m_zone, "zone1279", 1279,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2302", zones.back()->m_zone, "zone1222", 1222,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2358", zones.back()->m_zone, "zone1244", 1244,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2445", zones.back()->m_zone, "zone1277", 1277,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2448", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2449", zones.back()->m_zone, "zone1283", 1283,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2439", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2450", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2451", zones.back()->m_zone, "zone171", 171,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{51, 6, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2452", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2453", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2454", zones.back()->m_zone, "zone1281", 1281,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2444", zones.back()->m_zone, "zone1277", 1277,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2454", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2455", zones.back()->m_zone, "zone1302", 1302,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2456", zones.back()->m_zone, "zone1285", 1285,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2457", zones.back()->m_zone, "zone1283", 1283,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1694", zones.back()->m_zone, "zone1045", 1045,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{12, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2268", zones.back()->m_zone, "zone1212", 1212,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2447", zones.back()->m_zone, "zone1278", 1278,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2450", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2458", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2459", zones.back()->m_zone, "zone1283", 1283,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2264", zones.back()->m_zone, "zone1211", 1211,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2449", zones.back()->m_zone, "zone1279", 1279,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2457", zones.back()->m_zone, "zone1281", 1281,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2459", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2460", zones.back()->m_zone, "zone1288", 1288,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2452", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2461", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2462", zones.back()->m_zone, "zone170", 170,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2463", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2464", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2465", zones.back()->m_zone, "zone1285", 1285,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2456", zones.back()->m_zone, "zone1281", 1281,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2465", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2466", zones.back()->m_zone, "zone1302", 1302,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2467", zones.back()->m_zone, "zone1290", 1290,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2468", zones.back()->m_zone, "zone1288", 1288,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1705", zones.back()->m_zone, "zone1047", 1047,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2274", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2458", zones.back()->m_zone, "zone1282", 1282,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2461", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2469", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2470", zones.back()->m_zone, "zone1288", 1288,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B263", zones.back()->m_zone, "zone1042", 1042,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B326", zones.back()->m_zone, "zone1231", 1231,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B339", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A342", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A343", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A344", zones.back()->m_zone, "zone978", 978, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2270", zones.back()->m_zone, "zone1213", 1213,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2460", zones.back()->m_zone, "zone1283", 1283,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2468", zones.back()->m_zone, "zone1285", 1285,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2470", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2471", zones.back()->m_zone, "zone1292", 1292,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2463", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2472", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2473", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2474", zones.back()->m_zone, "zone168", 168,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2475", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2476", zones.back()->m_zone, "zone1290", 1290,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2467", zones.back()->m_zone, "zone1285", 1285,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2476", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2477", zones.back()->m_zone, "zone1294", 1294,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2478", zones.back()->m_zone, "zone1292", 1292,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2479", zones.back()->m_zone, "zone1306", 1306,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1714", zones.back()->m_zone, "zone1049", 1049,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2469", zones.back()->m_zone, "zone1286", 1286,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2475", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2480", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2481", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2482", zones.back()->m_zone, "zone1292", 1292,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2471", zones.back()->m_zone, "zone1288", 1288,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2478", zones.back()->m_zone, "zone1290", 1290,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2482", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2483", zones.back()->m_zone, "zone1296", 1296,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2484", zones.back()->m_zone, "zone1346", 1346,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2472", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2485", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2486", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2487", zones.back()->m_zone, "zone154", 154,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2488", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2489", zones.back()->m_zone, "zone1294", 1294,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2477", zones.back()->m_zone, "zone1290", 1290,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2489", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2490", zones.back()->m_zone, "zone1299", 1299,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2491", zones.back()->m_zone, "zone1296", 1296,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2492", zones.back()->m_zone, "zone1311", 1311,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1723", zones.back()->m_zone, "zone1051", 1051,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 6, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2480", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2488", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2493", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2494", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2495", zones.back()->m_zone, "zone1296", 1296,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2483", zones.back()->m_zone, "zone1292", 1292,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2491", zones.back()->m_zone, "zone1294", 1294,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2495", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2496", zones.back()->m_zone, "zone1301", 1301,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2497", zones.back()->m_zone, "zone1341", 1341,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2278", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2485", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2498", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2499", zones.back()->m_zone, "zone156", 156,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{51, 8, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2500", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2501", zones.back()->m_zone, "zone1299", 1299,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B264", zones.back()->m_zone, "zone1054", 1054,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B325", zones.back()->m_zone, "zone1220", 1220,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B342", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A345", zones.back()->m_zone, "zone977", 977, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A346", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A347", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2283", zones.back()->m_zone, "zone1216", 1216,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2490", zones.back()->m_zone, "zone1294", 1294,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2501", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2502", zones.back()->m_zone, "zone1315", 1315,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2503", zones.back()->m_zone, "zone1301", 1301,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1730", zones.back()->m_zone, "zone1055", 1055,
                                                Ioss::IJK_t{{-3, -2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2286", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2493", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2498", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2504", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2505", zones.back()->m_zone, "zone1301", 1301,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2291", zones.back()->m_zone, "zone1218", 1218,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2496", zones.back()->m_zone, "zone1296", 1296,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2503", zones.back()->m_zone, "zone1299", 1299,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2505", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2506", zones.back()->m_zone, "zone1337", 1337,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2455", zones.back()->m_zone, "zone1281", 1281,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2466", zones.back()->m_zone, "zone1285", 1285,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2507", zones.back()->m_zone, "zone1304", 1304,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2508", zones.back()->m_zone, "zone1306", 1306,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2509", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2453", zones.back()->m_zone, "zone1280", 1280,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2464", zones.back()->m_zone, "zone1284", 1284,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2509", zones.back()->m_zone, "zone1302", 1302,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2510", zones.back()->m_zone, "zone172", 172,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2511", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2512", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2256", zones.back()->m_zone, "zone1208", 1208,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2446", zones.back()->m_zone, "zone1277", 1277,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2507", zones.back()->m_zone, "zone1302", 1302,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2513", zones.back()->m_zone, "zone1308", 1308,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2514", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2259", zones.back()->m_zone, "zone1210", 1210,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2440", zones.back()->m_zone, "zone1274", 1274,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2512", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2514", zones.back()->m_zone, "zone1304", 1304,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2515", zones.back()->m_zone, "zone175", 175,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2516", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2479", zones.back()->m_zone, "zone1290", 1290,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2508", zones.back()->m_zone, "zone1302", 1302,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2517", zones.back()->m_zone, "zone1311", 1311,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2518", zones.back()->m_zone, "zone1308", 1308,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2519", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2473", zones.back()->m_zone, "zone1289", 1289,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2511", zones.back()->m_zone, "zone1303", 1303,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2519", zones.back()->m_zone, "zone1306", 1306,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2520", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2521", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2522", zones.back()->m_zone, "zone173", 173,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2429", zones.back()->m_zone, "zone1270", 1270,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2513", zones.back()->m_zone, "zone1304", 1304,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2518", zones.back()->m_zone, "zone1306", 1306,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2523", zones.back()->m_zone, "zone1313", 1313,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2524", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B269", zones.back()->m_zone, "zone1065", 1065,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B320", zones.back()->m_zone, "zone1209", 1209,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B347", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 11, 8}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A348", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A349", zones.back()->m_zone, "zone973", 973, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A350", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2433", zones.back()->m_zone, "zone1271", 1271,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2516", zones.back()->m_zone, "zone1305", 1305,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2521", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2524", zones.back()->m_zone, "zone1308", 1308,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2525", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2526", zones.back()->m_zone, "zone174", 174,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2492", zones.back()->m_zone, "zone1294", 1294,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2517", zones.back()->m_zone, "zone1306", 1306,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2527", zones.back()->m_zone, "zone1313", 1313,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2528", zones.back()->m_zone, "zone1315", 1315,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2529", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2486", zones.back()->m_zone, "zone1293", 1293,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2520", zones.back()->m_zone, "zone1307", 1307,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2529", zones.back()->m_zone, "zone1311", 1311,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2530", zones.back()->m_zone, "zone155", 155,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2531", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2532", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2415", zones.back()->m_zone, "zone1266", 1266,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2523", zones.back()->m_zone, "zone1308", 1308,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2527", zones.back()->m_zone, "zone1311", 1311,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2533", zones.back()->m_zone, "zone1317", 1317,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2534", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2418", zones.back()->m_zone, "zone1267", 1267,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2525", zones.back()->m_zone, "zone1310", 1310,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2532", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2534", zones.back()->m_zone, "zone1313", 1313,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2535", zones.back()->m_zone, "zone161", 161,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2536", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2284", zones.back()->m_zone, "zone1216", 1216,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2502", zones.back()->m_zone, "zone1299", 1299,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2528", zones.back()->m_zone, "zone1311", 1311,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2537", zones.back()->m_zone, "zone1317", 1317,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2538", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2276", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2500", zones.back()->m_zone, "zone1297", 1297,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2531", zones.back()->m_zone, "zone1312", 1312,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2538", zones.back()->m_zone, "zone1315", 1315,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2539", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2540", zones.back()->m_zone, "zone157", 157,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2423", zones.back()->m_zone, "zone1268", 1268,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2533", zones.back()->m_zone, "zone1313", 1313,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2537", zones.back()->m_zone, "zone1315", 1315,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2541", zones.back()->m_zone, "zone1323", 1323,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2542", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2428", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2536", zones.back()->m_zone, "zone1314", 1314,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2539", zones.back()->m_zone, "zone1316", 1316,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2542", zones.back()->m_zone, "zone1317", 1317,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2543", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2544", zones.back()->m_zone, "zone194", 194,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2425", zones.back()->m_zone, "zone1269", 1269,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2545", zones.back()->m_zone, "zone24", 24, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2546", zones.back()->m_zone, "zone164", 164,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2547", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2548", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2549", zones.back()->m_zone, "zone1321", 1321,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x10x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B274", zones.back()->m_zone, "zone1076", 1076,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B318", zones.back()->m_zone, "zone1198", 1198,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B348", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 11, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A351", zones.back()->m_zone, "zone350", 350, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A352", zones.back()->m_zone, "zone972", 972, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A353", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2388", zones.back()->m_zone, "zone1256", 1256,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2422", zones.back()->m_zone, "zone1268", 1268,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2549", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2550", zones.back()->m_zone, "zone1327", 1327,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2551", zones.back()->m_zone, "zone1323", 1323, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2275", zones.back()->m_zone, "zone1215", 1215,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2543", zones.back()->m_zone, "zone1318", 1318,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2547", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2552", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2553", zones.back()->m_zone, "zone160", 160,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2554", zones.back()->m_zone, "zone1323", 1323,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2281", zones.back()->m_zone, "zone1216", 1216,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2541", zones.back()->m_zone, "zone1317", 1317,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2551", zones.back()->m_zone, "zone1321", 1321, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2554", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2555", zones.back()->m_zone, "zone1325", 1325, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1781", zones.back()->m_zone, "zone1067", 1067,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2285", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2299", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2552", zones.back()->m_zone, "zone1322", 1322,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2556", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2557", zones.back()->m_zone, "zone1325", 1325,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2289", zones.back()->m_zone, "zone1218", 1218,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2294", zones.back()->m_zone, "zone1219", 1219,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2555", zones.back()->m_zone, "zone1323", 1323, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2557", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2558", zones.back()->m_zone, "zone1327", 1327,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1831", zones.back()->m_zone, "zone1080", 1080,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2325", zones.back()->m_zone, "zone1230", 1230,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2548", zones.back()->m_zone, "zone1319", 1319,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2556", zones.back()->m_zone, "zone1324", 1324,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2559", zones.back()->m_zone, "zone37", 37,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 12, 1}},
                                                Ioss::IJK_t{{11, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2560", zones.back()->m_zone, "zone1327", 1327,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2322", zones.back()->m_zone, "zone1229", 1229,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2393", zones.back()->m_zone, "zone1257", 1257, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2550", zones.back()->m_zone, "zone1321", 1321,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2558", zones.back()->m_zone, "zone1325", 1325,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2560", zones.back()->m_zone, "zone1326", 1326,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2290", zones.back()->m_zone, "zone1218", 1218,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2293", zones.back()->m_zone, "zone1219", 1219,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2561", zones.back()->m_zone, "zone1330", 1330,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2562", zones.back()->m_zone, "zone1337", 1337,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2563", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1746", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{5, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2287", zones.back()->m_zone, "zone1217", 1217,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2300", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2563", zones.back()->m_zone, "zone1328", 1328,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2564", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2565", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2240", zones.back()->m_zone, "zone1204", 1204,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2561", zones.back()->m_zone, "zone1328", 1328,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2566", zones.back()->m_zone, "zone1335", 1335,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2567", zones.back()->m_zone, "zone1333", 1333,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2568", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B278", zones.back()->m_zone, "zone1087", 1087,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B350", zones.back()->m_zone, "zone1309", 1309,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B353", zones.back()->m_zone, "zone1320", 1320,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A354", zones.back()->m_zone, "zone342", 342, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A355", zones.back()->m_zone, "zone974", 974, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A356", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1747", zones.back()->m_zone, "zone1058", 1058,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2245", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2564", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2568", zones.back()->m_zone, "zone1330", 1330,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2569", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2570", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2246", zones.back()->m_zone, "zone1206", 1206,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2292", zones.back()->m_zone, "zone1219", 1219,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2567", zones.back()->m_zone, "zone1330", 1330,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2571", zones.back()->m_zone, "zone1452", 1452,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2572", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1776", zones.back()->m_zone, "zone1066", 1066,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2250", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2297", zones.back()->m_zone, "zone1221", 1221,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2569", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2572", zones.back()->m_zone, "zone1333", 1333,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2573", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2235", zones.back()->m_zone, "zone1202", 1202,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2566", zones.back()->m_zone, "zone1330", 1330,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2574", zones.back()->m_zone, "zone1339", 1339,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2575", zones.back()->m_zone, "zone1337", 1337,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2576", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1725", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2238", zones.back()->m_zone, "zone1203", 1203,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2570", zones.back()->m_zone, "zone1332", 1332,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2576", zones.back()->m_zone, "zone1335", 1335,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2577", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2578", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2506", zones.back()->m_zone, "zone1301", 1301,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2562", zones.back()->m_zone, "zone1328", 1328,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2575", zones.back()->m_zone, "zone1335", 1335,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2579", zones.back()->m_zone, "zone1341", 1341,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2580", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1726", zones.back()->m_zone, "zone1052", 1052,
                                                Ioss::IJK_t{{3, -2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2504", zones.back()->m_zone, "zone1300", 1300,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2565", zones.back()->m_zone, "zone1329", 1329,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2577", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2580", zones.back()->m_zone, "zone1337", 1337,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2581", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2205", zones.back()->m_zone, "zone1193", 1193,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2574", zones.back()->m_zone, "zone1335", 1335,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2582", zones.back()->m_zone, "zone1344", 1344,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2583", zones.back()->m_zone, "zone1341", 1341,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2584", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1715", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2212", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2578", zones.back()->m_zone, "zone1336", 1336,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2584", zones.back()->m_zone, "zone1339", 1339,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2585", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2586", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2497", zones.back()->m_zone, "zone1296", 1296,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2579", zones.back()->m_zone, "zone1337", 1337,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2583", zones.back()->m_zone, "zone1339", 1339,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2587", zones.back()->m_zone, "zone1346", 1346,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2588", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B280", zones.back()->m_zone, "zone1098", 1098,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B346", zones.back()->m_zone, "zone1298", 1298,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B356", zones.back()->m_zone, "zone1331", 1331,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A357", zones.back()->m_zone, "zone975", 975,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A358", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A359", zones.back()->m_zone, "zone331", 331,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1719", zones.back()->m_zone, "zone1050", 1050,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2494", zones.back()->m_zone, "zone1295", 1295,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2581", zones.back()->m_zone, "zone1338", 1338,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2586", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2588", zones.back()->m_zone, "zone1341", 1341,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2589", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2198", zones.back()->m_zone, "zone1191", 1191,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2582", zones.back()->m_zone, "zone1339", 1339,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2590", zones.back()->m_zone, "zone1383", 1383,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2591", zones.back()->m_zone, "zone1346", 1346,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2592", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1709", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2202", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2585", zones.back()->m_zone, "zone1340", 1340,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2592", zones.back()->m_zone, "zone1344", 1344,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2593", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2594", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2271", zones.back()->m_zone, "zone1213", 1213,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2484", zones.back()->m_zone, "zone1292", 1292,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2587", zones.back()->m_zone, "zone1341", 1341,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2591", zones.back()->m_zone, "zone1344", 1344,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2595", zones.back()->m_zone, "zone1347", 1347,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1710", zones.back()->m_zone, "zone1048", 1048,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2273", zones.back()->m_zone, "zone1214", 1214,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2481", zones.back()->m_zone, "zone1291", 1291,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2589", zones.back()->m_zone, "zone1343", 1343,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2593", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2595", zones.back()->m_zone, "zone1346", 1346,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2596", zones.back()->m_zone, "zone1350", 1350,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2597", zones.back()->m_zone, "zone1352", 1352,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2598", zones.back()->m_zone, "zone290", 290, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2599", zones.back()->m_zone, "zone1368", 1368,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2600", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1917", zones.back()->m_zone, "zone1102", 1102,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2600", zones.back()->m_zone, "zone1348", 1348,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2601", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2602", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2603", zones.back()->m_zone, "zone293", 293, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2604", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2596", zones.back()->m_zone, "zone1348", 1348,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2605", zones.back()->m_zone, "zone1366", 1366,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2606", zones.back()->m_zone, "zone1357", 1357,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2607", zones.back()->m_zone, "zone1352", 1352,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2608", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1923", zones.back()->m_zone, "zone1103", 1103,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2601", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2608", zones.back()->m_zone, "zone1350", 1350,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2609", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2610", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2611", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2597", zones.back()->m_zone, "zone1348", 1348,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2607", zones.back()->m_zone, "zone1350", 1350,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2612", zones.back()->m_zone, "zone1355", 1355,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2613", zones.back()->m_zone, "zone301", 301, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2614", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B286", zones.back()->m_zone, "zone1109", 1109,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B343", zones.back()->m_zone, "zone1287", 1287,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B358", zones.back()->m_zone, "zone1342", 1342,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A360", zones.back()->m_zone, "zone315", 315, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A361", zones.back()->m_zone, "zone982", 982, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A362", zones.back()->m_zone, "zone1364", 1364,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1953", zones.back()->m_zone, "zone1112", 1112,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2604", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2609", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2614", zones.back()->m_zone, "zone1352", 1352,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2615", zones.back()->m_zone, "zone305", 305, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2616", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2612", zones.back()->m_zone, "zone1352", 1352,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2617", zones.back()->m_zone, "zone1357", 1357,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2618", zones.back()->m_zone, "zone1363", 1363,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2619", zones.back()->m_zone, "zone322", 322, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2620", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1951", zones.back()->m_zone, "zone1111", 1111,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2616", zones.back()->m_zone, "zone1354", 1354,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2620", zones.back()->m_zone, "zone1355", 1355,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2621", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2622", zones.back()->m_zone, "zone320", 320, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2623", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2606", zones.back()->m_zone, "zone1350", 1350,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2617", zones.back()->m_zone, "zone1355", 1355,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2624", zones.back()->m_zone, "zone1359", 1359,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2625", zones.back()->m_zone, "zone1393", 1393,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2626", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1928", zones.back()->m_zone, "zone1104", 1104,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2610", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2621", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2626", zones.back()->m_zone, "zone1357", 1357,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2627", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2628", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2624", zones.back()->m_zone, "zone1357", 1357,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2629", zones.back()->m_zone, "zone1391", 1391,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2630", zones.back()->m_zone, "zone1395", 1395,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2631", zones.back()->m_zone, "zone1363", 1363,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2632", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1929", zones.back()->m_zone, "zone1105", 1105,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2627", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2632", zones.back()->m_zone, "zone1359", 1359,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2633", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2634", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2635", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2636", zones.back()->m_zone, "zone1395", 1395,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2637", zones.back()->m_zone, "zone1400", 1400,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2638", zones.back()->m_zone, "zone334", 334, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2639", zones.back()->m_zone, "zone1363", 1363,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2640", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1943", zones.back()->m_zone, "zone1108", 1108,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2640", zones.back()->m_zone, "zone1361", 1361,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2641", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2642", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2643", zones.back()->m_zone, "zone338", 338, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2644", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2618", zones.back()->m_zone, "zone1355", 1355,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2631", zones.back()->m_zone, "zone1359", 1359,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2639", zones.back()->m_zone, "zone1361", 1361,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2645", zones.back()->m_zone, "zone323", 323, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2646", zones.back()->m_zone, "zone1365", 1365,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 8}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B288", zones.back()->m_zone, "zone1120", 1120,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B338", zones.back()->m_zone, "zone1264", 1264,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{12, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B341", zones.back()->m_zone, "zone1276", 1276,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 11, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B362", zones.back()->m_zone, "zone1353", 1353,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A363", zones.back()->m_zone, "zone981", 981, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A364", zones.back()->m_zone, "zone309", 309, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 8, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1945", zones.back()->m_zone, "zone1110", 1110,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2623", zones.back()->m_zone, "zone1356", 1356,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2633", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2642", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2646", zones.back()->m_zone, "zone1363", 1363,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2647", zones.back()->m_zone, "zone327", 327, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{8, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2605", zones.back()->m_zone, "zone1350", 1350,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2648", zones.back()->m_zone, "zone1368", 1368,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2649", zones.back()->m_zone, "zone1379", 1379,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2650", zones.back()->m_zone, "zone1393", 1393,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2651", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1909", zones.back()->m_zone, "zone1100", 1100,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2611", zones.back()->m_zone, "zone1351", 1351,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2651", zones.back()->m_zone, "zone1366", 1366,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2652", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2653", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2654", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2599", zones.back()->m_zone, "zone1348", 1348,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2648", zones.back()->m_zone, "zone1366", 1366,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2655", zones.back()->m_zone, "zone279", 279, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2656", zones.back()->m_zone, "zone1374", 1374,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2657", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1915", zones.back()->m_zone, "zone1101", 1101,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2602", zones.back()->m_zone, "zone1349", 1349,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2652", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2657", zones.back()->m_zone, "zone1368", 1368,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2658", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2659", zones.back()->m_zone, "zone283", 283, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2660", zones.back()->m_zone, "zone1487", 1487,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2661", zones.back()->m_zone, "zone1372", 1372,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2662", zones.back()->m_zone, "zone248", 248, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2663", zones.back()->m_zone, "zone1491", 1491,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2664", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2034", zones.back()->m_zone, "zone1138", 1138,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2664", zones.back()->m_zone, "zone1370", 1370,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2665", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2666", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2667", zones.back()->m_zone, "zone250", 250, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2668", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2661", zones.back()->m_zone, "zone1370", 1370,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2669", zones.back()->m_zone, "zone1377", 1377,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2670", zones.back()->m_zone, "zone1374", 1374,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2671", zones.back()->m_zone, "zone267", 267,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2672", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2040", zones.back()->m_zone, "zone1139", 1139,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2668", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2672", zones.back()->m_zone, "zone1372", 1372,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2673", zones.back()->m_zone, "zone264", 264,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2674", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2675", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2656", zones.back()->m_zone, "zone1368", 1368,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2670", zones.back()->m_zone, "zone1372", 1372,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2676", zones.back()->m_zone, "zone270", 270,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2677", zones.back()->m_zone, "zone1379", 1379,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2678", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B12", zones.back()->m_zone, "zone720", 720, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B21", zones.back()->m_zone, "zone942", 942, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A365", zones.back()->m_zone, "zone1442", 1442,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A366", zones.back()->m_zone, "zone838", 838, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A367", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A368", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1887", zones.back()->m_zone, "zone1094", 1094,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2658", zones.back()->m_zone, "zone1369", 1369,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2674", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2678", zones.back()->m_zone, "zone1374", 1374,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2679", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2680", zones.back()->m_zone, "zone271", 271,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2669", zones.back()->m_zone, "zone1372", 1372,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2681", zones.back()->m_zone, "zone1484", 1484,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2682", zones.back()->m_zone, "zone1381", 1381,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2683", zones.back()->m_zone, "zone1379", 1379,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2684", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2055", zones.back()->m_zone, "zone1145", 1145,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2675", zones.back()->m_zone, "zone1373", 1373,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2684", zones.back()->m_zone, "zone1377", 1377,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2685", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2686", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2687", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2649", zones.back()->m_zone, "zone1366", 1366,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2677", zones.back()->m_zone, "zone1374", 1374,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2683", zones.back()->m_zone, "zone1377", 1377,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2688", zones.back()->m_zone, "zone1389", 1389,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2689", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1882", zones.back()->m_zone, "zone1093", 1093,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2654", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2679", zones.back()->m_zone, "zone1376", 1376,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2686", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2689", zones.back()->m_zone, "zone1379", 1379,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2690", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2215", zones.back()->m_zone, "zone1195", 1195,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2682", zones.back()->m_zone, "zone1377", 1377,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2691", zones.back()->m_zone, "zone1383", 1383,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2692", zones.back()->m_zone, "zone1389", 1389,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2693", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2069", zones.back()->m_zone, "zone1149", 1149,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2217", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2687", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2693", zones.back()->m_zone, "zone1381", 1381,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2694", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2695", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2204", zones.back()->m_zone, "zone1193", 1193,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2590", zones.back()->m_zone, "zone1344", 1344,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2691", zones.back()->m_zone, "zone1381", 1381,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2696", zones.back()->m_zone, "zone1484", 1484,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2697", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2064", zones.back()->m_zone, "zone1148", 1148,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2209", zones.back()->m_zone, "zone1194", 1194,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2594", zones.back()->m_zone, "zone1345", 1345,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2694", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2697", zones.back()->m_zone, "zone1383", 1383,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2698", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2214", zones.back()->m_zone, "zone1195", 1195,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2221", zones.back()->m_zone, "zone1197", 1197,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2699", zones.back()->m_zone, "zone1391", 1391,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2700", zones.back()->m_zone, "zone1389", 1389,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2701", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B34", zones.back()->m_zone, "zone1164", 1164,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A41", zones.back()->m_zone, "zone113", 113, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A42", zones.back()->m_zone, "zone694", 694, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A43", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A44", zones.back()->m_zone, "zone2", 2, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A45", zones.back()->m_zone, "zone696", 696, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B3", zones.back()->m_zone, "zone1", 1, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B367", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A369", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A370", zones.back()->m_zone, "zone251", 251,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A371", zones.back()->m_zone, "zone722", 722, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A372", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1864", zones.back()->m_zone, "zone1089", 1089,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2218", zones.back()->m_zone, "zone1196", 1196,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2228", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2701", zones.back()->m_zone, "zone1385", 1385,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2702", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2703", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2688", zones.back()->m_zone, "zone1379", 1379,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2692", zones.back()->m_zone, "zone1381", 1381,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2700", zones.back()->m_zone, "zone1385", 1385,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2704", zones.back()->m_zone, "zone1393", 1393,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2705", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1877", zones.back()->m_zone, "zone1092", 1092,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2690", zones.back()->m_zone, "zone1380", 1380,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2695", zones.back()->m_zone, "zone1382", 1382,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2703", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2705", zones.back()->m_zone, "zone1389", 1389,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2706", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2629", zones.back()->m_zone, "zone1359", 1359,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2699", zones.back()->m_zone, "zone1385", 1385,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2707", zones.back()->m_zone, "zone1393", 1393,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2708", zones.back()->m_zone, "zone1397", 1397,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2709", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1901", zones.back()->m_zone, "zone1097", 1097,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2635", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2702", zones.back()->m_zone, "zone1388", 1388,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2709", zones.back()->m_zone, "zone1391", 1391,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2710", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2711", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2625", zones.back()->m_zone, "zone1357", 1357,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2650", zones.back()->m_zone, "zone1366", 1366,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2704", zones.back()->m_zone, "zone1389", 1389,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2707", zones.back()->m_zone, "zone1391", 1391,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2712", zones.back()->m_zone, "zone1394", 1394,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1907", zones.back()->m_zone, "zone1099", 1099,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2628", zones.back()->m_zone, "zone1358", 1358,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2653", zones.back()->m_zone, "zone1367", 1367,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2706", zones.back()->m_zone, "zone1390", 1390,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2710", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2712", zones.back()->m_zone, "zone1393", 1393,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2630", zones.back()->m_zone, "zone1359", 1359,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2636", zones.back()->m_zone, "zone1361", 1361,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2713", zones.back()->m_zone, "zone1400", 1400,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2714", zones.back()->m_zone, "zone1397", 1397,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2715", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1934", zones.back()->m_zone, "zone1106", 1106,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2634", zones.back()->m_zone, "zone1360", 1360,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2641", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2715", zones.back()->m_zone, "zone1395", 1395,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2716", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2717", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2220", zones.back()->m_zone, "zone1197", 1197,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2708", zones.back()->m_zone, "zone1391", 1391,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2714", zones.back()->m_zone, "zone1395", 1395,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2718", zones.back()->m_zone, "zone1402", 1402,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2719", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B223", zones.back()->m_zone, "zone943", 943,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B372", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A373", zones.back()->m_zone, "zone724", 724, Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{8, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A374", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A375", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A376", zones.back()->m_zone, "zone262", 262, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1899", zones.back()->m_zone, "zone1096", 1096,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{5, 31, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2225", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2711", zones.back()->m_zone, "zone1392", 1392,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2717", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2719", zones.back()->m_zone, "zone1397", 1397,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2720", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2637", zones.back()->m_zone, "zone1361", 1361,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2713", zones.back()->m_zone, "zone1395", 1395,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2721", zones.back()->m_zone, "zone1402", 1402,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2722", zones.back()->m_zone, "zone355", 355, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2723", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1939", zones.back()->m_zone, "zone1107", 1107,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{5, 11, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2644", zones.back()->m_zone, "zone1362", 1362,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2716", zones.back()->m_zone, "zone1396", 1396,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2723", zones.back()->m_zone, "zone1400", 1400,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2724", zones.back()->m_zone, "zone354", 354, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2725", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x100x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2718", zones.back()->m_zone, "zone1397", 1397,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2721", zones.back()->m_zone, "zone1400", 1400,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2726", zones.back()->m_zone, "zone1404", 1404,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2727", zones.back()->m_zone, "zone357", 357, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2728", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{31, 101, 8}}, Ioss::IJK_t{{31, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "30x5x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1892", zones.back()->m_zone, "zone1095", 1095,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 31, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2720", zones.back()->m_zone, "zone1399", 1399,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2725", zones.back()->m_zone, "zone1401", 1401,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2728", zones.back()->m_zone, "zone1402", 1402,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{31, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2729", zones.back()->m_zone, "zone360", 360, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2730", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2223", zones.back()->m_zone, "zone1197", 1197,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2726", zones.back()->m_zone, "zone1402", 1402,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2731", zones.back()->m_zone, "zone1406", 1406,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2732", zones.back()->m_zone, "zone368", 368,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2733", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B1874", zones.back()->m_zone, "zone1091", 1091,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{5, 8, 1}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2226", zones.back()->m_zone, "zone1199", 1199,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2730", zones.back()->m_zone, "zone1403", 1403,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{31, 1, 8}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2733", zones.back()->m_zone, "zone1404", 1404,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2734", zones.back()->m_zone, "zone372", 372,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2735", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B2230", zones.back()->m_zone, "zone1200", 1200,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2731", zones.back()->m_zone, "zone1404", 1404,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2736", zones.back()->m_zone, "zone1444", 1444,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2737", zones.back()->m_zone, "zone381", 381,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{31, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2738", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 31}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2108", zones.back()->m_zone, "zone1166", 1166,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2233", zones.back()->m_zone, "zone1201", 1201,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 31}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2735", zones.back()->m_zone, "zone1405", 1405,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 1, 31}},
                                                Ioss::IJK_t{{1, 6, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B2738", zones.back()->m_zone, "zone1406", 1406,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A2739", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2740", zones.back()->m_zone, "zone383", 383,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 31}}, Ioss::IJK_t{{31, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("A2741", zones.back()->m_zone, "zone1432", 1432,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2742", zones.back()->m_zone, "zone1410", 1410,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2743", zones.back()->m_zone, "zone1417", 1417,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2744", zones.back()->m_zone, "zone1418", 1418,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2745", zones.back()->m_zone, "zone50", 50,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B17", zones.back()->m_zone, "zone831", 831, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 6}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B27", zones.back()->m_zone, "zone1053", 1053,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B368", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A377", zones.back()->m_zone, "zone1475", 1475,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A378", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A379", zones.back()->m_zone, "zone946", 946, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2742", zones.back()->m_zone, "zone1408", 1408,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2746", zones.back()->m_zone, "zone1416", 1416,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2747", zones.back()->m_zone, "zone1413", 1413,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2748", zones.back()->m_zone, "zone1411", 1411,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2749", zones.back()->m_zone, "zone48", 48,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{1, 11, 12}}, Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2748", zones.back()->m_zone, "zone1410", 1410,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2750", zones.back()->m_zone, "zone1414", 1414,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2751", zones.back()->m_zone, "zone471", 471, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2752", zones.back()->m_zone, "zone1416", 1416,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2753", zones.back()->m_zone, "zone45", 45,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1822", zones.back()->m_zone, "zone1078", 1078,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2754", zones.back()->m_zone, "zone46", 46, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2755", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2756", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2757", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2758", zones.back()->m_zone, "zone1413", 1413,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2747", zones.back()->m_zone, "zone1410", 1410,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2758", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2759", zones.back()->m_zone, "zone1432", 1432,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2760", zones.back()->m_zone, "zone1436", 1436,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2761", zones.back()->m_zone, "zone1414", 1414,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2750", zones.back()->m_zone, "zone1411", 1411,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2761", zones.back()->m_zone, "zone1413", 1413,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2762", zones.back()->m_zone, "zone1457", 1457,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2763", zones.back()->m_zone, "zone476", 476, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2764", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1828", zones.back()->m_zone, "zone1079", 1079,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{6, 11, 8}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2755", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2764", zones.back()->m_zone, "zone1414", 1414,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2765", zones.back()->m_zone, "zone44", 44, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2766", zones.back()->m_zone, "zone478", 478, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2767", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2746", zones.back()->m_zone, "zone1410", 1410,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2752", zones.back()->m_zone, "zone1411", 1411,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2768", zones.back()->m_zone, "zone463", 463, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2769", zones.back()->m_zone, "zone1417", 1417,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2770", zones.back()->m_zone, "zone54", 54,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2743", zones.back()->m_zone, "zone1408", 1408,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2769", zones.back()->m_zone, "zone1416", 1416,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2771", zones.back()->m_zone, "zone457", 457,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2772", zones.back()->m_zone, "zone1418", 1418,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2773", zones.back()->m_zone, "zone53", 53,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2744", zones.back()->m_zone, "zone1408", 1408,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2772", zones.back()->m_zone, "zone1417", 1417,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2774", zones.back()->m_zone, "zone453", 453, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2775", zones.back()->m_zone, "zone1426", 1426,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2776", zones.back()->m_zone, "zone52", 52, Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 101, 1}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("A2777", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2778", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2779", zones.back()->m_zone, "zone153", 153,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2780", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2781", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2782", zones.back()->m_zone, "zone1421", 1421,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B8", zones.back()->m_zone, "zone609", 609, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B369", zones.back()->m_zone, "zone1387", 1387,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B378", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A380", zones.back()->m_zone, "zone957", 957,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A381", zones.back()->m_zone, "zone253", 253,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A382", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2782", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2783", zones.back()->m_zone, "zone1423", 1423,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2784", zones.back()->m_zone, "zone1438", 1438,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2785", zones.back()->m_zone, "zone1434", 1434,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2786", zones.back()->m_zone, "zone1424", 1424,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2777", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2787", zones.back()->m_zone, "zone425", 425,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2788", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2789", zones.back()->m_zone, "zone151", 151,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2790", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2791", zones.back()->m_zone, "zone1423", 1423,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2783", zones.back()->m_zone, "zone1421", 1421,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2791", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2792", zones.back()->m_zone, "zone422", 422,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2793", zones.back()->m_zone, "zone1443", 1443,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2794", zones.back()->m_zone, "zone1424", 1424,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2786", zones.back()->m_zone, "zone1421", 1421,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2794", zones.back()->m_zone, "zone1423", 1423,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2795", zones.back()->m_zone, "zone1428", 1428,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2796", zones.back()->m_zone, "zone429", 429, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2797", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2778", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2788", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2797", zones.back()->m_zone, "zone1424", 1424,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2798", zones.back()->m_zone, "zone152", 152,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{51, 6, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2799", zones.back()->m_zone, "zone430", 430, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2800", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2775", zones.back()->m_zone, "zone1418", 1418,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2801", zones.back()->m_zone, "zone1432", 1432,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2802", zones.back()->m_zone, "zone441", 441, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2803", zones.back()->m_zone, "zone1428", 1428,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2804", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2804", zones.back()->m_zone, "zone1426", 1426,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2805", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2806", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2807", zones.back()->m_zone, "zone146", 146,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{51, 11, 1}},
                                                Ioss::IJK_t{{51, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2808", zones.back()->m_zone, "zone444", 444, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2809", zones.back()->m_zone, "zone51", 51, Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{11, 1, 1}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2795", zones.back()->m_zone, "zone1424", 1424,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2803", zones.back()->m_zone, "zone1426", 1426,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2810", zones.back()->m_zone, "zone1434", 1434,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2811", zones.back()->m_zone, "zone434", 434, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2812", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2800", zones.back()->m_zone, "zone1425", 1425,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2806", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2812", zones.back()->m_zone, "zone1428", 1428,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2813", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2814", zones.back()->m_zone, "zone150", 150,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2815", zones.back()->m_zone, "zone438", 438, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2757", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2805", zones.back()->m_zone, "zone1427", 1427,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2816", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2817", zones.back()->m_zone, "zone148", 148,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2818", zones.back()->m_zone, "zone49", 49, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 12, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2819", zones.back()->m_zone, "zone1432", 1432,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B314", zones.back()->m_zone, "zone1187", 1187,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B375", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B382", zones.back()->m_zone, "zone1420", 1420,
                                                Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A383", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A384", zones.back()->m_zone, "zone958", 958, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A385", zones.back()->m_zone, "zone260", 260, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{1, 31, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2741", zones.back()->m_zone, "zone1408", 1408,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2759", zones.back()->m_zone, "zone1413", 1413,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2801", zones.back()->m_zone, "zone1426", 1426,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2819", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2820", zones.back()->m_zone, "zone1434", 1434,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2780", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2813", zones.back()->m_zone, "zone1429", 1429,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2816", zones.back()->m_zone, "zone1430", 1430,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2821", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2822", zones.back()->m_zone, "zone149", 149,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{51, 11, 12}},
                                                Ioss::IJK_t{{51, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2823", zones.back()->m_zone, "zone1434", 1434,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2785", zones.back()->m_zone, "zone1421", 1421,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2810", zones.back()->m_zone, "zone1428", 1428,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2820", zones.back()->m_zone, "zone1432", 1432,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2823", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2824", zones.back()->m_zone, "zone1436", 1436,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1775", zones.back()->m_zone, "zone1064", 1064,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2756", zones.back()->m_zone, "zone1412", 1412,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2821", zones.back()->m_zone, "zone1433", 1433,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2825", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2826", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2827", zones.back()->m_zone, "zone1436", 1436,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2760", zones.back()->m_zone, "zone1413", 1413,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2824", zones.back()->m_zone, "zone1434", 1434,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2827", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2828", zones.back()->m_zone, "zone1438", 1438,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2829", zones.back()->m_zone, "zone1455", 1455,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1755", zones.back()->m_zone, "zone1060", 1060,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}},
                                                Ioss::IJK_t{{12, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2781", zones.back()->m_zone, "zone1419", 1419,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2826", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2830", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2831", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2832", zones.back()->m_zone, "zone1438", 1438,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 6}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2784", zones.back()->m_zone, "zone1421", 1421,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2828", zones.back()->m_zone, "zone1436", 1436,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2832", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2833", zones.back()->m_zone, "zone1443", 1443,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2834", zones.back()->m_zone, "zone1448", 1448,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2835", zones.back()->m_zone, "zone1446", 1446,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2836", zones.back()->m_zone, "zone1448", 1448,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2837", zones.back()->m_zone, "zone1443", 1443,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2838", zones.back()->m_zone, "zone406", 406, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2839", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1738", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2839", zones.back()->m_zone, "zone1439", 1439,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2840", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2841", zones.back()->m_zone, "zone409", 409, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2842", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2843", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1743", zones.back()->m_zone, "zone1057", 1057,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 12}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2790", zones.back()->m_zone, "zone1422", 1422,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2831", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{12, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2842", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2844", zones.back()->m_zone, "zone415", 415,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2845", zones.back()->m_zone, "zone1443", 1443,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 12}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B33", zones.back()->m_zone, "zone1164", 1164, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B365", zones.back()->m_zone, "zone1375", 1375,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A386", zones.back()->m_zone, "zone1475", 1475,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A387", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A388", zones.back()->m_zone, "zone1486", 1486,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A389", zones.back()->m_zone, "zone836", 836, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2793", zones.back()->m_zone, "zone1423", 1423,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2833", zones.back()->m_zone, "zone1438", 1438,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{12, 1, 6}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2837", zones.back()->m_zone, "zone1439", 1439,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2845", zones.back()->m_zone, "zone1441", 1441,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2846", zones.back()->m_zone, "zone412", 412,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2239", zones.back()->m_zone, "zone1204", 1204,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2736", zones.back()->m_zone, "zone1406", 1406,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2847", zones.back()->m_zone, "zone1446", 1446,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2848", zones.back()->m_zone, "zone394", 394, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2849", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2104", zones.back()->m_zone, "zone1163", 1163,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2243", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2739", zones.back()->m_zone, "zone1407", 1407,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2849", zones.back()->m_zone, "zone1444", 1444,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2850", zones.back()->m_zone, "zone396", 396, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2851", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2835", zones.back()->m_zone, "zone1439", 1439,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2847", zones.back()->m_zone, "zone1444", 1444,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2852", zones.back()->m_zone, "zone1450", 1450,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2853", zones.back()->m_zone, "zone401", 401, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2854", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1734", zones.back()->m_zone, "zone1056", 1056,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2840", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2851", zones.back()->m_zone, "zone1445", 1445,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2854", zones.back()->m_zone, "zone1446", 1446,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2855", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2856", zones.back()->m_zone, "zone405", 405, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2834", zones.back()->m_zone, "zone1438", 1438,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2836", zones.back()->m_zone, "zone1439", 1439,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2857", zones.back()->m_zone, "zone1450", 1450,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2858", zones.back()->m_zone, "zone1455", 1455,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2859", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 5}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1751", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{5, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2830", zones.back()->m_zone, "zone1437", 1437,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2843", zones.back()->m_zone, "zone1440", 1440,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2859", zones.back()->m_zone, "zone1448", 1448,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2860", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2861", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2241", zones.back()->m_zone, "zone1204", 1204,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2852", zones.back()->m_zone, "zone1446", 1446,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2857", zones.back()->m_zone, "zone1448", 1448,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2862", zones.back()->m_zone, "zone1452", 1452,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2863", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1750", zones.back()->m_zone, "zone1059", 1059,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2244", zones.back()->m_zone, "zone1205", 1205,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2855", zones.back()->m_zone, "zone1447", 1447,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2860", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2863", zones.back()->m_zone, "zone1450", 1450,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2864", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2571", zones.back()->m_zone, "zone1333", 1333,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2862", zones.back()->m_zone, "zone1450", 1450,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2865", zones.back()->m_zone, "zone1459", 1459,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2866", zones.back()->m_zone, "zone1455", 1455,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2867", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 11}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x7x30"));

  zones.back()->m_zoneConnectivity.emplace_back("B218", zones.back()->m_zone, "zone931", 931,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{1, 8, 31}},
                                                Ioss::IJK_t{{5, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B374", zones.back()->m_zone, "zone1398", 1398,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 31}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B387", zones.back()->m_zone, "zone1442", 1442,
                                                Ioss::IJK_t{{1, 3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A390", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A391", zones.back()->m_zone, "zone728", 728, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A392", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 8, 1}},
      Ioss::IJK_t{{5, 8, 31}}, Ioss::IJK_t{{5, 8, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1768", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{1, -3, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 11, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2573", zones.back()->m_zone, "zone1334", 1334,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2864", zones.back()->m_zone, "zone1451", 1451,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2867", zones.back()->m_zone, "zone1452", 1452,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2868", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2869", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2829", zones.back()->m_zone, "zone1436", 1436,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2858", zones.back()->m_zone, "zone1448", 1448,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2866", zones.back()->m_zone, "zone1452", 1452,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{6, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2870", zones.back()->m_zone, "zone1457", 1457,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2871", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1769", zones.back()->m_zone, "zone1063", 1063,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 11, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2825", zones.back()->m_zone, "zone1435", 1435,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2861", zones.back()->m_zone, "zone1449", 1449,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2868", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2871", zones.back()->m_zone, "zone1455", 1455,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2872", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2762", zones.back()->m_zone, "zone1414", 1414,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2870", zones.back()->m_zone, "zone1455", 1455,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2873", zones.back()->m_zone, "zone1459", 1459,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2874", zones.back()->m_zone, "zone482", 482, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2875", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1766", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{3, 1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 5, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2767", zones.back()->m_zone, "zone1415", 1415,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2872", zones.back()->m_zone, "zone1456", 1456,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2875", zones.back()->m_zone, "zone1457", 1457,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2876", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2877", zones.back()->m_zone, "zone481", 481, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2248", zones.back()->m_zone, "zone1206", 1206,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2865", zones.back()->m_zone, "zone1452", 1452,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2873", zones.back()->m_zone, "zone1457", 1457,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2878", zones.back()->m_zone, "zone496", 496, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2879", zones.back()->m_zone, "zone1460", 1460,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1764", zones.back()->m_zone, "zone1062", 1062,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2251", zones.back()->m_zone, "zone1207", 1207,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2869", zones.back()->m_zone, "zone1454", 1454,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2876", zones.back()->m_zone, "zone1458", 1458,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2879", zones.back()->m_zone, "zone1459", 1459,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2880", zones.back()->m_zone, "zone493", 493, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2144", zones.back()->m_zone, "zone1175", 1175,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2167", zones.back()->m_zone, "zone1182", 1182,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2881", zones.back()->m_zone, "zone1481", 1481,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2882", zones.back()->m_zone, "zone1469", 1469,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2883", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 5}}, Ioss::IJK_t{{11, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1812", zones.back()->m_zone, "zone1074", 1074,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 1, 5}}, Ioss::IJK_t{{6, 11, 1}},
                                                Ioss::IJK_t{{6, 1, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2151", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}}, Ioss::IJK_t{{6, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2170", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2883", zones.back()->m_zone, "zone1461", 1461,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{11, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2884", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2885", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 6, 5}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1855", zones.back()->m_zone, "zone1086", 1086,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{6, 11, 12}},
                                                Ioss::IJK_t{{6, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2174", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2886", zones.back()->m_zone, "zone70", 70, Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 12, 1}}, Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2887", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2888", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2889", zones.back()->m_zone, "zone1465", 1465,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x7x30 "));

  zones.back()->m_zoneConnectivity.emplace_back("B310", zones.back()->m_zone, "zone1176", 1176,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 31}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{12, 1, 31}},
                                                Ioss::IJK_t{{1, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B383", zones.back()->m_zone, "zone1431", 1431,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("B390", zones.back()->m_zone, "zone1453", 1453,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 8, 31}}, Ioss::IJK_t{{1, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 31}});

  zones.back()->m_zoneConnectivity.emplace_back("A393", zones.back()->m_zone, "zone1475", 1475,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 8, 1}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A394", zones.back()->m_zone, "zone25", 25, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 1, 31}}, Ioss::IJK_t{{1, 8, 1}}, Ioss::IJK_t{{12, 8, 31}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A395", zones.back()->m_zone, "zone944", 944, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 8, 31}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 8, 31}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2182", zones.back()->m_zone, "zone1185", 1185, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2889", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2890", zones.back()->m_zone, "zone1474", 1474,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2891", zones.back()->m_zone, "zone1467", 1467,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2892", zones.back()->m_zone, "zone1481", 1481, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "10x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("A2893", zones.back()->m_zone, "zone1474", 1474,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2894", zones.back()->m_zone, "zone1471", 1471,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2895", zones.back()->m_zone, "zone539", 539, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2896", zones.back()->m_zone, "zone1467", 1467,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2897", zones.back()->m_zone, "zone71", 71,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{11, 101, 8}}, Ioss::IJK_t{{11, 1, 11}},
                                                Ioss::IJK_t{{11, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x10 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2891", zones.back()->m_zone, "zone1465", 1465,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2896", zones.back()->m_zone, "zone1466", 1466,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{11, 1, 8}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2898", zones.back()->m_zone, "zone526", 526, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2899", zones.back()->m_zone, "zone1469", 1469,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2900", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 11}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1857", zones.back()->m_zone, "zone1088", 1088,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2888", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2900", zones.back()->m_zone, "zone1467", 1467,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2901", zones.back()->m_zone, "zone68", 68, Ioss::IJK_t{{2, 3, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{11, 1, 1}}, Ioss::IJK_t{{11, 8, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2902", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2903", zones.back()->m_zone, "zone530", 530, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2882", zones.back()->m_zone, "zone1461", 1461,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2899", zones.back()->m_zone, "zone1467", 1467,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2904", zones.back()->m_zone, "zone1482", 1482,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2905", zones.back()->m_zone, "zone525", 525, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2906", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1817", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{-3, -1, 2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 5, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2884", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2902", zones.back()->m_zone, "zone1468", 1468,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2906", zones.back()->m_zone, "zone1469", 1469,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2907", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2908", zones.back()->m_zone, "zone521", 521, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2894", zones.back()->m_zone, "zone1466", 1466,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2909", zones.back()->m_zone, "zone544", 544, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2910", zones.back()->m_zone, "zone1474", 1474,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2911", zones.back()->m_zone, "zone1472", 1472,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2912", zones.back()->m_zone, "zone73", 73,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 8, 1}},
                                                Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2911", zones.back()->m_zone, "zone1471", 1471,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2913", zones.back()->m_zone, "zone552", 552,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2914", zones.back()->m_zone, "zone1473", 1473,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2915", zones.back()->m_zone, "zone8", 8, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{11, 1, 8}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2916", zones.back()->m_zone, "zone74", 74,
                                                Ioss::IJK_t{{-3, 1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 8, 12}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2181", zones.back()->m_zone, "zone1185", 1185,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2914", zones.back()->m_zone, "zone1472", 1472,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2917", zones.back()->m_zone, "zone8", 8, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2918", zones.back()->m_zone, "zone1474", 1474, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2919", zones.back()->m_zone, "zone76", 76,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2890", zones.back()->m_zone, "zone1465", 1465,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2893", zones.back()->m_zone, "zone1466", 1466,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2910", zones.back()->m_zone, "zone1471", 1471,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2918", zones.back()->m_zone, "zone1473", 1473, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2920", zones.back()->m_zone, "zone72", 72,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}},
                                                Ioss::IJK_t{{11, 11, 1}}, Ioss::IJK_t{{11, 1, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back(
      "B38", zones.back()->m_zone, "zone1275", 1275, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{12, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B377", zones.back()->m_zone, "zone1409", 1409,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B386", zones.back()->m_zone, "zone1442", 1442,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B393", zones.back()->m_zone, "zone1464", 1464,
                                                Ioss::IJK_t{{-1, -3, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 8, 1}},
                                                Ioss::IJK_t{{1, 1, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A396", zones.back()->m_zone, "zone945", 945, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{51, 1, 1}}, Ioss::IJK_t{{51, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A397", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2121", zones.back()->m_zone, "zone1170", 1170,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{11, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2173", zones.back()->m_zone, "zone1184", 1184,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2921", zones.back()->m_zone, "zone188", 188,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2922", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2923", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2924", zones.back()->m_zone, "zone1477", 1477,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2116", zones.back()->m_zone, "zone1169", 1169,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{11, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2179", zones.back()->m_zone, "zone1185", 1185,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2924", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("A2925", zones.back()->m_zone, "zone1481", 1481,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2926", zones.back()->m_zone, "zone7", 7, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
      Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{1, 101, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x11"));

  zones.back()->m_zoneConnectivity.emplace_back("B1684", zones.back()->m_zone, "zone1043", 1043,
                                                Ioss::IJK_t{{-2, 3, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 12}}, Ioss::IJK_t{{12, 6, 6}},
                                                Ioss::IJK_t{{1, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2171", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2927", zones.back()->m_zone, "zone6", 6, Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2928", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2929", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2930", zones.back()->m_zone, "zone1479", 1479,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 12}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x11 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2166", zones.back()->m_zone, "zone1182", 1182,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2930", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2931", zones.back()->m_zone, "zone7", 7, Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{12, 1, 6}}, Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2932", zones.back()->m_zone, "zone1481", 1481,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2933", zones.back()->m_zone, "zone1496", 1496,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 12}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B1814", zones.back()->m_zone, "zone1075", 1075,
                                                Ioss::IJK_t{{3, -1, -2}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 11}}, Ioss::IJK_t{{1, 11, 1}},
                                                Ioss::IJK_t{{1, 1, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2885", zones.back()->m_zone, "zone1462", 1462,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 11}}, Ioss::IJK_t{{11, 1, 5}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2887", zones.back()->m_zone, "zone1463", 1463,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2923", zones.back()->m_zone, "zone1476", 1476,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 1, 11}},
                                                Ioss::IJK_t{{12, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2929", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2934", zones.back()->m_zone, "zone1481", 1481,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 11}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x10"));

  zones.back()->m_zoneConnectivity.emplace_back("B2881", zones.back()->m_zone, "zone1461", 1461,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{11, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "B2892", zones.back()->m_zone, "zone1465", 1465, Ioss::IJK_t{{-1, 2, -3}},
      Ioss::IJK_t{{1, 1, 11}}, Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 1, 11}},
      Ioss::IJK_t{{1, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2925", zones.back()->m_zone, "zone1477", 1477,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 11}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2932", zones.back()->m_zone, "zone1479", 1479,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2934", zones.back()->m_zone, "zone1480", 1480,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 11}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 11}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2135", zones.back()->m_zone, "zone1173", 1173,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2145", zones.back()->m_zone, "zone1175", 1175,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 101, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2904", zones.back()->m_zone, "zone1469", 1469,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2935", zones.back()->m_zone, "zone516", 516, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2936", zones.back()->m_zone, "zone1483", 1483,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1818", zones.back()->m_zone, "zone1077", 1077,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2142", zones.back()->m_zone, "zone1174", 1174,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2150", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 11}},
                                                Ioss::IJK_t{{6, 6, 11}});

  zones.back()->m_zoneConnectivity.emplace_back("B2907", zones.back()->m_zone, "zone1470", 1470,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2936", zones.back()->m_zone, "zone1482", 1482,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2937", zones.back()->m_zone, "zone517", 517, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2199", zones.back()->m_zone, "zone1191", 1191,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2681", zones.back()->m_zone, "zone1377", 1377,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2696", zones.back()->m_zone, "zone1383", 1383,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2938", zones.back()->m_zone, "zone1487", 1487,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2939", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B2051", zones.back()->m_zone, "zone1144", 1144,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2201", zones.back()->m_zone, "zone1192", 1192,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2685", zones.back()->m_zone, "zone1378", 1378,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2698", zones.back()->m_zone, "zone1384", 1384,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2939", zones.back()->m_zone, "zone1484", 1484,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2940", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "4x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B43", zones.back()->m_zone, "zone1386", 1386,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{5, 1, 8}}, Ioss::IJK_t{{5, 1, 1}},
                                                Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B388", zones.back()->m_zone, "zone1442", 1442,
                                                Ioss::IJK_t{{1, -2, -3}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{1, 6, 8}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A398", zones.back()->m_zone, "zone36", 36, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{5, 6, 1}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A399", zones.back()->m_zone, "zone14", 14, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{12, 1, 1}}, Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A400", zones.back()->m_zone, "zone735", 735, Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{5, 1, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{6, 1, 1}}, Ioss::IJK_t{{1, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A401", zones.back()->m_zone, "zone3", 3, Ioss::IJK_t{{-1, 3, 2}}, Ioss::IJK_t{{1, 6, 1}},
      Ioss::IJK_t{{5, 6, 8}}, Ioss::IJK_t{{5, 1, 1}}, Ioss::IJK_t{{1, 8, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2157", zones.back()->m_zone, "zone1180", 1180,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2660", zones.back()->m_zone, "zone1370", 1370,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2938", zones.back()->m_zone, "zone1484", 1484,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2941", zones.back()->m_zone, "zone1489", 1489,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2942", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B2033", zones.back()->m_zone, "zone1137", 1137,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2161", zones.back()->m_zone, "zone1181", 1181,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2665", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2940", zones.back()->m_zone, "zone1485", 1485,
                                                Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2942", zones.back()->m_zone, "zone1487", 1487,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2943", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x5 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2143", zones.back()->m_zone, "zone1175", 1175,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2164", zones.back()->m_zone, "zone1182", 1182,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2941", zones.back()->m_zone, "zone1487", 1487,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2944", zones.back()->m_zone, "zone1491", 1491,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2945", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 6}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x5"));

  zones.back()->m_zoneConnectivity.emplace_back("B1678", zones.back()->m_zone, "zone1041", 1041,
                                                Ioss::IJK_t{{2, -1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2148", zones.back()->m_zone, "zone1177", 1177,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2169", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2943", zones.back()->m_zone, "zone1488", 1488,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{1, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("B2945", zones.back()->m_zone, "zone1489", 1489,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back("A2946", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 6}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2663", zones.back()->m_zone, "zone1370", 1370,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2944", zones.back()->m_zone, "zone1489", 1489,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 6}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2947", zones.back()->m_zone, "zone235", 235, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2948", zones.back()->m_zone, "zone1493", 1493,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2949", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{6, 101, 8}}, Ioss::IJK_t{{6, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "5x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1670", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{-2, 1, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 1, 8}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{5, 1, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2666", zones.back()->m_zone, "zone1371", 1371,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2946", zones.back()->m_zone, "zone1490", 1490,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 1}}, Ioss::IJK_t{{6, 1, 6}},
                                                Ioss::IJK_t{{6, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2949", zones.back()->m_zone, "zone1491", 1491,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{6, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2950", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2951", zones.back()->m_zone, "zone238", 238, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 8}},
      Ioss::IJK_t{{6, 6, 8}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{6, 6, 1}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x100x4 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2165", zones.back()->m_zone, "zone1182", 1182,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2948", zones.back()->m_zone, "zone1491", 1491,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 101, 1}}, Ioss::IJK_t{{6, 1, 8}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2952", zones.back()->m_zone, "zone234", 234, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 101, 5}}, Ioss::IJK_t{{8, 1, 1}}, Ioss::IJK_t{{8, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2953", zones.back()->m_zone, "zone1496", 1496,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2954", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{8, 101, 5}}, Ioss::IJK_t{{8, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "7x5x4"));

  zones.back()->m_zoneConnectivity.emplace_back("B1671", zones.back()->m_zone, "zone1039", 1039,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 1, 5}}, Ioss::IJK_t{{5, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2172", zones.back()->m_zone, "zone1183", 1183,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 5}}, Ioss::IJK_t{{6, 1, 1}},
                                                Ioss::IJK_t{{6, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2950", zones.back()->m_zone, "zone1492", 1492,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2954", zones.back()->m_zone, "zone1493", 1493,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{8, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("A2955", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2956", zones.back()->m_zone, "zone232", 232, Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{8, 1, 1}},
      Ioss::IJK_t{{8, 6, 5}}, Ioss::IJK_t{{1, 1, 1}}, Ioss::IJK_t{{1, 6, 5}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x5x7"));

  zones.back()->m_zoneConnectivity.emplace_back("B1676", zones.back()->m_zone, "zone1040", 1040,
                                                Ioss::IJK_t{{1, 2, 3}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("B2928", zones.back()->m_zone, "zone1478", 1478,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("B2955", zones.back()->m_zone, "zone1494", 1494,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{12, 1, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{1, 1, 5}},
                                                Ioss::IJK_t{{8, 6, 5}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2957", zones.back()->m_zone, "zone4", 4, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{1, 1, 1}},
      Ioss::IJK_t{{1, 6, 8}}, Ioss::IJK_t{{1, 1, 8}}, Ioss::IJK_t{{1, 6, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("A2958", zones.back()->m_zone, "zone229", 229,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{8, 1, 1}},
                                                Ioss::IJK_t{{8, 6, 12}});

  zones.back()->m_zoneConnectivity.emplace_back("A2959", zones.back()->m_zone, "zone1496", 1496,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 6, 1}},
                                                Ioss::IJK_t{{12, 6, 8}}, Ioss::IJK_t{{12, 101, 1}},
                                                Ioss::IJK_t{{1, 101, 8}});

  zones.push_back(new Iocgns::StructuredZoneData(zone++, "11x100x7 "));

  zones.back()->m_zoneConnectivity.emplace_back("B2933", zones.back()->m_zone, "zone1479", 1479,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{12, 101, 1}}, Ioss::IJK_t{{6, 1, 12}},
                                                Ioss::IJK_t{{6, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back("B2953", zones.back()->m_zone, "zone1493", 1493,
                                                Ioss::IJK_t{{3, 2, -1}}, Ioss::IJK_t{{1, 1, 1}},
                                                Ioss::IJK_t{{1, 101, 8}}, Ioss::IJK_t{{8, 1, 5}},
                                                Ioss::IJK_t{{1, 101, 5}});

  zones.back()->m_zoneConnectivity.emplace_back("B2959", zones.back()->m_zone, "zone1495", 1495,
                                                Ioss::IJK_t{{-1, -2, 3}}, Ioss::IJK_t{{1, 101, 1}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 6, 1}},
                                                Ioss::IJK_t{{1, 6, 8}});

  zones.back()->m_zoneConnectivity.emplace_back("A2960", zones.back()->m_zone, "zone228", 228,
                                                Ioss::IJK_t{{-3, 2, 1}}, Ioss::IJK_t{{1, 1, 8}},
                                                Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{1, 1, 12}},
                                                Ioss::IJK_t{{1, 101, 1}});

  zones.back()->m_zoneConnectivity.emplace_back(
      "A2961", zones.back()->m_zone, "zone5", 5, Ioss::IJK_t{{-1, 2, -3}}, Ioss::IJK_t{{12, 1, 1}},
      Ioss::IJK_t{{12, 101, 8}}, Ioss::IJK_t{{12, 1, 8}}, Ioss::IJK_t{{12, 101, 1}});

  double load_balance_tolerance = 1.05;

  for (size_t proc_count = 2048; proc_count > 1; proc_count /= 2) {
    std::string name = "Rocket_1496_ProcCount_" + std::to_string(proc_count);
    DOCTEST_SUBCASE(name.c_str()) { check_split_assign(zones, load_balance_tolerance, proc_count); }
  }
  cleanup(zones);
}
#endif
